
#include "atextbrowser.h"

#include <qclipboard.h>
#include <qapplication.h>
#include <qpopupmenu.h>

#include <private/qrichtext_p.h>

ATextBrowser::ATextBrowser(QWidget *parent, const char *name ) 
  : QTextBrowser(parent,name)
{
}

ATextBrowser::~ATextBrowser(){
}

void ATextBrowser::setSource(const QString&)
{
//  qDebug("ATextBrowser::setSource");
}

bool ATextBrowser::isOnLink(const QPoint & pos)
{
  QTextCursor c(*textCursor());
  placeCursor(pos, &c);
  QTextParagraph *p = c.paragraph();
  int index = c.index();
  
  if( // if the cursor is over on the link
      p && p->at(index) && p->at(index)->isAnchor() &&
      // if the link is good
      !p->at(index)->anchorHref().isEmpty() &&
      index < p->length() - 1 )
  {
    linkName = p->at(index)->anchorName();
    linkHref = p->at(index)->anchorHref();

    return true;
  }
  
  return false;
}
  
QString ATextBrowser::anchorName() const
{
  return linkName;
}

QString ATextBrowser::anchorHref() const
{
  return linkHref;
}

void ATextBrowser::contentsMousePressEvent(QMouseEvent *e)
{
  if( Qt::RightButton == e->button() &&
      selectedText().isEmpty() &&
      isOnLink(e->pos()) )
  {
    // popup menu for copying the href of the link
    QPopupMenu p(this, "ATextBrowser");
    
    if( !anchorName().isEmpty() )
      p.insertItem( tr("Copy"), 1);
    if( !anchorHref().isEmpty() )
      p.insertItem( tr("Copy the link address"), 2);
    
    int iRet = p.exec( e->globalPos() );
    
    switch( iRet )
    {
      default:
      case -1:
        break;
        
      case 1:
        QApplication::clipboard()->setText( anchorName(), QClipboard::Selection);
        QApplication::clipboard()->setText( anchorName(), QClipboard::Clipboard);
        break;
        
      case 2:
        QApplication::clipboard()->setText( anchorHref(), QClipboard::Selection);
        QApplication::clipboard()->setText( anchorHref(), QClipboard::Clipboard);
        break;
    }
  }
  else
  {
    QTextBrowser::contentsMousePressEvent(e);
  }
}
