/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  folderlistwidget.h
  -  Shows a list of VFS folders.
                             -------------------
    begin                : Sat Dec  8 14:20:03 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
    modified             : Ilie Ghiciuc <ilie@eco-spaces.com.ro> 9 May 2003
 ***************************************************************************/

#ifndef FOLDER_LIST_WIDGET_H
#define FOLDER_LIST_WIDGET_H

#include <qlistview.h>

#include <defines.h>

class VFSModule;

/**
 * Shows a list of VFS folders.
 */
class AETHERA_API FolderListWidget : public QListView
{
  Q_OBJECT

  public:

    /**
     * @param root - Root VFS path, e.g. "/Mail".
     */
    FolderListWidget
      (
        const QString   & root,
        QWidget         * parent  = 0,
        const char      * name    = 0
      );

    virtual ~FolderListWidget();

    /**
     * @return full VFS path for the given item.
     */
    QString vfsPath(QListViewItem *) const;

public slots:
    void slotFolderChanged(QListViewItem*);

signals:
    void fullPathText(const QString&);


  protected:

    /**
     * Clear, then read the folder list from VFS and create the items.
     */
    void fill();

    /**
     * Look for a QListViewItem matching path, and create one if
     * not found. Doesn't create anything for root of VFS, so expect 0.
     */
    QListViewItem * findOrCreateFolderItem(const QString & path);

    /**
     * Look for a QListViewItem which is a direct child of root,
     * and create one if not found.
     */
    QListViewItem * findOrCreateChild
      (
        QListViewItem * root,
        const QString & text
      );

  private:

    class Private;
    Private * d;
};

#endif
