/***************************************************************************
                          mail.cpp  -  description
                             -------------------
    begin                : Tue Oct 1 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mail.h"

#include <qstring.h>
#include <qpixmap.h>

#include <qiconloader.h>

Mail * Mail::instance_ = 0L;

Mail::Mail()
{
}

Mail::~Mail()
{
}

Mail * Mail::instance()
{
  return instance_ ? instance_ : instance_ = new Mail();
}

QString Mail::toString(Mail::Attachment attach)
{
  QString text;
  
  switch( attach )
  {
    case HasAttachment:
      text = "attachment";
      break;
    case NoAttachment:
    default:
      break;
  }
  
  return text;
}

QPixmap Mail::toPixmap(Mail::Attachment attach)
{
  QPixmap pixmap;
  
  switch( attach )
  {
    case HasAttachment:
      pixmap = QIconLoader::instance()->image("attachment");
      break;
    case NoAttachment:
    default:
      pixmap = QIconLoader::instance()->image("no-attachment");
      break;
  }
  
  return pixmap;
}

Mail::Attachment Mail::toAttachment(const QString & text)
{
  if( "attachment" == text )
    return Mail::HasAttachment;
  
  return Mail::NoAttachment;
}

QString Mail::toString(Mail::Priority priority)
{
  QString text;
  
  switch( priority )
  {
    case Mail::Lowest :
      text = "Lowest";
      break;
    case Mail::Low :
      text = "Low";
      break;
    case Mail::High :
      text = "High";
      break;
    case Mail::Highest :
      text = "Highest";
      break;
    case Mail::Normal :
    default:
      text = "Normal";
      break;
  }
  
  return text;
}

QPixmap Mail::toPixmap(Mail::Priority priority)
{
  QPixmap pixmap;
    
  switch( priority )
  {
    case Mail::Lowest :
      pixmap = QIconLoader::instance()->image("LowestPriority");
      break;
    case Mail::Low :
      pixmap = QIconLoader::instance()->image("LowPriority");
      break;
    case Mail::High :
      pixmap = QIconLoader::instance()->image("HighPriority");
      break;
    case Mail::Highest :
      pixmap = QIconLoader::instance()->image("HighestPriority");
      break;
    case Mail::Normal :
    default:
      pixmap = QIconLoader::instance()->image("NormalPriority");
      break;
  }
  
  return pixmap;
}

Mail::Priority Mail::toPriority(const QString & text)
{
  Mail::Priority priority;
  char c = text[0].lower().latin1();
  
  switch(c)
  {
    case 'l':
      if( "low" == text.lower() )
        priority = Mail::Low;
      else if( "lowest" == text.lower() )
        priority = Mail::Lowest;
      else
        priority = Mail::Normal;
      break;
      
    case 'h':
      if( "high" == text.lower() )
        priority = Mail::High;
      else if( "highest" == text.lower() )
        priority = Mail::Highest;
      else
        priority = Mail::Normal;
      break;
    
    default:
      priority = Mail::Normal;
      break;
  }
  
  return priority;
}

QString Mail::toString(Mail::Status status)
{
  QString text;
  
  switch( status )
  {
    case Mail::Forwarded :
      text = "Forwarded";
      break;
    case Mail::Replyed :
      text = "Replyed";
      break;
    case Mail::ReplyedForwarded :
      text = "ReplyedForwarded";
      break;
    case Mail::Default :
    default:
      text = "Default";
      break;
  }
  
  return text;
}

QPixmap Mail::toPixmap(Mail::Status status)
{
  QPixmap pixmap;
  
  switch( status )
  {
    case Mail::Forwarded :
      pixmap = QIconLoader::instance()->image("forward-mail");
      break;
    case Mail::Replyed :
      pixmap = QIconLoader::instance()->image("reply-mail");
      break;
    case Mail::ReplyedForwarded :
      pixmap = QIconLoader::instance()->image("reply-forward-mail");
      break;
    case Mail::Default :
    default:
      pixmap = QIconLoader::instance()->image("normal-mail");
      break;
  }
  
  return pixmap;
}

Mail::Status Mail::toStatus(const QString & text)
{
  Mail::Status status;
  char c = text[0].lower().latin1();
  
  switch(c)
  {
    case 'r':
      if( "replyed" == text.lower() )
        status = Mail::Replyed;
      else if( "replyedforwarded" == text.lower() )
        status = Mail::ReplyedForwarded;
      else
        status = Mail::Default;
      break;
      
    case 'f':
      if( "forwarded" == text.lower() )
        status = Mail::Forwarded;
      else
        status = Mail::Default;
      break;
    
    default:
      status = Mail::Default;
      break;
  }
  
  return status;
}

