/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  settingsdialog.cpp
  -  Container for settings widgets provided by plugins.
                             -------------------
    begin                : Tue Dec 18 15:06:07 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qcursor.h>
#include <qptrlist.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtabwidget.h>
#include <qapplication.h>
#include <qmap.h>
#include <qmessagebox.h>

#include <korecomponents.h>
#include <komaetheraplugin.h>
#include <settingsdialog.h>

class WidgetAndService
{
  public:
    WidgetAndService(QWidget* w=0L, KomConfigService* s=0L);
    ~WidgetAndService();
    
    bool operator==( const WidgetAndService& ) const;
    
    QWidget* widget;
    KomConfigService* service;
};

WidgetAndService::WidgetAndService(QWidget* w, KomConfigService* s)
{
  widget = w;
  service = s;
}

WidgetAndService::~WidgetAndService()
{
}

bool WidgetAndService::operator==( const WidgetAndService & right) const
{
  if( widget == right.widget &&
      service == right.service )
    return true;
  
  return false;
}

class SettingsDialog::Private
{
  public:
    QPtrList<WidgetAndService> serviceList;
};

SettingsDialog::SettingsDialog(QWidget * parent, const char * name)
  : QTabDialog(parent, name, true) /** let's try modal */
{
  d = new Private;
  d->serviceList.setAutoDelete(true);

  setOkButton();
  setApplyButton();
  setCancelButton();
  setDefaultButton();

  setCaption(QString("Settings - %1").arg(AppName));

  connect(this, SIGNAL(applyButtonPressed()),   SLOT(slotApply()));
  connect(this, SIGNAL(defaultButtonPressed()), SLOT(slotDefault()));
  connect(this, SIGNAL(aboutToShow()),          SLOT(slotAboutToShow()));

  loadServices();
}

SettingsDialog::~SettingsDialog()
{
  delete d;
}

  void
SettingsDialog::loadServices()
{
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

  // rikkus: Is the kore API meant to be a joke ?
  // It doesn't use the STL where it should use it, and where it does use it,
  // it uses the non-standard SGI extensions.

  ComponentRef *references=0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
    references = trader->registeredComponents( AETHERA_CONFIGSERVICE );
  else
  {
    qDebug("No component trader available!");
    return;
  }
  

  if ( !references[0].isValid() )
  {
    qDebug("No service providers registered for %s", AETHERA_CONFIGSERVICE);
  }

  uint i = 0;

  kore::ComponentRef provider;

  for (; (provider = references[i]).isValid(); i++)
  {
    KomConfigService * s = dynamic_cast<KomConfigService *>(provider.instance());

    if (0 == s)
    {
      qDebug("Can't dynamic_cast to KomConfigService");
      continue;
    }

    QWidget * w = s->init(this);

    if (0 == w)
    {
      qDebug("Service didn't give me a widget");
      continue;
    }

    // TODO: Figure out how to get keyboard accels. Can't really do it
    // from captions.

    addTab(w, s->caption());

    //put the widget and the service in the WidgetAndService list
    WidgetAndService* configTab = new WidgetAndService();
    configTab->widget = w;
    configTab->service = s;
    d->serviceList.append( configTab );

    // XXX: Really need to speed up dialog loading instead.
    qApp->processEvents();
  }

  delete [] references;

  QApplication::restoreOverrideCursor();
}

  void
SettingsDialog::slotAboutToShow()
{
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

  QPtrListIterator<WidgetAndService> it(d->serviceList);

  for (; it.current(); ++it)
  {
    it.current()->service->load();

    // XXX: Really need to speed up dialog loading instead.
    qApp->processEvents();
  }

  QApplication::restoreOverrideCursor();
}

  void
SettingsDialog::slotDefault()
{
  QPtrListIterator<WidgetAndService> serviceIterator(d->serviceList);

  QWidget* currentWidget = currentPage();
  
  for( ; serviceIterator.current(); ++serviceIterator )
  {
    if( currentWidget == serviceIterator.current()->widget )
    {
      serviceIterator.current()->service->defaults();
      break;
    }
  }
}

  void
SettingsDialog::slotApply()
{
  static bool warningDisplayed = false;

  if( !warningDisplayed )
  {
    QMessageBox::information( this,
                              tr( "Warning"),
                              tr( "Some settings will not be aplyed until after\n"
                                  "you restart Aethera"
                                )
                             );
    warningDisplayed = true;
  }
  
  QPtrListIterator<WidgetAndService> it(d->serviceList);

  for (; it.current(); ++it)
  {
    it.current()->service->save();
  }
}

