/***************************************************************************
                          icaodialog.h  -  description
                             -------------------
    begin                : Fri Jun 13 2003
    copyright            : (C) 2003 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#include "icaodialog.h"

#include <qlineedit.h>
#include <qlistview.h>
#include <qfile.h>

#include <dirs.h>

ICAODialog::ICAODialog(QWidget *parent, const char *name ) 
  : ICAODialogBase(parent,name)
{
  // remove the default info added by designer
  icaoListView->clear();
  icaoListView->removeColumn(0);
  icaoListView->setSelectionMode( QListView::NoSelection );
  
  // add the columns
  icaoListView->addColumn( tr("ICAO code"), 70 );
  icaoListView->addColumn( tr("Country"), 150 );
  icaoListView->addColumn( tr("Details"), 200 );
  
  icaoListView->setSorting(-1);
  init();
  icaoListView->setSorting(1, true);
  icaoListView->setShowSortIndicator(true);
  
  slotChangeSelection( icaoListView->firstChild() );
}

ICAODialog::~ICAODialog()
{
}

void ICAODialog::init()
{
  // fill the list view
  QFile locationsFile( Dirs::locate("data/nsd_cccc.txt") );
  
  if( locationsFile.open(IO_ReadOnly) )
  {
    QListViewItem * item = 0L;
    QString line;
    QTextStream stream( &locationsFile );
    int iFind1 = -1;
    int iFind2 = -1;
    
    while( !(line = stream.readLine()).isEmpty() )
    {
      QString code("NONE");
      QString country("UNKNOWN");
      QString details("NONE");
      QString temp;
      
      iFind1 = line.find(';');
      if(-1 != iFind1)
      {
        code = line.left(iFind1);
      }
      
      // pass over unused fields
      iFind1 = line.find(';', iFind1+1);
        
      // get the info
      iFind1 = line.find(';', iFind1+1);
      iFind2 = line.find(';', iFind1+1);
        
      if(-1 != iFind1 && -1 != iFind2)
      {
        temp = line.mid(iFind1+1, iFind2-iFind1-1);
        if( !temp.isEmpty() )
          details = temp;
          
        iFind1 = line.find(';', iFind2+1);
        iFind2 = line.find(';', iFind1+1);
        
        if(-1 != iFind1 && -1 != iFind2)
        {
          temp = line.mid(iFind1+1, iFind2-iFind1-1);
          if( !temp.isEmpty() )
          {
            country = temp;
          }
        }
      }
      
      // build the new entry
      item = new QListViewItem(icaoListView, code, country, details);
      if(!item)
      {
        qDebug("ICAODialog::init : cannot build a new list view item");
        break;
      }
    } // end of while
    
    locationsFile.close();
  }
  else
  {
    qDebug("ICAODialog::init : cannot open the data/nsd_cccc.txt file.");
  }
}

bool ICAODialog::isSelected()
{
  return (!selectedLineEdit->text().isEmpty());
}

QString ICAODialog::selected() const
{
  return selectedLineEdit->text();
}
    
