/***************************************************************************
                          weatherlib.h  -  description
                             -------------------
    begin                : Wed Jul 5 2000
    copyright            : (C) 2000 by Ian Reinhart Geiser
    email                : geiseri@msoe.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef WEATHERLIB_H
#define WEATHERLIB_H
#include <qobject.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qregexp.h>
#include <qdict.h>

#include <defines.h>

class QIODevice;

//
// Changed by Eugen C. <eug@thekompany.com> for Aethera
// 

class HOME_PLUGIN_API WeatherLib : public QObject
{
  Q_OBJECT
  
  public:
  
    enum WeatherType 
    {
      NoWeatherType,
      /** Sunny Default, Frozen Sunny, Springtime Sunny, Nice Sunny, Warm Sunny, Hot Sunny.*/
      Sunny=0, Sunny1, Sunny2, Sunny3, Sunny4, Sunny5,
      /** Cloud Default, One cloud, Few Clouds, Half Cloud and Half Sunny, Many Clouds, Many Dark Clouds.*/
      Cloudy, Cloudy1, Cloudy2, Cloudy3, Cloudy4, Cloudy5,
      /** Rain Default, Few drops, etc.*/
      Shower, Shower1, Shower2, Shower3, Shower4, Shower5,
      /** Snow Levels.*/
      Snow, Snow1, Snow2, Snow3, Snow4, Snow5
    };
    
    enum WeatherExtraType
    {
      NoWeatherExtraType,
      /** Storm types.*/
      Storm, ThunderStorm, Tornado, FunnelCloud, SandStorm, DustStorm,
      Fog, Mist, Freezing, Hail,
      /** Wind levels.*/
      Wind, Wind0, Wind1, Wind2, Wind3, Wind4, Wind5
    };

    WeatherLib(QObject *parent =0L, const char *name ="", bool bMetric=true);
    virtual ~WeatherLib();

    void setMetric(bool);
    
    void processData(QIODevice *);
    void processData(const QString&);
    
		QString temperature();
		QString dewPoint();
		QString relHumidity();
		QString heatIndex();
		QString windChill();
		QString wind();
		QString pressure();
		WeatherLib::WeatherType type();
		WeatherLib::WeatherExtraType extraType();
		QString date();
		QStringList weather();
		QString visibility();
		QStringList cover();
		
	private:
  
    class Data;
		
		bool parsePressure(QString tag, Data *d);
		bool parseVisibility(QString tag, Data *d);
		bool parseTime(QString tag, Data *d);
		bool parseTemperature(QString tag, Data *d);
		bool parseDate(QString tag, Data *d);
		bool parseCurrent(QString tag, Data *d);
		bool parseCover(QString tag, Data *d);
		bool parseWindSpeed(QString tag, Data *d);
		void calcWeatherType(Data *d);
		void calcWindChill(Data *d);

		void processData(QString metar, Data *d);

		QRegExp WindRegExp;
		QRegExp VisRegExp;
		QRegExp TempRegExp;
		QRegExp CoverRegExp;
		QRegExp CurrentRegExp;
		QRegExp TimeRegExp;
		QRegExp DateRegExp;
		QRegExp PressRegExp;

		bool MetricMode;

		Data * weatherData;
};

#endif
