/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <addressclass.h>
#include <addressbookview.h>
#include <ataddressbook.h>

#include <stdio.h>

#include <qiconloader.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include <qcstring.h>
#include <qstring.h>
#include <qheader.h>

#define IDSTRING "AddressBookView: "


//#define DEBUG_ABW


AddressBookViewInterface::AddressBookViewInterface( QWidget *parent, const char *name)
  : QListView(parent, name)
{
}

AddressBookView::AddressBookView( QWidget *parent, const char *name)
  : AddressBookViewInterface(parent, name)
{
  setSelectionMode(QListView::Extended);
  setShowSortIndicator(true);
  setRootIsDecorated(true);
  setAllColumnsShowFocus(true);
}

void AddressBookView::initMode(int mode, QString match, bool showMatched)
{
  AddressBook *instance = AddressBook::thisInstance();
  
  QStringList list;
  bool _show=true;
  
  // UI - showing the header..?
  if(mode & AddressBook::ShowHeader)
    header()->show();
  else
    header()->hide();
  // UI - as list? as tree?
  setRootIsDecorated(!(mode & AddressBook::ListOnly));
  if(mode & AddressBook::Groups)
  {
    addColumn("Names");
    addColumn("E-mail");
    setSorting(0);
  
    list=instance->completeNames("", AddressBook::Groups);
    for(uint i=0;i<list.count();i++)
    {
      if(match==QString::null)
      {
        _show=true;
      }
      else
      {
        if (showMatched)
          if(isMatched(instance->getEntry(list[i]),match))
            _show=true;
          else
            _show=false;
        else
          if(!isMatched(instance->getEntry(list[i]),match))
            _show=true;
          else
            _show=false;
      }
      if(_show)
      {
        QListViewItem *item=new QListViewItem(this);
        //QCheckListItem *item=new QCheckListItem(this, "test");
        item->setText(0,list[i]);
        item->setPixmap(0,BarIcon("Groups"));
        item->setText(2,instance->getEntry(list[i]).join(","));
        insertItem(item);
        QStringList members=instance->getEntry(list[i]);
        if(members.count()==0 || (mode & AddressBook::ListOnly))
          item->setExpandable(false);
        else
          item->setExpandable(true);
        for(uint i=0;i<members.count() && (!(mode & AddressBook::ListOnly));i++)
        {
          QListViewItem *subItem=new QListViewItem(item);
          QCString _tmp_=(const char *)members[i];
          AddressClass *ac=new AddressClass(_tmp_);
          if(ac->translated_name.isEmpty())
            delete subItem;
          else
          {
            subItem->setText(0,(const char *)ac->translated_name);
            subItem->setText(1,(const char *)ac->email);
            subItem->setText(2,(const char *)members[i]);
            subItem->setPixmap(0,BarIcon("Contacts"));
          }
          delete ac;
        }
      }
    }
  }
  if(mode & AddressBook::Contacts)
  {
    addColumn("Names");
    addColumn("E-mail");
    setSorting(0);
  
    // here is a list containing all the unique names...
//    QStringList uniq;
//    QString elem;
    list=instance->completeNames("", AddressBook::Contacts);
//    for(QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
//    {
//      elem = instance->getDefaultName((*it));
//      if( -1 == uniq.findIndex(elem) )
//        uniq.append(elem);
//    }
//    
//    list = uniq;
    
    list=instance->getEntries(list);
    
  {
    for(QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
    {
      QListViewItem *item=new QListViewItem(this);
      QStringList l=QStringList::split(',',(*it));
      
      if(l.count()<=1)
        item->setExpandable(false);
      else
        item->setExpandable(true);
      
      QCString cstr=(const char *)l[0];
      AddressClass *ac=new AddressClass(cstr);
      if(ac->translated_name.simplifyWhiteSpace().isEmpty())
        delete item;
      else
      {
        item->setText(0,(const char *)ac->translated_name);
        item->setText(1,(const char *)ac->email);
        item->setText(2,(const char *)l[0]);
        item->setPixmap(0,BarIcon("Contacts"));
      }
      delete ac;
      
      if(l.count()>1)
        for(uint j=1;j<l.count();j++)
        {
          QListViewItem *subItem=new QListViewItem(item);
          QCString _tmp_=(const char *)l[j];
          AddressClass *ac=new AddressClass(_tmp_);
          if(ac->translated_name.isEmpty())
            delete subItem;
          else
          {
            subItem->setText(0,(const char *)ac->translated_name);
            subItem->setText(1,(const char *)ac->email);
            subItem->setText(2,(const char *)l[j]);
            subItem->setPixmap(0,BarIcon("Contacts"));
          }
          delete ac;
        }
    }
  }
  }
  if(mode & AddressBook::History)
  {
    addColumn("Names");
    addColumn("E-mail");
    setSorting(0);
  
    list=instance->completeNames("", AddressBook::History);
#ifdef DEBUG_ABW
    printf("History: %d\n", list.count());
#endif
    for(uint i=0;i<list.count();i++)
    {
      QListViewItem *item=new QListViewItem(this);
      item->setText(0,list[i]);
      item->setPixmap(0,BarIcon("Contacts"));
      insertItem(item);
    }
  }
  if(mode & AddressBookViewInterface::SIP)
  {
    addColumn("Names");
    addColumn("SIP");
    setSorting(0);
  
//    list=instance->completeNames("", AddressBook::Contacts);
//    list=instance->getEntries(list);
//    for(QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
//    {
//      QListViewItem *item=new QListViewItem(this);
//    }
  }
}

void AddressBookView::getSelectedEmailList(QStringList &list)
{
  list.clear();
  QListViewItemIterator it(this);
  for ( ; it.current(); ++it )
  {
    if (it.current()->isSelected() && !it.current()->text(0).isEmpty())
    {
      list+=QStringList::split(',',it.current()->text(2));
    }
  }
}

bool AddressBookView::isMatched(QStringList l, QString check)
{
  AddressClass *checkAc = new AddressClass((const char *)check.lower());
  bool result=false;
  for(uint i=0;i<l.count();i++)
  {
    AddressClass *ac= new AddressClass((const char *)l[i]);
    result = 	ac->translated_name.lower()==checkAc->translated_name.lower() ||
              ac->email.lower()==checkAc->translated_name.lower();
    delete ac;
    if(result)
    {
      delete checkAc;
      return result;
    }
  }
  delete checkAc;
  return result;
}

/*
static const char* addressbookviewfactory_metainfo[][2]=
{
    { "/name", "Addressbook View Factory" },
    { "/class", "AddressBookViewFactory" },
    { "/class/fqn", "AddressBookViewFactory" },
    { "/factory", "addressbookviewfactory_factory" },
    { "/description", AETHERA_ADDRESSBOOK_VIEW_FACTORY_DESC },
    { "/service", AETHERA_ADDRESSBOOK_VIEW_FACTORY },
    { 0, 0 }
};
*/

kore::Interface* addressbookviewfactory_factory()
{
  return new AddressBookViewFactory();
}

AddressBookViewFactory::AddressBookViewFactory()
{
  // set the meta information
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Addressbook View Factory");
  info->replaceAll("class", "AddressBookViewFactory");
  info->replaceAll("class/fqn", "AddressBookViewFactory");
  info->replaceAll("factory", "addressbookviewfactory_factory");
  info->replaceAll("description", AETHERA_ADDRESSBOOK_VIEW_FACTORY_DESC);
  info->insert("service", AETHERA_ADDRESSBOOK_VIEW_FACTORY);
}

AddressBookViewFactory::~AddressBookViewFactory()
{
}

QWidget* AddressBookViewFactory::newWidget(QWidget *parent, const char *name, QWidget::WFlags)
{
  AddressBookView *newAddressBookView = new AddressBookView(parent, name);
  widgetsList.append(newAddressBookView);
  return newAddressBookView;
}

