/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    contactview.cpp
                             -------------------
    begin                : Tue Nov 27 17:14:32 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qcursor.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qregexp.h>

#include <qmessagebox.h>
#include <externalbrowser.h>
#include <vdragobject.h>
#include <folderchooserdialog.h>
#include <clipboard.h>
#include <qiconloader.h>
#include <contactsmainwidget.h>
#include <contactslistview.h>
#include <contactslistviewitem.h>
#include <clientvfs.h>
#include <vcontactobject.h>
#include <contactrichtextfactory.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <contacteditor.h>
#include <contactpreviewwidget.h>
#include <ataddressbook.h>
#include <appstatus.h>

/*
 * m_cutList concept:
 * When we cut items, we add their paths to m_cutList.
 * When we refresh, we avoid adding anything on m_cutList.
 * When we paste, we remove any pasted items from m_cutList.
 * When we start a new cut or copy, or we are deleted, we remove any
 * items in m_cutList from VFS.
 */

ContactsMainWidget::ContactsMainWidget
(
  VFSModule   * vfsModule,
  QWidget     * parent,
  const char  * name
)
  : QWidget(parent, name),
    m_currentPath ("/Aethera/VFS/Contacts"),
    m_vfsModule   (vfsModule)
{
  QSplitter * splitter = new QSplitter(Vertical, this);

  QVBoxLayout * layout = new QVBoxLayout(this);

  layout->addWidget(splitter);

  m_contactListView = new ContactsListView(splitter);
  m_contactView = new ContactPreviewWidget(splitter);

  connect
    (
      m_contactListView,
      SIGNAL(currentChanged(QListViewItem *)),
      SLOT(slotCurrentChanged(QListViewItem *))
    );
 
  connect
    (
      m_contactListView,
      SIGNAL(doubleClicked(QListViewItem *)),
      SLOT(slotDoubleClicked(QListViewItem *))
    );

  connect
    (
      m_contactListView,
      SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
      SLOT(slotContextMenuRequested(QListViewItem *, const QPoint &, int))
    );

  connect
    (
      m_contactListView,
      SIGNAL(selectionChanged()),
      SLOT(slotSelectionChanged())
    );



  connect
    (
      ContactEditor::instance(),



      SIGNAL(editFinished(VContactObject *, bool)),
      SLOT(slotEditFinished(VContactObject *, bool))
    );

  // Make the preview larger than the list.
  QValueList<int> sizeList;
  sizeList << height() / 3 << height() / 3 * 2;
  splitter->setSizes(sizeList);
}

ContactsMainWidget::~ContactsMainWidget()
{
  deleteItemsInCutList();
}

  void
ContactsMainWidget::slotSetFolder(QString path)
{
//  qDebug("ContactsMainWidget::slotSetFolder(%s)", path.ascii());

  if (path.isEmpty())
  {

    // Use same path.

    path = m_currentPath;
  }

  if (0 == path.contains(QRegExp("^/Aethera/VFS/Contacts")))
  {
//    qDebug("Not our folder. Give up.");
    return;
  }

  setUpdatesEnabled(false);

  m_currentPath = path;

//  qDebug("Fixed path is now %s", path.ascii());

  m_contactListView->clear();

  assert(0 != m_vfsModule);

  VFolder * folder = m_vfsModule->folderRef(path);

  if (0 == folder)
  {
    qDebug("Couldn't get ref to folder");
    setUpdatesEnabled(true);
    return;
  }

  QList<VObject> & objectList = folder->objectList();

//  qDebug("There are %d objects in this folder", objectList.count());

  AppStatus * status = AppStatus::instance();
  status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
    "Loading...", 0, objectList.count());

  int i=1;  
  for (QListIterator<VObject> it(objectList); it.current(); ++it)
  {
    VContactObject * contact = static_cast<VContactObject *>(it.current());

    if (0 == m_cutList.contains(contact->path()))
    {
      (void) new ContactsListViewItem(contact, m_contactListView);
    }
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
      "Loading...", i);
    i++;
  }

  setUpdatesEnabled(true);

  slotCurrentChanged(m_contactListView->currentItem());
  
  status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, "Folder is loaded");
}


  QStringList
ContactsMainWidget::getSelection()
{
  QStringList ret;
  return ret;
}

  VFolder *
ContactsMainWidget::parentFolder()
{
  QString path(m_currentPath);

  if (path.isEmpty())
  {
    path = "/Aethera/VFS/Contacts";
  }

  VFolder * folder = m_vfsModule->folderRef(path);

  Q_ASSERT(0 != folder);

  if (0 == folder)
    return 0;

  else
    return m_vfsModule->parentFolder(folder);
}

  void
ContactsMainWidget::newObject()
{
  if (m_currentPath.isEmpty())
  {
    qDebug("Hmm, I have no path");
    return;
  }

  QString contactName = m_vfsModule->createObject(m_currentPath);

  if (!contactName.isEmpty())
    m_toBeEditedWhenCreated = m_currentPath + "/" + contactName;
}

  void
ContactsMainWidget::selectAll()
{
  m_contactListView->selectAll(true);
}

  void
ContactsMainWidget::unselectAll()
{
  m_contactListView->clearSelection();
}

  bool
ContactsMainWidget::isSelectionEmpty()
{
  return true;
}


  bool

ContactsMainWidget::isSelectionSingle()
{
  switch (m_contactListView->selectionMode())
  {
    case QListView::Single:
      return true;
      break;

    default:
      return false;
      break;
  }
}

  bool
ContactsMainWidget::isSelectionMultiple()
{
  switch (m_contactListView->selectionMode())
  {
    case QListView::Multi:
    case QListView::Extended:
      return true;
      break;

    default:
      return false;
      break;

  }
}

  void
ContactsMainWidget::slotCurrentChanged(QListViewItem * item)

{
  if (0 == item)
    return;

//  qDebug("ContactsMainWidget::slotCurrentChanged(%s)", item->text(0).ascii());

  VContactObject * contact =
    static_cast<ContactsListViewItem *>(item)->contact();

  if (0 == contact)
  {
    qDebug("0 contact ?");
    return;
  }

  m_contactView->setContact(contact);
}

  void
ContactsMainWidget::slotDoubleClicked(QListViewItem * item)
{
  if (0 == item)
    return;


//  qDebug("ContactsMainWidget::slotDoubleClicked(%s)", item->text(0).ascii());

  QString path(m_currentPath);

  if (path.isEmpty())
  {
    qDebug("No current path");
  }

  VContactObject * contact =
    static_cast<ContactsListViewItem *>(item)->contact();

  QString contactPath(path + "/" + contact->name());

//  qDebug("Contact path: %s", contactPath.ascii());

  VContactObject * obj =
    static_cast<VContactObject *>( m_vfsModule->_fact_ref(contactPath, VFS_CONTACTS_MIMETYPE) );

  if (0 == obj)
  {
    qDebug("VFS couldn't find object");
    return;
  }

  // Looks like dynamic_cast won't work here.
  VContactObject * contactObject = static_cast<VContactObject *>(obj);

  if (0 == contactObject)
  {
    qDebug("contactObject is 0");
    return;
  }

  ContactEditor::instance()->edit(contactObject, this);
}

  void
ContactsMainWidget::slotContextMenuRequested(QListViewItem * i, const QPoint &, int /*col*/)
{
  enum { Create, Remove, Cut, Copy, Paste, Edit, MoveTo, CopyTo };

  int action = -1;

  {
    QPopupMenu pop(m_contactListView);

    if (0 == i)
    {
      pop.insertItem("&New contact", Create);
      pop.insertItem("&Paste",    Paste);

      if (!canPaste())
        pop.setItemEnabled(Paste, false);
    }
    else
    {
      pop.insertItem("&Edit", Edit);
      pop.insertSeparator();
      pop.insertItem("&Cut",  Cut);
      pop.insertItem("&Copy", Copy);
      pop.insertSeparator();
      pop.insertItem("&Remove", Remove);
      pop.insertSeparator();
      pop.insertItem("Copy to...", CopyTo);
      pop.insertItem("&Move to...", MoveTo);
    }

    action = pop.exec(QCursor::pos());
  }

  // Get selection.

  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  switch (action)
  {
    case Create:  newObject();            break;
    case Remove:  remove(selection);      break;
    case Edit:    slotDoubleClicked(i);   break;

    case Cut:     cut(selection);         break;
    case Copy:    copy(selection);        break;
    case Paste:   paste();                break;
    case MoveTo:  moveTo(selection);      break;
    case CopyTo:  copyTo(selection);      break;
    default:                              break;
  }
}

  bool
ContactsMainWidget::canPaste() const
{
  qDebug("ContactsMainWidget::canPaste()");

  QMimeSource * data = ClipboardWrapper::thisInstance()->data();


  if (0 == data)
  {
    qDebug("No data in clipboard.");
    return false;

  }

  if (!QUriDrag::canDecode(data))
  {
    qDebug("Not QUriDrag");
    return false;
  }

  QStringList pathList;

  if (!QUriDrag::decodeLocalFiles(data, pathList))
  {
    qDebug("QUriDrag lied when it said it could decode");
    return false;
  }

  if (pathList.isEmpty())
  {
    qDebug("No URIs");
    return false;
  }

  qDebug("First URI on clipboard: `%s'", pathList.first().ascii());

  if (0 == pathList.first().contains(QRegExp("^/Aethera/VFS/Contacts/")))
  {
    qDebug("Not contacts on the clipboard");
    return false;
  }

  return true;
}

  void
ContactsMainWidget::cut(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
  {
    emit(pastePossible(false));
    return;
  }

  deleteItemsInCutList();

  QStringList pathList = vObjectListToPathList(list);

  qDebug("ContactsMainWidget::cut: %d paths to cut", pathList.count());

  for (
    QStringList::ConstIterator it(pathList.begin());
    it != pathList.end();
    ++it
  )
  {

    m_cutList << *it;
  }

  // Don't do list.setAutoDelete(true), because we're will update before
  // the list would delete itself.

  QUriDrag * dragObject =
    new QUriDrag(m_contactListView, "Contact(s) drag object");

  dragObject->setFileNames(m_cutList);

  ClipboardWrapper::thisInstance()->cutData(dragObject);

  slotDeleteVInstances(m_cutList);

  emit(pastePossible(true));
}

  void
ContactsMainWidget::copy(const QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
  {
    emit(pastePossible(false));
    return;
  }

  deleteItemsInCutList();

  QStringList pathList;

  for (QPtrListIterator<ContactsListViewItem> it(list); it.current(); ++it)
  {
    VContactObject * contact = it.current()->contact();

    if (0 == contact)
    {
      qDebug("ContactsMainWidget::cut: Found a 0 contact");
      return;
    }

    pathList << contact->path();
  }

  QUriDrag * dragObject =
    new QUriDrag(m_contactListView, "Contact(s) drag object");

  dragObject->setFileNames(pathList);

  ClipboardWrapper::thisInstance()->copyData(dragObject);

  emit(pastePossible(true));
}

  void
ContactsMainWidget::paste()
{
  if (!canPaste())
    return;

  QMimeSource * mimeSource = ClipboardWrapper::thisInstance()->pasteData();

  if (0 == mimeSource)
  {
    qDebug("ClipboardWrapper gave me 0 to paste");
    return;
  }

  if (!QUriDrag::canDecode(mimeSource))
  {
    qDebug("Can't decode this mime source");
    return;
  }

  QStringList pathList;

  bool ok = QUriDrag::decodeLocalFiles(mimeSource, pathList);

  if (!ok)
  {
    qDebug("QUriDrag said it could decode, but it was lying.");
    return;
  }
  
  if( !pathList.count() )
    return;

  bool wasInCutList = (0 != m_cutList.remove(pathList[0]));

  if (wasInCutList)
  {
    // Cut 'n' paste.
    m_vfsModule->move(pathList, m_currentPath);
  }
  else
  {
    // Copy 'n' paste.
    m_vfsModule->copy(pathList, m_currentPath);
  }
}

  void
ContactsMainWidget::slotUpdateVInstances(const QStringList & l)
{
//  qDebug("ContactsMainWidget::slotUpdateVInstances");

  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
  {
//    qDebug("update wanted for: `%s'", (*it).ascii());
  }

  {
  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
  {
    ContactsListViewItem * i = static_cast<ContactsListViewItem *>(it.current());

    if (l.contains(i->contact()->path()))
    {
      AddressBook::thisInstance()->setChanged();
      i->update();
    }
  }
  }

  VContactObject * currentPreviewedContact = m_contactView->contact();

  if (0 != currentPreviewedContact)
  {
    if (l.contains(currentPreviewedContact->path()))
      m_contactView->refresh();
  }
}

  void
ContactsMainWidget::slotCreateVInstances(const QStringList & l)
{
  // TODO: Use a more efficient version.

  slotSetFolder("");

  if (m_toBeEditedWhenCreated.isEmpty())
    return;

  QString toEdit = m_toBeEditedWhenCreated;

  m_toBeEditedWhenCreated = QString::null;

  QStringList::ConstIterator it = l.begin();

  for (; it != l.end(); ++it)
  {
    if ((*it) == toEdit)
    {
      qDebug
        (
          "ContactsMainWidget::slotCreateVInstances: Editing `%s'",
          toEdit.ascii()
        );

      VContactObject * obj =
        static_cast<VContactObject *>( m_vfsModule->_fact_ref(toEdit, VFS_CONTACTS_MIMETYPE) );

      if (0 == obj)
      {
        qDebug("VFS couldn't find object");
        return;
      }

      // Looks like dynamic_cast won't work here.
      VContactObject * contactObject = static_cast<VContactObject *>(obj);

      if (0 == contactObject)
      {
        qDebug("contactObject is 0");
        return;
      }

      ContactEditor::instance()->edit
        (contactObject, this, ContactEditDialog::RemoveIfUnedited);
      
      AddressBook::thisInstance()->setChanged();
    }
  }
}

  void
ContactsMainWidget::slotDeleteVInstances(const QStringList & l)
{
  // TODO: Use a more efficient version.


  slotSetFolder("");
  return;

  QPtrList<ContactsListViewItem> deathRow;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
  {
    ContactsListViewItem * i = static_cast<ContactsListViewItem *>(it.current());

    if (l.contains(i->contact()->path()))
      deathRow.append(i);
  }

  {
  for (QPtrListIterator<ContactsListViewItem> it(deathRow); it.current(); ++it)
    delete it.current();
  }
  
  if(deathRow.count())
    AddressBook::thisInstance()->setChanged();
}

  void
ContactsMainWidget::deleteItemsInCutList()

{
  if( m_cutList.count() )
  {
    m_vfsModule->remove( m_cutList );
    
    m_cutList.clear();
  }
}

  void
ContactsMainWidget::cut()
{
  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  cut(selection);
}

  void
ContactsMainWidget::copy()
{
  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  copy(selection);
}

  void
ContactsMainWidget::slotSelectionChanged()
{
  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
  {
    if (it.current()->isSelected())
    {
      emit(copyPossible(true));
      return;
    }

  }

  emit(copyPossible(false));
}

  void
ContactsMainWidget::moveTo(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);

  if( pathList.count() )
  {
    QString targetPath = FolderChooserDialog::getFolder("/Contacts", this);
  
    if (!targetPath || targetPath == m_currentPath)
      return;
  
    m_vfsModule->move(pathList, targetPath);
  }
}

  void
ContactsMainWidget::copyTo(const QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);

  if( pathList.count() )
  {
    QString targetPath = FolderChooserDialog::getFolder("/Contacts", this);
  
    if (!targetPath || targetPath == m_currentPath)
      return;
  
    m_vfsModule->copy(pathList, targetPath);
  }
}

  QStringList
ContactsMainWidget::vObjectListToPathList
(
  const QPtrList<ContactsListViewItem> & list
) const
{
  if (list.isEmpty())
    return QStringList();

  QStringList pathList;

  for (QPtrListIterator<ContactsListViewItem> it(list); it.current(); ++it)
  {
    VContactObject * contact = it.current()->contact();

    if (0 == contact)
    {
      qDebug("ContactsMainWidget::cut: Found a 0 contact");
      continue;
    }

    pathList << contact->path();
  }


  return pathList;
}

  void
ContactsMainWidget::remove(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
  {
    return;
  }

  QStringList pathList = vObjectListToPathList(list);
  
  if( pathList.count() )
  {
    assert(0 != m_vfsModule);
    m_vfsModule->remove( pathList );
    
    AddressBook::thisInstance()->setChanged();
  }
}

  void
ContactsMainWidget::slotEditFinished(VContactObject * contact, bool del)
{
  qDebug("ContactsMainWidget::slotEditFinished");

  if (del)
  {
    m_vfsModule->remove(contact->path());
  }
}

