//own includes
#include <vfscontactlistview.h>

//komcontactsplugin includes
#include <vcontactobject.h>

//Aethera includes
#include<vfsmodule.h> //vfs service - required t build contacts list
#include<vfolder.h>

VFSContactListView::VFSContactListView(QWidget* parent, const char* name)
  :VFSObjectListView(parent,name)
{
  setSelectionMode(QListView::Single);
  setShowSortIndicator(true);
  setAllColumnsShowFocus(true);
  setSorting(0);

  buildHeader();

  connect(this,
          SIGNAL( selectionChanged(QListViewItem*) ),
          SLOT( slotObjectSelected(QListViewItem*) )
         );
}


VFSContactListView::~VFSContactListView()
{
  
}


void VFSContactListView::slotObjectSelected(QListViewItem* )
{
  QString tmp = getSelectedObject();

  emit objectSelected(tmp); 
}


void VFSContactListView::slotSetFolder(const QString& fld)
{
  contactFilesMap.clear();//clean the contacts file map
  clear(); //clear the contacts list first

  //fill the list with contact entries
  VFSModule *vfs=0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  //scanning for contacts
  if(vfs)
  {
    VFolder *folder=0L;

    folder = vfs->folderRef( fld );
    if(!folder)
    {
      qDebug("No such folder %s",(vfs->root()+fld).latin1());
      return;
    }
    if( "contacts" == folder->type() )
    {
      QPtrList<VObject> & entries = folder->objectList();
      for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
        if( folder->isSimpleObject( it1.current()->name() ) )
        {
          VContactObject * contact = dynamic_cast<VContactObject*>(it1.current());
          if(contact)
            addListObject(contact,folder->path()+"/"+contact->name());
        }
    }
  }
}


QString VFSContactListView::getSelectedObject()
{
  return contactFilesMap[ currentItem() ];
}


void VFSContactListView::addListObject(VContactObject* c, const QString& path)
{
  int headerIndex=0;

  VFSObjectListViewItem *itm = new VFSObjectListViewItem(this);
  contactFilesMap[itm]=path;

  QValueListIterator<QString> it;
  for(it=headerColumns.begin();it!=headerColumns.end();it++, headerIndex++)
  {
    if(*it=="Name") //if we have a name column put the name
      itm->setText(headerIndex,(c->n())[1]);
    if(*it=="First name")
      itm->setText(headerIndex,(c->n())[0]);
    if(*it=="Nick name")
      itm->setText(headerIndex,(c->fn()));
    if(*it=="Email")
      itm->setText(headerIndex,c->emails()[0]);
    if(*it=="Home telephone")
      itm->setText(headerIndex,(c->phone("home")));
    if(*it=="Work telephone")
      itm->setText(headerIndex,(c->phone("work")));
    if(*it=="Mobile telephone")
      itm->setText(headerIndex,(c->phone("mobile")));
    if(*it=="Fax number")
      itm->setText(headerIndex,(c->phone("fax")));
    if(*it=="Pager number")
      itm->setText(headerIndex,(c->phone("pager")));
    if(*it=="SIP")
      itm->setText(headerIndex,(c->phone("sip")));
    if(*it=="Home address")
      itm->setText(headerIndex,(c->homeAdr())[1]+(c->homeAdr())[2]);
    if(*it=="Residence city")
      itm->setText(headerIndex,(c->homeAdr())[3]);
    if(*it=="Residence state")
      itm->setText(headerIndex,(c->homeAdr())[4]);
    if(*it=="Residence country")
      itm->setText(headerIndex,(c->homeAdr())[5]);
    if(*it=="Personal WWW address")
      itm->setText(headerIndex,(c->homeAdr())[6]);
    if(*it=="Organization")
      itm->setText(headerIndex,(c->org()));
    if(*it=="Title")
      itm->setText(headerIndex,(c->title()));
    if(*it=="Department")
      itm->setText(headerIndex,(c->role()));
  }  
}


void VFSContactListView::buildHeader()
{

    headerColumns.append("Name");
    headerColumns.append("First name");
    headerColumns.append("Nick name");
    headerColumns.append("Email");
    headerColumns.append("Organization");
    headerColumns.append("Department");
    headerColumns.append("Work telephone");
    headerColumns.append("Mobile telephone");
    headerColumns.append("Fax number");
    headerColumns.append("Pager number");
    headerColumns.append("Home telephone");
    headerColumns.append("Home address");
    headerColumns.append("Residence city");
    headerColumns.append("Residence state");
    headerColumns.append("Residence country");
    headerColumns.append("Personal WWW address");

    QValueListIterator<QString> it;
    for(it=headerColumns.begin();it!=headerColumns.end();++it)
    {
            addColumn(*it);
    }
}  
