/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  accountmanagerwidget.cpp
  -  Account manager widget - derived from AccountManagerWidgetBase to handle
     the complexity.
                             -------------------
    begin                : Thu Oct  4 23:42:06 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qtl.h>

#include <qlistview.h>
#include <qheader.h>

#include <accounts.h> // Unfortunate naming. It actually contains class Account.
#include <accountmanagerwidget.h>
#include <accountmanagerlistitem.h>
#include <accountwizard.h>
#include <accounteditdialog.h>

AccountManagerWidget::AccountManagerWidget(QWidget * parent, const char * name)
  : AccountManagerWidgetBase(parent, name)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != clientRMI);
  
  // Button clicked connections are already made in designer.
  accountList->header()->hide();
}

AccountManagerWidget::~AccountManagerWidget()
{
}

  void
AccountManagerWidget::load()
{
  qDebug("Getting account info from manager");

  accountList->clear();

  // Make sure the manager is up-to-date.

  clientRMI->loadAccounts();

  QStringList accountNameList;

  accountNameList += clientRMI->getAccountNames(Account::MAILBOX);
  accountNameList += clientRMI->getAccountNames(Account::POP3);
  accountNameList += clientRMI->getAccountNames(Account::IMAP4);

  QString defaultAccount = clientRMI->getDefaultAccount();

  QStringList::ConstIterator it;

  for (it = accountNameList.begin(); it != accountNameList.end(); ++it)
  {
    AccountManagerListItem * item = new AccountManagerListItem
      (
        accountList,
        clientRMI->getAccount(*it)
      );

    item->setDefault((*it) == defaultAccount);
  }
}

  void
AccountManagerWidget::save()
{
  qDebug("\n Save the accounts settings.");
  
  // TODO : The account manager should save all the accounts
  // Just use Account::save() for each account
  clientRMI->saveAccounts();
}

  void
AccountManagerWidget::defaults()
{
  qDebug("AccountManagerWidget::defaults: STUB - logical ?");
}

  void
AccountManagerWidget::slotAddPOP3()
{
  AccountWizard wizard(Account::POP3, this, "Account wizard (POP3)");

  if (QDialog::Accepted != wizard.exec())
    return;

  Account account = wizard.account();

  qDebug("Adding account with type: %d", account.type);

  if( clientRMI->addAccount(account) )
  {
    AccountManagerListItem * i =
      new AccountManagerListItem(accountList, account);
  
    i->setEnabled(true);
  }
}

  void
AccountManagerWidget::slotAddIMAP4()
{
  AccountWizard wizard(Account::IMAP4, this, "Account wizard (IMAP4)");

  if (QDialog::Accepted != wizard.exec())
    return;

  Account account = wizard.account();

  qDebug("Adding account with type: %d", account.type);

  if( clientRMI->addAccount(account) )
  {
    AccountManagerListItem * i =
      new AccountManagerListItem(accountList, account);
  
    i->setEnabled(true);
  }
}

  void
AccountManagerWidget::slotEdit()
{
  QListViewItem * i = accountList->currentItem();

  if (0 == i)
    return;

  AccountManagerListItem * item = static_cast<AccountManagerListItem *>(i);

  QString accountName = item->account().accname;

  QStringList accountNameList;

  accountNameList += clientRMI->getAccountNames(Account::MAILBOX);
  accountNameList += clientRMI->getAccountNames(Account::POP3);
  accountNameList += clientRMI->getAccountNames(Account::IMAP4);

  if (!accountNameList.contains(accountName))
  {
    qDebug("BUG: Can't find account name in server's list");
    return;
  }

  Account acc = clientRMI->getAccount(accountName);

  AccountEditDialog dialog(acc, this, "Account edit dialog");

  if (QDialog::Accepted != dialog.exec())
    return;

  Account account = dialog.account();

  // TODO: sometime is enough to change the account attributes
  // Also IMAP needs something special
  // 
  // remove the edited account
  clientRMI->deleteAccount(accountName);
  // add the new one
  clientRMI->addAccount(account);

  item->setAccount(account);
}

  void
AccountManagerWidget::slotRemove()
{
  QListViewItem * currentItem = accountList->currentItem();

  if (0 == currentItem)
  {
    qDebug("AccountManagerWidget::slotRemove() : current item is null");
    return;
  }

  clientRMI->deleteAccount
    (static_cast<AccountManagerListItem *>(currentItem)->account().accname);

  delete currentItem;
  currentItem = 0;
}

  void
AccountManagerWidget::slotSetDefault()
{
  QListViewItem * currentItem = accountList->currentItem();

  if (0 == currentItem)
  {
    qDebug("Set 0 to default ? I don't think so.");
    return;
  }

  for (QListViewItemIterator it(accountList); it.current(); ++it)
  {
    AccountManagerListItem * item =
      static_cast<AccountManagerListItem *>(it.current());

    item->setDefault(false);
  }

  AccountManagerListItem * item =
    (static_cast<AccountManagerListItem *>(currentItem));

  if( item )
  {
    clientRMI->setDefaultAccount(item->account().accname);
  
    item->setDefault(true);
  }
}

  void
AccountManagerWidget::slotToggleEnable()
{
  QListViewItem * currentItem = accountList->currentItem();

  if (0 == currentItem)
  {
    qDebug("Toggle 0 QListViewItem en/disabled ? I don't think so.");
    return;
  }

  AccountManagerListItem * item =
    (dynamic_cast<AccountManagerListItem *>(currentItem));

  if( item )
  {
    clientRMI->toggleAccountActive(item->account().accname);
  
    item->toggleAccountEnabled();
  }
  else
    qDebug("AccountManagerWidget::slotToggleEnable() : item null");
}

