/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <defines.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qcstring.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qmime.h>

#include <composerattpane.h>
#include <attachmentpanewidget.h>
#include <partobj.h>
#include <mailobj.h>
#include <vmailattribute.h>
#include <vfsmimetype.h>

ComposerAttachmentPane::ComposerAttachmentPane(VFSModule *vfsModule, QWidget *composer, QWidget *parent, const char *name, WFlags f)
  : AttachmentPane(vfsModule, parent, name, f), _composer(composer)
{
  setSelectionMode(QIconView::Extended);

//  viewport()->setBackgroundColor(QColor("white"));
  makeGlobalPopup();
  makeChildPopup();
}

ComposerAttachmentPane::~ComposerAttachmentPane()
{
}

void ComposerAttachmentPane::makeGlobalPopup()
{
  mainPopup=new QPopupMenu();

  mainPopup->insertItem("Add attachment", this, SLOT(slotNewObject()));
  mainPopup->insertItem("Remove all", this, SLOT(slotRemoveAll()));
}

void ComposerAttachmentPane::makeChildPopup()
{
  childPopup=new QPopupMenu();

  childPopup->insertItem("Add attachment", this, SLOT(slotNewObject()));
  childPopup->insertItem("Remove attachment", this, SLOT(slotDeleteSelection()));
  childPopup->insertSeparator();
  childPopup->insertItem("Remove all", this, SLOT(slotRemoveAll()));
//	childPopup->insertSeparator();
//	childPopup->insertItem("Properties", this, SLOT(slotEditProperties()));
}

void ComposerAttachmentPane::newObject()
{
  if(_address.isEmpty() || !vfs->exists(_address))
  {
    QMessageBox::warning(_composer,
      QString("%1: Cannot add attachment").arg(AppName),
      "Attachments cannot be added to this object because\n"
      "it doesn't exist on the server.\n"
      "Please restart the application and try again.",
      QMessageBox::Ok, QMessageBox::NoButton);
  }
  else
  {
    QStringList files = QFileDialog::getOpenFileNames(QString::null, QString::null, _composer,
                          QString("%1: Attach Files").arg(AppName),
                          "Select one or more files for adding to the email attachment.");

    for(QStringList::ConstIterator it=files.begin(); it!=files.end(); ++it)
    {
      QString fileName(*it);
      if(!fileName.isEmpty())
      {
        if(QFile::exists(fileName) && QFileInfo(fileName).isReadable())
        {
          QString partPath=vfs->createObject(_address);
          if(partPath.isEmpty())
          {
            QMessageBox::warning(_composer,
              QString("%1: Cannot add attachment").arg(AppName),
              QString("Could not create attachment object.\n") +
              "The server reply was:\n" + vfs->lastError(),
              QMessageBox::Ok, QMessageBox::NoButton);
          }
          else
          {
            PartObject *item=(PartObject *)vfs->_fact_ref(partPath, VFS_PART_MIMETYPE );
            if(!item)
            {
              QMessageBox::warning(_composer,
                QString("%1: Cannot add attachment").arg(AppName),
                "Could not retrieve attachment object.\n"
                "Please restart the application.",
                QMessageBox::Ok, QMessageBox::NoButton);
            }
            else
            {
              QFile attachmentFile(fileName);

              if(attachmentFile.open(IO_ReadOnly))
              {
                int fileSize=attachmentFile.size();
                char *buf=new char[fileSize];
                attachmentFile.readBlock(buf, fileSize);

                QByteArray data;
                data.setRawData(buf, fileSize);

                item->setFilename(QFileInfo(fileName).fileName());
                item->setPartType("attachment");
                const QMimeSource * mime = QMimeSourceFactory::defaultFactory()->data(fileName);
                if( mime && mime->format() )
                item->setMimeType( mime->format() );
                item->setEncoding("base64");

                // debug
                // printf("composerattachmentpane: [new object] saving attachment properties...\n");

                bool savePartInfo=item->save();

                // debug
                // printf("composerattachmentpane: [new object] save properties result: %s\n", savePartInfo?"true":"false");

                // debug
                // printf("composerattachmentpane: [new object] saving attachment data...\n");

                bool setData=item->setData(data);

                // debug
                // printf("composerattachmentpane: [new object] save data result: %s\n", setData?"true":"false");

                if(!setData || !savePartInfo)
                {
                  QMessageBox::warning(_composer,
                    QString("%1: Cannot add attachment").arg(AppName),
                    QString("Could not save attachment object.\n") +
                    "Reason given by server: \n" + vfs->regetLastError(),
                    QMessageBox::Ok, QMessageBox::NoButton);
                  
                  delete item;
                  vfs->remove(partPath);
                }
                else
                {
                  MailObject *obj = (MailObject *)
                                    (vfs->_fact_ref(_address, VFS_MAILS_MIMETYPE));
                  if( obj )
                    obj->sync();

                  AttachmentPaneWidget *view = new AttachmentPaneWidget(item, this);
                  if( !view )
                  {
                    qDebug("AttachmentPaneWidget : not alocated");
                  }

                  show();
                  
                  slotStatusChanged();
                }
              }
              else
              {
                QMessageBox::warning(_composer,
                  QString("%1: Cannot add attachment").arg(AppName),
                  "Could not open file for reading.\n"
                  "Please check that it exists and try again.",
                  QMessageBox::Ok, QMessageBox::NoButton);
                
                delete item;
                vfs->remove(partPath);
              }
            }
          }
        }
        else
        {
          QMessageBox::warning(_composer,
            QString("%1: Cannot add attachment").arg(AppName),
            "Could not open file for reading.\n"
            "Please check the file permissions and try again.",
            QMessageBox::Ok, QMessageBox::NoButton);
        }
      }
    }
  }
}

void ComposerAttachmentPane::deleteSelection()
{
  QStringList selection = getSelection();
  sortPaths(selection);
  
  clear();
  QIconView::slotUpdate();

  QStringList::ConstIterator it = selection.fromLast();
  for(; it != selection.end(); --it)
    vfs->remove(*it);

  setAddress(_address);

  if( !count() )
    hide();
  
  slotStatusChanged();
}

void ComposerAttachmentPane::slotRemoveAll()
{
  QStringList deleteURL;
  for(SortingWidgetItem *item=(SortingWidgetItem*)this->firstItem(); item; item=(SortingWidgetItem*)item->nextItem())
  {
    deleteURL += item->path();
  }
  sortPaths(deleteURL);

  clear();
  QIconView::slotUpdate();
  
  QStringList::ConstIterator it = deleteURL.fromLast();
  for(; it != deleteURL.end(); --it)
    vfs->remove(*it);
  
  setAddress(_address);

  if( !count() )
    hide();
  
  slotStatusChanged();
}

void ComposerAttachmentPane::slotDeleteSelection()
{
  deleteSelection();
}

void ComposerAttachmentPane::slotNewObject()
{
  newObject();
}

void ComposerAttachmentPane::slotEditProperties()
{
  editProperties();
}

void ComposerAttachmentPane::sortPaths(QStringList & list)
{
  QStringList ordered;

  VMailAttribute *attr=(VMailAttribute *)vfs->_fact_ref(_address+".attr", VFS_MAILATTRIBUTE_MIMETYPE );

  if(attr && attr->sync())
  {
    for(int i=0;i<attr->partCount;i++)
    {
      for(QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
      {
        if( (*it) == (_address+"."+QString::number(i)) )
        {
          ordered += (*it);
        }
      }
    }

    list = ordered;
  }
  else
  {
    // sort alphabeticaly, it's better than nothing
    list.sort();
  }
}

