/***************************************************************************
                          composerfactory.cpp  -  description
                             -------------------
    begin                : Wed Sep 25 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "composerfactory.h"
#include "composerwindow.h"
#include <vfsmodule.h>

#include <qptrlist.h>

ComposerFactory * ComposerFactory::instance_ = 0L;

class ComposerFactory::Private
{
  public:
    Private();
    ~Private();

    QPtrList<QMainWindow> windows;
    ClientRMIModule * clientRMI;
    VFSModule * vfs;
};

ComposerFactory::Private::Private()
  : clientRMI(0L),
    vfs(0L)
{
  ComponentTrader * trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
    ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  assert(0L != clientRMI);
  assert(0L != vfs);

  windows.setAutoDelete(true);
}

ComposerFactory::Private::~Private()
{
  windows.clear();
}

ComposerFactory::ComposerFactory()
{
  d = new Private;
}

ComposerFactory::~ComposerFactory()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

ComposerFactory * ComposerFactory::instance()
{
  if(0 == instance_)
    instance_ = new ComposerFactory;

  return instance_;
}

void ComposerFactory::openComposer()
{
  ComposerWindow * cWin = new ComposerWindow(d->vfs, d->clientRMI, d->windows.count());
  
  d->windows.append( dynamic_cast<QMainWindow*>(cWin) );
  
  cWin->show();
}

void ComposerFactory::openComposer(const QString & vfsUrl, ComposerFactory::Mode mode)
{
  ComposerWindow * cWin = new ComposerWindow(d->vfs, d->clientRMI, d->windows.count(), vfsUrl, mode);

  d->windows.append( dynamic_cast<QMainWindow*>(cWin) );

  cWin->show();
}

void ComposerFactory::openComposer(const QStringList & vfsUrlList, ComposerFactory::Mode mode)
{
  for(QStringList::ConstIterator it=vfsUrlList.begin(); it!=vfsUrlList.end(); ++it)
  {
    openComposer((*it), mode);
  }
}

void ComposerFactory::slotCloseWindow(QMainWindow * window)
{
  d->windows.remove(window);
}

const QPtrList<QMainWindow> & ComposerFactory::windows()
{
  return d->windows;
}

void ComposerFactory::clear()
{
  d->windows.clear();
}

