/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * $Id: composerwidget.cpp,v 1.9 2003/08/04 21:24:12 eug Exp $
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "composerwidget.h"

#include <qlineedit.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qsettings.h>
#include <qtextedit.h>

#include <mail.h>
#include <mailobj.h>
#include <vmailattribute.h>
#include <partobj.h>
#include <vpartattribute.h>
#include <addresslistclass.h>
#include <elineedit.h>
#include <qsplitter.h>
#include <composerattpane.h>
#include <composerwindow.h>

#include <settings.h>
#include <texthtml.h>
#include <textprocessor.h>

ComposerWidget::ComposerWidget( VFSModule * vfsModule, ClientRMIModule * clientRmiModule,
                                ComposerWindow * parent, const char * name)
  : QWidget(parent, name),
    vfs( vfsModule ),
    clientRMI( clientRmiModule )
{
	layout=new QVBoxLayout(this, 0, -1, "ComposerWidget::layout");
	
	// edit fields
	toField = new AddressFieldEdit(this, "toField");
	ccField = new AddressFieldEdit(this, "ccField");
	bccField = new AddressFieldEdit(this, "bccField");
	subjectField = new EditField(this, "subjectField");
	
	toField->setLabel("To:");
	ccField->setLabel("Cc:");
	bccField->setLabel("Bcc:");
	subjectField->label()->setText("Subject:");
  subjectField->label()->setFixedSize(81, 25);
	
	// main area
	attachmentSplitter = new QSplitter(QSplitter::Vertical, this, "attachmentSplitter");
	editor_ = new QTextEdit(attachmentSplitter, "editor");
  
  attachmentPane = new ComposerAttachmentPane(vfsModule, parent, attachmentSplitter, "attachmentPane");
  
  QValueList<int> sizeList;
  sizeList << height() / 5 * 4 << height() / 5 ;
  attachmentSplitter->setSizes(sizeList);
  
	// add widgets in the layout
	layout->addWidget(toField);
	layout->addWidget(ccField);
	layout->addWidget(bccField);
	layout->addWidget(subjectField);
	layout->addWidget(attachmentSplitter);
	
	toField->show();
	ccField->show();
	
	// check if the bcc field is enabled
	if( Settings::instance()->readBoolEntry("/Aethera/Composer/Bcc", false))
		bccField->show();
	else
		bccField->hide();
	
	subjectField->show();
	editor_->show();
	attachmentPane->hide();
	attachmentSplitter->show();
	
  
//  // connect the edit fields to the signature update slot
//  connect(toField, SIGNAL(addressChanged()), this, SLOT(updateSignature()));
//  connect(ccField, SIGNAL(addressChanged()), this, SLOT(updateSignature()));
//  connect(bccField, SIGNAL(addressChanged()), this, SLOT(updateSignature()));
//  connect(subjectField, SIGNAL(textChanged(const QString &)), this, SLOT(updateSignature()));
}

ComposerWidget::~ComposerWidget()
{
}

AddressListClass *ComposerWidget::toAddresses()
{
	return toField->addressList();
}

AddressListClass *ComposerWidget::ccAddresses()
{
	return ccField->addressList();
}

AddressListClass *ComposerWidget::bccAddresses()
{
	return bccField->addressList();
}

void ComposerWidget::setToAddresses(AddressListClass addr)
{
	toField->setAddressList(addr);
}

void ComposerWidget::setCcAddresses(AddressListClass addr)
{
	ccField->setAddressList(addr);
}

void ComposerWidget::setBccAddresses(AddressListClass addr)
{
	bccField->setAddressList(addr);
}

bool ComposerWidget::isToFieldEmpty()
{
  return toField->isEmpty();
}

void ComposerWidget::loadMessage(MailObject * obj)
{
  if(obj)
  {
    QString text;
    
    if(obj->hasText())
    {
      text = obj->text();
    }
    else if(obj->hasHTML())
    {
      text = obj->html();
//  textHtmlConverter->html2text(text);
    }
   
    setText(text);

    setToAddresses(obj->to());
    setCcAddresses(obj->cc());
    setBccAddresses(obj->bcc());
    
    setSubject(obj->subject());

    setURL( obj->url() );
    
    if(obj->attachments())
      attachmentPane->show();
  }
}

void ComposerWidget::loadReply( MailObject * obj,
                                const QCString & myEmail, const QCString & myReplyTo,
                                bool replyAll)
{
  if(obj)
  {
    // load original text
//    QString text;
//
//    if(obj->hasText())
//    {
//      text=obj->text();
//    }
//    else if(obj->hasHTML())
//    {
//      text=obj->html();
////    textHtmlConverter->html2text(text);
//    }
//
//    // quote text
//    QString quoteSeq =
//      Settings::instance()->readEntry("/Aethera/Composer/Quote char");
//
//    if( quoteSeq.isEmpty() )
//      quoteSeq = "> ";
//
//    text = TextProcessor::quoteText(text, quoteSeq);
//    setText(text);




          QString format = Settings::instance()->readEntry( "/Aethera/Composer/Format", "plain" );

          QString data = QString::null;

          if( format=="plain" )
          {
            if( obj->hasHTML() )
            {
              QString text = obj->html();

              QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
                if(quoteSeq.isEmpty()) quoteSeq="> ";

              data=TextProcessor::quoteText(text, quoteSeq);

              setText( data );
            }
            else
            {
              QString text = obj->text();

              QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
                if(quoteSeq.isEmpty()) quoteSeq="> ";

              data=TextProcessor::quoteText(text, quoteSeq);

              setText( data );
            }
          }
          else if( format=="HTML" )
          {
            if( obj->hasHTML() )
            {
              QString text = obj->html();

              QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
                if(quoteSeq.isEmpty()) quoteSeq="> ";

              data = text;

              setText( data );
              
              int lines=0;
              for( int i=0; i<editor_->paragraphs(); i++)
              {
                for( int j=0; j<editor_->linesOfParagraph(i); j++, lines++ )
                  editor_->insertAt( quoteSeq, lines, false );
              }
            }
            else
            {
              QString text = obj->text();

              QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
                if(quoteSeq.isEmpty()) quoteSeq="> ";

              data=TextProcessor::quoteText(text, quoteSeq);

              setText( data );
            }
          }


    // set subject
    QString replyPrefix =
      Settings::instance()->readEntry("/Aethera/Composer/Reply prefix");
      
    if( !replyPrefix.contains("%1") )
      replyPrefix="Re: %1";
    
    setSubject( replyPrefix.arg(obj->subject()) );
    
    if(replyAll)
    {
      // get address pool
      AddressListClass origTo(obj->to());
      AddressClass origFrom(obj->from());;
      AddressListClass origCc(obj->cc());
      
      // map from + to => to, cc => cc except for our address
      QCString destTo=origFrom.toQCString() + ", ";
    
      for(int i=0;i<origTo.count();i++)
      {
        QCString toAdr=origTo[i].toQCString();
        if(toAdr!=myEmail && toAdr!=myReplyTo)
        {
          destTo.append(toAdr);
          destTo.append(", ");
        }
      }
      
      // remove trailing ", "
      if( static_cast<unsigned int>(destTo.findRev(", ")) == (destTo.length()-2) )
        destTo.truncate(destTo.length()-2);
      
      QCString destCc;

      {
        for (int i=0; i < origCc.count(); i++)
        {
          QCString ccAdr=origCc[i].toQCString();
          if (ccAdr!=myEmail && ccAdr!=myReplyTo)
          {
            destCc.append(ccAdr);
            destCc.append(", ");
          }
        }
      }
      
      // remove trailing ", "
      if (static_cast<unsigned int>(destCc.findRev(", ")) == (destCc.length()-2))
        destCc.truncate(destCc.length()-2);
      
      // load them in the To: and Cc: composer fields
      setToAddresses(AddressListClass(destTo));
      setCcAddresses(AddressListClass(destCc));
    }
    else
    {
      setToAddresses(AddressListClass(obj->from().toQCString()));
    }
  }
}

void ComposerWidget::setURL(const QString & vfsUrl)
{
  attachmentPane->setAddress( vfsUrl );
}

void ComposerWidget::usingRecipient(const QString & vfsUrl)
{
  toField->setAddressList(AddressListClass((const char *)vfsUrl));
}

QString ComposerWidget::text()
{
  return editor_->text();
}

void ComposerWidget::setText(const QString & txt)
{
  editor_->setText(txt);
}

QString ComposerWidget::subject()
{
  return subjectField->lineEdit()->text();
}

void ComposerWidget::setSubject(const QString & txt)
{
  subjectField->lineEdit()->setText(txt);
}

void ComposerWidget::slotAttachFile()
{
  attachmentPane->newObject();
}

QTextEdit * ComposerWidget::editor()
{
  return editor_;
}

EditField * ComposerWidget::subjectEdit()
{
  return subjectField;
}

void ComposerWidget::slotUseRichText()
{
  editor_->setTextFormat( Qt::RichText );
}

ComposerAttachmentPane * ComposerWidget::attachmentsWidget()
{
  return attachmentPane;
}
