/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
 komcomposeconfigservice.cpp
 -  The compose configuration service for the compose plug in
                             -------------------
    begin                : Thu Oct  4 20:51:43 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <komcomposeconfigservice.h>
#include <composeconfigwidget.h>
#include <qsettingsextra.h>
#include <qfontchooser.h>

#include <qsettings.h>
#include <qapplication.h>
#include <qmultilineedit.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

static const char * const configGroup			= "/Aethera/Composer";
static const char * const defQuote				= "> ";
static const bool 				defBcc					= true;
static const char * const defReplyLines		= 0;
static const char * const defReplyPrefix	= "";

#include <settings.h>

class KomComposeConfigService::Private
{
  public:

    ComposeConfigWidget * widget;
};

/*
static const char* komcomposeconfigservice_metainfo[][2]=
{
    { "/name", "Kom Compose Config Service" },
    { "/class", "KomComposeConfigService" },
    { "/class/fqn", "KomComposeConfigService" },
    { "/factory", "komcomposeconfigservice_factory" },
    { 0, 0 }
};
*/

kore::Interface* komcomposeconfigservice_factory()
{
  return new KomComposeConfigService();
}
    
KomComposeConfigService::KomComposeConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Compose Config Service");
  info->replaceAll("class", "KomComposeConfigService");
  info->replaceAll("class/fqn", "KomComposeConfigService");
  info->replaceAll("factory", "komcomposeconfigservice_factory");
  
  d = new Private;
  d->widget = 0;
}

KomComposeConfigService::~KomComposeConfigService()
{
  delete d->widget;
  d->widget = 0;
  delete d;
}

  QWidget *
KomComposeConfigService::init(QWidget * parent)
{
	d->widget =
		new ComposeConfigWidget(parent, "KomComposeConfigService top widget");

  return d->widget;
}

	void
KomComposeConfigService::load()
{
	if (0 == d->widget)
		return;

	QFont myFont = d->widget->font();

	QFont savedFont =
		QSettingsExtra::readFontEntry
		(Settings::instance(), configGroup + QString("Font"), &myFont);

	QString savedQuoteString =
		Settings::instance()->readEntry(configGroup + QString("QuoteChar"), defQuote);

	bool savedBcc =
		Settings::instance()->readBoolEntry(configGroup + QString("Bcc"), defBcc);

	QStringList entryList = Settings::instance()->entryList(configGroup);

	QString savedReplyPrefix;

	if (entryList.find("ReplyPrefix") != entryList.end())
	{
		savedReplyPrefix = Settings::instance()->readEntry("ReplyPrefix");
	}
	{
		// Read it old-style.
		int savedReplyPrefixLines =
			Settings::instance()->readNumEntry("Prefix reply lines");

		QString pattern("Prefix reply %1");

		for (int i = 0; i < savedReplyPrefixLines; i++)
		{
			if (i != 0)
				savedReplyPrefix.append("\n");

			savedReplyPrefix.append
				(Settings::instance()->readEntry(configGroup + pattern.arg(i)));
		}
	}

	d->widget->fontChooser->setSelectedFont(savedFont);

	d->widget->quoteTextLineEdit->setText(savedQuoteString);

	d->widget->showBccCheckBox->setChecked(savedBcc);

	d->widget->replyPrefixMultiLineEdit->setText(savedReplyPrefix);
}

	void
KomComposeConfigService::save()
{
	if (0 == d->widget)
		return;

	QSettingsExtra::writeEntry
		(
		 Settings::instance(),
		 configGroup + QString("Font"),
		 d->widget->fontChooser->selectedFont()
		);

	Settings::instance()->writeEntry
		(configGroup + QString("QuoteChar"), d->widget->quoteTextLineEdit->text());

	Settings::instance()->writeEntry
		(configGroup + QString("Bcc"), d->widget->showBccCheckBox->isChecked());

	Settings::instance()->writeEntry
		(
		 configGroup + QString("ReplyPrefix"),
		 d->widget->replyPrefixMultiLineEdit->text()
		);
}

	void
KomComposeConfigService::defaults()
{
	if (0 == d->widget)
		return;

	d->widget->fontChooser->setSelectedFont(d->widget->font());

	d->widget->quoteTextLineEdit->setText(defQuote);

	d->widget->showBccCheckBox->setChecked(defBcc);

	d->widget->replyPrefixMultiLineEdit->setText(defReplyPrefix);
}

	QPixmap
KomComposeConfigService::icon()
{
	QPixmap p(32, 32);
	p.fill(Qt::blue);
	return p;
}

	QString
KomComposeConfigService::caption()
{
	return qApp->translate("Compose configuration caption", "Composing");
}

	bool
KomComposeConfigService::modified()
{
	return false;
}

// vim:ts=2:sw=2:tw=78:noet


