/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                  kommailtreeitem.cpp  -  Mail tree item
                             -------------------
    begin                : Thu Mar 7 2002
    copyright            : (C) 2002 by theKompany.com
    email                : eug@thekompany.com
 ***************************************************************************/


#include "kommailtreeitem.h"
#include <vfolder.h>
#include <vfolderattribute.h>
#include <vfsmodule.h>
#include <mailobj.h>

#include <qpainter.h>

KomMailTreeItem::KomMailTreeItem( QListView * parent, KomItemType& treeItemType )
  : KomTreeItem(parent, treeItemType),
    folder(0L),
    folderAttribute(0L),
    recent(0),
    bUpdate(true)
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  folder = vfs->folderRef( "/Aethera/VFS/Mail" );
  folderAttribute = static_cast<VFolderAttribute *>(vfs->_fact_ref( "/Aethera/VFS/Mail/default.fattr", "vfolderattribute"));
}

KomMailTreeItem::KomMailTreeItem( KomTreeItem* parent, KomItemType& treeItemType )
  : KomTreeItem(parent, treeItemType),
    folder(0L),
    folderAttribute(0L),
    bUpdate(true)
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString parentPath = static_cast<KomMailTreeItem*>(parent)->path();
  
//  qDebug("FolderPath = %s", (parentPath+"/"+text(0)).latin1());
  
  folder = vfs->folderRef( parentPath+"/"+text(0) );
  folderAttribute = static_cast<VFolderAttribute *>(vfs->_fact_ref( parentPath+"/"+text(0)+"/default.fattr", "vfolderattribute"));
}

KomMailTreeItem::~KomMailTreeItem()
{
}

void KomMailTreeItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align)
{
  QFont itemFont = p->font();
  if( hasUnreadMails() )
    itemFont.setBold(true);
  else
    itemFont.setBold(false);
  p->setFont(itemFont);
    
  KomTreeItem::paintCell(p, cg, column, width, align);
  
  itemFont.setBold(false);
  p->setFont(itemFont);
}

QString KomMailTreeItem::path()
{
  if( folder )
    return folder->path();
  else
    return QString::null;
}


bool KomMailTreeItem::hasUnreadMails()
{
  if( folder && 
      "genmail" != folder->viewType() )
  {
    if( bUpdate )
    {
      bUpdate = false;
      
      VFSModule * vfs = 0;
      ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
      if( trader )
      {
        ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
        if( ref.isValid() )
          vfs = static_cast<VFSModule*>(ref.instance());
      }
      if (0 == vfs)
      {
        DEBUG_MSG("No VFS module ?");
        return false;
      }
    
      folderAttribute = static_cast<VFolderAttribute *>(vfs->_fact_ref( folder->path() + "/default.fattr", "vfolderattribute"));
      if( folderAttribute && folderAttribute->sync() )
      {
        listView()->setUpdatesEnabled(false);
        setText(1, QString("%1").arg(folderAttribute->recent) );
        setText(2, QString("%1").arg(folderAttribute->items) );
        listView()->setUpdatesEnabled(true);
        
        recent = folderAttribute->recent;
        
        if( folderAttribute->recent )
          return true;
      }
    }
    else
    {
      if( recent )
        return true;
    }
  }
  
  // check it recursively
  // TODO
  
  return false;
}

void KomMailTreeItem::forceUpdate()
{
  bUpdate = true;
}

