/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
 komsendreceiveconfigservice.cpp
 -  The send/receive configuration service for the mail plug in
                             -------------------
    begin                : Thu Oct  4 20:51:31 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <komsendreceiveconfigservice.h>
#include <sendreceiveconfigwidget.h>

#include <qsettings.h>
#include <qapplication.h>

#include <qcheckbox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>

static const char * const configGroup = "/Misc Mail";

static const bool defSaveSent = true;
static const int defCheckInterval = 600;
static const int defMaxWasted = 32;
static const bool defCheckAll = true;
static const bool defAutomaticCheck = true;

#include <settings.h>

class KomSendReceiveConfigService::Private
{
  public:

    SendReceiveConfigWidget * widget;
};

/*
static const char* komsendreceiveconfigservice_metainfo[][2]=
{
    { "/name", "Kom SendReceive Config Service" },
    { "/class", "KomSendReceiveConfigService" },
    { "/class/fqn", "KomSendReceiveConfigService" },
    { "/factory", "komsendreceiveconfigservice_factory" },
    { 0, 0 }
};
*/


kore::Interface* komsendreceiveconfigservice_factory()
{
  return new KomSendReceiveConfigService();
}
    
KomSendReceiveConfigService::KomSendReceiveConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom SendReceive Config Service");
  info->replaceAll("class", "KomSendReceiveConfigService");
  info->replaceAll("class/fqn", "KomSendReceiveConfigService");
  info->replaceAll("factory", "komsendreceiveconfigservice_factory");
  
  d = new Private;
  d->widget = 0;
}

KomSendReceiveConfigService::~KomSendReceiveConfigService()
{
  delete d->widget;
  d->widget = 0;
  delete d;
}

  QWidget *
KomSendReceiveConfigService::init(QWidget * parent)
{
  d->widget =
    new SendReceiveConfigWidget
    (parent, "KomSendReceiveConfigService top widget");

  return d->widget;
}

  void
KomSendReceiveConfigService::load()
{
  if (0 == d->widget)
    return;

  bool saveSent =
    Settings::instance()->readBoolEntry
    (Settings::instance()->name() + "/Send Options" + QString("/Save sent"), defSaveSent);

  bool automaticCheck =
    Settings::instance()->readBoolEntry
    (Settings::instance()->name() + configGroup + QString("/AutomaticCheck"), defAutomaticCheck);
  

  int checkInterval =
    Settings::instance()->readNumEntry
    (Settings::instance()->name() + configGroup + QString("/Interval"), defCheckInterval);

  int maxWasted =
    Settings::instance()->readNumEntry
    (Settings::instance()->name() + configGroup + QString("/Waste"), defMaxWasted);

  bool checkAll =
    Settings::instance()->readEntry
    (Settings::instance()->name() + configGroup + QString("/AutoCheckAccounts"), (defCheckAll ? "all" : "default")) == "all";

  d->widget->saveCopyCheckBox->setChecked(saveSent);

  if ( automaticCheck )
  {
    d->widget->checkNewCheckBox->setChecked(true);
    d->widget->checkIntervalSpinBox->setEnabled(true);
    d->widget->accountOptionGroup->setEnabled(true);
  }
  else
  {
    d->widget->checkNewCheckBox->setChecked(false);
    d->widget->checkIntervalSpinBox->setEnabled(false);
    d->widget->accountOptionGroup->setEnabled(false);  
  }

  if (checkAll)
    d->widget->checkAllRadioButton->setChecked(true);
  else
    d->widget->checkDefaultOnlyRadioButton->setChecked(true);

  d->widget->checkIntervalSpinBox->setValue( checkInterval );
             
  d->widget->maxWastedSpinBox->setValue(maxWasted);
}

  void
KomSendReceiveConfigService::save()
{
  if (0 == d->widget)
    return;

  Settings::instance()->writeEntry
    (
     Settings::instance()->name() + "/Send Options" + QString("/Save sent"),
     d->widget->saveCopyCheckBox->isChecked()
    );

  int checkInterval = d->widget->checkIntervalSpinBox->value();

  Settings::instance()->writeEntry
    (
     Settings::instance()->name() + configGroup + QString("/Interval"),
     checkInterval
    );

  Settings::instance()->writeEntry
    (
     Settings::instance()->name() + configGroup + QString("/Waste"),
     d->widget->maxWastedSpinBox->value()
    );

  Settings::instance()->writeEntry
    (
     Settings::instance()->name() + configGroup + QString("/AutoCheckAccounts"),
     (d->widget->checkAllRadioButton->isChecked() ? "all" : "default")
    );

  Settings::instance()->writeEntry
    (
     Settings::instance()->name() + configGroup + QString("/AutomaticCheck"),
     d->widget->checkNewCheckBox->isChecked()
    );
}


  void
KomSendReceiveConfigService::defaults()
{
  if (0 == d->widget)
    return;

  // Notice the "Permanent delete" value is saved backwards.
  d->widget->saveCopyCheckBox      ->setChecked(defSaveSent);
  d->widget->checkNewCheckBox      ->setChecked( defAutomaticCheck );
  d->widget->checkIntervalSpinBox  ->setValue(defCheckInterval);
  d->widget->maxWastedSpinBox      ->setValue(defMaxWasted);
  d->widget->checkAllRadioButton  ->setChecked(defCheckAll);
}

  QPixmap
KomSendReceiveConfigService::icon()
{
  QPixmap p(32, 32);
  p.fill(Qt::blue);
  return p;
}

  QString
KomSendReceiveConfigService::caption()
{
  return qApp->translate("SendReceive configuration caption", "Send/Receive");
}

  bool
KomSendReceiveConfigService::modified()
{
  return false;
}

// vim:ts=2:sw=2:tw=78:noet


