/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/** 
  * Changed by Eugen C. for using KURLLabel and all associated code.
  */ 

#include <qiconloader.h>

#include <qcstring.h>
#include <qstring.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qcursor.h>
#include <qaction.h>

#include <maillabel.h>
#include <conversions.h>
#include <mailobj.h>
#include <mailview.h>
#include <vfsmimetype.h>

// contacts
#include <contactsfactory.h>

MailLabel::MailLabel(VFSModule *vfsModule, MailView *parent, const char *name)
  : QWidget(parent, name)
{
  vfs = vfsModule;

  setFixedHeight(55);
  
  // KURLLabel
  FromLabel=new KURLLabel(this);
  SubjectLabel=new KURLLabel(this);
  ToLabel=new KURLLabel(this);
  CcLabel=new KURLLabel(this);
  
  // QLabel
  NothingLabel=new QLabel(this);
  FromText=new QLabel(this);
  SubjectText=new QLabel(this);
  ToText=new QLabel(this);
  CcText=new QLabel(this);
  
  // Label alignment
  FromLabel->setAlignment(AlignLeft);
  ToLabel->setAlignment(AlignLeft);
  CcLabel->setAlignment(AlignLeft);
  SubjectLabel->setAlignment(AlignLeft);
  NothingLabel->setAlignment(AlignCenter);

  // Popup menu
  fromMenu = new QPopupMenu(this, "FromMenu");
  subjectMenu = new QPopupMenu(this, "SubjectMenu");
  toMenu = new QPopupMenu(this, "ToMenu");
  ccMenu = new QPopupMenu(this, "CCMenu");
  
  // Actions for popup menu
  QAction *action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotFromCopy()));
  action->addTo( fromMenu );
  
  action = new QAction( QObject::tr("Add to addressbook."),
                        QObject::tr("&Add to addressbook."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotFromAdd()));
  action->addTo( fromMenu );
  
  action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotToCopy()));
  action->addTo( toMenu );
  
  action = new QAction( QObject::tr("Add to addressbook."),
                        QObject::tr("&Add to addressbook."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotToAdd()));
  action->addTo( toMenu );
  
  action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotCcCopy()));
  action->addTo( ccMenu );
  
  action = new QAction( QObject::tr("Add to addressbook."),
                        QObject::tr("&Add to addressbook."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotCcAdd()));
  action->addTo( ccMenu );
  
  action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotSubjectCopy()));
  action->addTo( subjectMenu );
  
  QFont f(font());
  f.setBold(true);
  fm = new QFontMetrics(f);

  FromText->setFont(f);
  ToText->setFont(f);
  CcText->setFont(f);
  SubjectText->setFont(f);

  FromText->setText("From:");
  ToText->setText("To:");
  SubjectText->setText("Subject:");
  CcText->setText("Cc:");
  NothingLabel->setText("No item selected");

  f.setBold(false);

  FromLabel->setFont(f);
  ToLabel->setFont(f);
  CcLabel->setFont(f);
  SubjectLabel->setFont(f);
  NothingLabel->setFont(f);

  FromText->setGeometry(5, 5, fm->width("From:"), fm->height());
  ToText->setGeometry(5, 25, fm->width("To:"), fm->height());
  SubjectText->setGeometry((width()-105)/2+5, 5, fm->width("Subject:"), fm->height());
  CcText->setGeometry((width()-105)/2+5, 25, fm->width("Cc:"), fm->height());
  NothingLabel->setGeometry(0, 0, width(), height());
  h_line=new QFrame(this);
  h_line->setFrameStyle(QFrame::HLine|QFrame::Sunken);
  h_line->setLineWidth(1);
  h_line->setGeometry(0, 50, parent->width(), 2);
  h_line->show();
  NothingLabel->show();

  attbutton=new EButton(this, "attachment button");
  attbutton->setNormalPixmap(BarIcon("attach-normal.xpm"));
  attbutton->setFocusPixmap(BarIcon("attach-focus.xpm"));
  attbutton->setClickedPixmap(BarIcon("attach-clicked.xpm"));
  attbutton->setGeometry(parent->width()-65, 0, 40, 40);

  address="";

  connect(attbutton, SIGNAL(clicked()), parent, SLOT(togglePane()));
  connect(FromLabel, SIGNAL(leftClickedURL()), this, SLOT(slotFromLeftClicked()));
  connect(FromLabel, SIGNAL(rightClickedURL()), this, SLOT(slotFromRightClicked()));
  connect(CcLabel, SIGNAL(leftClickedURL()), this, SLOT(slotCcLeftClicked()));
  connect(CcLabel, SIGNAL(rightClickedURL()), this, SLOT(slotCcRightClicked()));
  connect(ToLabel, SIGNAL(leftClickedURL()), this, SLOT(slotToLeftClicked()));
  connect(ToLabel, SIGNAL(rightClickedURL()), this, SLOT(slotToRightClicked()));
  connect(SubjectLabel, SIGNAL(leftClickedURL()), this, SLOT(slotSubjectLeftClicked()));
  connect(SubjectLabel, SIGNAL(rightClickedURL()), this, SLOT(slotSubjectRightClicked()));
  
  showDefault();
}

void MailLabel::setAddress(QString addr)
{
  if(addr.isEmpty())
  {
    showDefault();
    return;
  }

// I deleted the next line for IMAP
//	if(addr==address) return;

  address=addr;
  int attachments=0;

  MailObject *item=(MailObject *)vfs->_fact_ref(address, VFS_MAILS_MIMETYPE);

  if(item)
  {
    FromLabel->setText((QString)item->from());
    FromLabel->setTipText((QString)item->from());
    FromLabel->setUseTips(true);
    ToLabel->setText((QString)item->to());
    ToLabel->setTipText((QString)item->to());
    ToLabel->setUseTips(true);
    SubjectLabel->setText((QString)item->subject());
    SubjectLabel->setTipText((QString)item->subject());
    SubjectLabel->setUseTips(true);
    CcLabel->setText((QString)item->cc());
    CcLabel->setTipText((QString)item->cc());
    CcLabel->setUseTips(true);
    attachments=item->attachments();
  }

  FromText->setGeometry(5, 5, fm->width("From:"), fm->height());
  ToText->setGeometry(5, 25, fm->width("To:"), fm->height());
  SubjectText->setGeometry((width()-105)/2+5, 5, fm->width("Subject:"), fm->height());
  CcText->setGeometry((width()-105)/2+5, 25, fm->width("Cc:"), fm->height());

  FromLabel->setGeometry(10+FromText->width(), 5, (width()-105)/2-fm->width("From:")-10,
      fm->height());
  ToLabel->setGeometry(10+ToText->width(), 25, (width()-105)/2-fm->width("To:")-10,
      fm->height());
  SubjectLabel->setGeometry((width()-105)/2+10+SubjectText->width(), 5,
      (width()-105)/2-fm->width("Subject:")-10, fm->height());
  CcLabel->setGeometry((width()-105)/2+10+CcText->width(), 25,
      (width()-105)/2-fm->width("Cc:")-10, fm->height());
  h_line->setGeometry(0, 50, parentWidget()->width(), 2);
  attbutton->setGeometry(parentWidget()->width()-65, 0, 40, 40);

  FromText->show();
  ToText->show();
  SubjectText->show();
  CcText->show();
  FromLabel->show();
  ToLabel->show();
  SubjectLabel->show();
  CcLabel->show();

  NothingLabel->hide();

  if(attachments) attbutton->show();
  else attbutton->hide();

  ((MailView *)parent())->hideAttachmentPane();
}

void MailLabel::resizeEvent(QResizeEvent *)
{
  if(address.isEmpty())
  {
    NothingLabel->setGeometry(0, 0, width(), height());
  }
  else
  {
    FromText->move(5, 5);
    ToText->move(5, 25);
    SubjectText->move((width()-105)/2+5, 5);
    CcText->move((width()-105)/2+5, 25);
    h_line->setGeometry(0, 50, width(), 2);

    FromLabel->setGeometry(5+FromText->width(), 5, (width()-105)/2-fm->width("From:")-5,
        fm->height());
    ToLabel->setGeometry(5+ToText->width(), 25, (width()-105)/2-fm->width("To:")-5,
        fm->height());
    SubjectLabel->setGeometry((width()-105)/2+5+SubjectText->width(), 5,
        (width()-105)/2-fm->width("Subject:")-5, fm->height());
    CcLabel->setGeometry((width()-105)/2+5+CcText->width(), 25,
        (width()-105)/2-fm->width("Cc:")-5, fm->height());
    attbutton->setGeometry(width()-65, 0, 40, 40);
  }
}

void MailLabel::showDefault()
{
  FromLabel->hide();
  ToLabel->hide();
  CcLabel->hide();
  SubjectLabel->hide();
  FromText->hide();
  ToText->hide();
  SubjectText->hide();
  CcText->hide();
  attbutton->hide();
  h_line->setGeometry(0, 50, parentWidget()->width(), 2);

  NothingLabel->show();
}

QString MailLabel::getAddress()
{
  return address;
}

void MailLabel::slotFromLeftClicked()
{
}

void MailLabel::slotFromRightClicked()
{
  if(fromMenu)
    fromMenu->popup(QCursor::pos());
}

void MailLabel::slotCcLeftClicked()
{
}

void MailLabel::slotCcRightClicked()
{
  if(ccMenu)
    ccMenu->popup(QCursor::pos());
}

void MailLabel::slotToLeftClicked()
{
}

void MailLabel::slotToRightClicked()
{
  if(toMenu)
    toMenu->popup(QCursor::pos());
}

void MailLabel::slotSubjectLeftClicked()
{
}

void MailLabel::slotSubjectRightClicked()
{
  if(subjectMenu)
    subjectMenu->popup(QCursor::pos());

}

void MailLabel::slotFromCopy()
{
  
//  qDebug("slotFromCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(FromLabel->text());
  cb->setSelectionMode(false);
  cb->setText(FromLabel->text());
}

void MailLabel::slotCcCopy()
{
  
//  qDebug("slotCcCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(CcLabel->text());
  cb->setSelectionMode(false);
  cb->setText(CcLabel->text());
}

void MailLabel::slotToCopy()
{
  
//  qDebug("slotToCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(ToLabel->text());
  cb->setSelectionMode(false);
  cb->setText(ToLabel->text());
}

void MailLabel::slotSubjectCopy()
{
  
//  qDebug("slotSubjectCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(SubjectLabel->text());
  cb->setSelectionMode(false);
  cb->setText(SubjectLabel->text());
}

void MailLabel::slotFromAdd()
{
  addToAddressBook( FromLabel->text() );
}

void MailLabel::slotCcAdd()
{
  addToAddressBook( CcLabel->text() );
}

void MailLabel::slotToAdd()
{
  addToAddressBook( ToLabel->text() );
}

void MailLabel::addToAddressBook(const QString & line)
{
  QStringList addresses = QStringList::split(",", line);
  
  ContactsFactory * contactsFactory = 0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_CONTACTS_FACTORY );
    if( ref.isValid() )
      contactsFactory = static_cast<ContactsFactory*>( ref.instance() );
  }  
  
  if( !contactsFactory )
  {
    qDebug("MailLabel: Can't find the Contacts Factory");
    return;
  }
  
  VContactObject * contact = 0L;
  for(QStringList::ConstIterator it(addresses.begin()); it!=addresses.end(); ++it)
  {
    contact = contactsFactory->createContact(*it);
    if( !contact )
    {
      qDebug("MailLabel: Couldn't create the contact object.");
      return;
    }
  }
}
