/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "maillistview.h"

#include <qapplication.h>
#include <qfile.h>
#include <qaccel.h>
#include <qfiledialog.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include <qstringlist.h>
#include <qcursor.h>

// TINO include
#include <tino/uipartmanager.h>

#include <settings.h>
#include <vdragobject.h>
#include <qiconloader.h>
#include <clipboard.h>
#include <maillabel.h>
#include <mailobj.h>
#include <vfolder.h>
#include <indexmailitem.h>
#include <magellan.h>
#include <clientrmi.h>
#include <messagepropertiesdialog.h>
#include <composerfactory.h>
#include <komaetheratools.h>
#include <appstatus.h>

MailListView::MailListView(VFSModule *vfsModule, VFolder *folder, QWidget *parent, const char *name)
  : EListView(parent, name),
    DataHandler(vfsModule),
    sortingColumn(0),
    bSort(false)
{
  setAlternativeBackgroundColor( QColor(230, 230, 250) );
  
  setParentFolder( folder );

  // save type

  type = enumType( folder->viewType() );
  
  // get the display date option
  recDate =
    Settings::instance()->readBoolEntry
    ("/Aethera/Read Options/Use received date", false);

  // set listview properties
  setAllColumnsShowFocus(true);
  setShowSortIndicator(true);
  setSelectionMode(QListView::Extended);
  
  // make new popup menu
  listpopup=new QPopupMenu();
  
  // add columns and menu options
  switch(type)
  {
    case Magellan::mail:
      addColumn("", 20); // priority
      addColumn("", 20); // attachments
      addColumn("From", 120);
      addColumn("Subject", 250);
      addColumn("Date");
      addColumn("Status");
      addColumn("", 20); // flag
      addColumn("Size");
      setSorting(4, false);
      sortingColumn = 4;
      bSort = false;
      
      listpopup->insertItem(BarIcon("Reply.png"), "Reply", this, SLOT(reply()));
      listpopup->insertItem(BarIcon("ReplyToAll.png"), "Reply to all", this, SLOT(replyAll()));
      listpopup->insertItem(BarIcon("Forward.png"), "Forward", this, SLOT(forward()));
      listpopup->insertSeparator();
      break;
      
    case Magellan::sent:
      addColumn("", 20); // priority
      addColumn("", 20); // attachments
      addColumn("Recipient", 120);
      addColumn("Subject", 250);
      addColumn("Date");
      addColumn("Status");
      addColumn("Size");
      setSorting(4, false);
      sortingColumn = 4;
      bSort = false;
      
      listpopup->insertItem(BarIcon("Forward.png"), "Forward", this, SLOT(forward()));
      listpopup->insertSeparator();
      break;
      
    case Magellan::drafts:
      addColumn("", 20); // priority
      addColumn("", 20); // attachments
      addColumn("Recipient", 120);
      addColumn("Subject", 250);
      addColumn("Size");
      setSorting(3, true);
      sortingColumn = 3;
      bSort = true;
      
      listpopup->insertItem(BarIcon("open.xpm"), "Open", this, SLOT(slotOpenSelection()));
      listpopup->insertSeparator();
      break;
      
    case Magellan::outgoing:
      addColumn("", 20); // priority
      addColumn("", 20); // attachments
      addColumn("Recipient", 120);
      addColumn("Subject", 250);
      addColumn("Scheduling");
      addColumn("Size");
      setSorting(3, true);
      sortingColumn = 3;
      bSort = true;
      
      listpopup->insertItem(BarIcon("open.xpm"), "Open", this, SLOT(slotOpenSelection()));
      listpopup->insertSeparator();
      break;
      
    default:
      addColumn("", 20); // priority
      addColumn("", 20); // attachments
      addColumn("From", 120);
      addColumn("Subject", 250);
      addColumn("Date");
      addColumn("Status");
      addColumn("", 20); // flag
      addColumn("Size");
      setSorting(4, false);
      sortingColumn = 4;
      bSort = false;
      
      listpopup->insertItem(BarIcon("Reply.png"), "Reply", this, SLOT(reply()));
      listpopup->insertItem(BarIcon("ReplyToAll.png"), "Reply to all", this, SLOT(replyAll()));
      listpopup->insertItem(BarIcon("Forward.png"), "Forward", this, SLOT(forward()));
      listpopup->insertSeparator();
      break;
  }

  listpopup->insertItem(BarIcon("Cut.png"), "Cut", this, SLOT(slotCutSelection()));
  listpopup->insertItem(BarIcon("Copy.png"), "Copy", this, SLOT(slotCopySelection()));
  listpopup->insertItem(BarIcon("Paste.png"), "Paste", this, SLOT(slotPaste()));
  listpopup->insertItem(BarIcon("Delete.png"), "Delete", this, SLOT(slotDeleteSelection()));
  listpopup->insertSeparator();
  listpopup->insertItem(BarIcon("Save.png"), "Save", this, SLOT(slotSaveSelection()));
  listpopup->insertSeparator();
  listpopup->insertItem(BarIcon("flag.png"), "Toggle flag", this, SLOT(toggleFlag()));
  listpopup->insertItem(BarIcon("mark.xpm"), "Mark as unread", this, SLOT(markUnread()));
  listpopup->insertSeparator();
  listpopup->insertItem("Properties", this, SLOT(slotEditProperties()));

  connect(this, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)),
          this, SLOT(showPopup(QListViewItem *, const QPoint &, int)));
  
  connect((QObject*)header(), SIGNAL(clicked(int)), this, SLOT(slotSortingColumn(int)));
  
  connect(this, SIGNAL(doubleClicked(QListViewItem*)), SLOT(slotEditProperties()));

  QAccel * accel = new QAccel(this);
  accel->connectItem
    (accel->insertItem(Key_Delete), this, SLOT(slotDeleteSelection()));
}

MailListView::~MailListView()
{
  disconnect(this, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), this, SLOT(showPopup(QListViewItem *, const QPoint &, int)));
}

void MailListView::slotSortingColumn(int column)
{
  if( sortingColumn == column )
    bSort = false;
  else
    bSort = true;
    
  sortingColumn = column;
}

void MailListView::setSortingColumn()
{
  setSorting(sortingColumn, bSort);
}

void MailListView::loadMails()
{
  VFolder * folder = parentFolder();
  
  // scan for messages
  QStringList urls = vfs->children(folder->path());
  QStringList temp;
  int i=1;
  
  if( !urls.count() )
  {
    qDebug("loadMails(): No children to build.");
    return;
  }
  
  AppStatus * status = AppStatus::instance();
  
  if( !status )
  {
    qDebug("MailListView::loadMails() : no status bar");
    return;
  }
  
  status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, "Loading...", 0, urls.count());
  
  // for speeding reasons
  setSorting( -1 );

  MailObject * mail = 0L;
  QString mailId;
  QDict<VObject> & mailDict = folder->objectDict();
    
  for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
  {
    temp.push_back(*it);
    if( !(i%150) )
    {
      vfs->createVInstances(temp, false);

      for(QStringList::Iterator it1=temp.begin(); it1!=temp.end(); ++it1)
      {
        // this is a trick for speed reasons, you should use vfs->objectRef()
        mailId = (*it1).mid( (*it1).findRev("/")+1 );
        mail = dynamic_cast<MailObject*>(mailDict.find(mailId));

//        new IndexMailItem(this, vfs, (*it1));
        if( mail )
        {
          new IndexMailItem(this, vfs, mail);
        }
        else
        {
          qDebug("Hmm, wrong mail object");
        }
      }
      
      qApp->processEvents(1); // Let loaded mails appear.
      
      temp.clear();
      folder->freeObjects();
    }
    
    i++;
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, "Loading...", i);
  }
  
  // create the last 150(or less) mails
  vfs->createVInstances(temp, false);
      
  for(QStringList::Iterator it1=temp.begin(); it1!=temp.end(); ++it1)
  {
    // this is a trick for speed reasons, you should use vfs->objectRef()
    mailId = (*it1).mid( (*it1).findRev("/")+1 );
    mail = dynamic_cast<MailObject*>(mailDict.find(mailId));

    if( mail )
    {
      new IndexMailItem(this, vfs, mail);
    }
    else
    {
      qDebug("Hmm, wrong mail object");
    }
//    new IndexMailItem(this, vfs, (*it1));
  }
      
  temp.clear();

  setSortingColumn();
  sort();
  folder->freeObjects();
  
  status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, "Folder is loaded");
}

void MailListView::showPopup(QListViewItem *, const QPoint &pos, int)
{
  listpopup->popup(pos);
}

QStringList MailListView::getSelection()
{
  QStringList selection;


  if( !isSelectionEmpty() && INDEX_MAIL_ITEM_ID == currentItem()->rtti() )
  {
    for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); item; item=(IndexMailItem*)item->itemBelow())
      if( item->isSelected() )
      {
        if( item->vObject() )
          selection.append( item->vObject()->path() );
        else
          qDebug("Item from SortingWidget without object!");
      }
  }

  return selection;
}

bool MailListView::isSelectionMultiple()
{
	int selCount=0;
	IndexMailItem *item=(IndexMailItem *)firstChild();
	while(item && selCount<2)
	{
		if(item->isSelected()) selCount++;
		item=(IndexMailItem *)item->itemBelow();
	}
	
	if(selCount==2) return true;
	return false;

}

bool MailListView::isSelectionSingle()
{
	IndexMailItem *item=(IndexMailItem *)firstChild();
  int iCount=0;
  
	while(item && iCount<2)
	{
		if(item->isSelected())
      iCount++;
		item=(IndexMailItem *)item->itemBelow();
	}
	
	return iCount==1 ? true : false;
}

bool MailListView::isSelectionEmpty()
{

	IndexMailItem *item=(IndexMailItem *)firstChild();
	while(item)
	{
		if(item->isSelected()) return false;
		item=(IndexMailItem *)item->itemBelow();
	}
	
	return true;
}


//void MailListView::update()
//
//{
//  qDebug("---- MailListView::update ----");
//  
//  for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); item; item=(IndexMailItem*)item->itemBelow())
//    item->update();
//}

IndexMailItem *MailListView::firstSelected()
{
	IndexMailItem *item=(IndexMailItem *)firstChild();
	while(item)
	{
		if(item->isSelected()) return item;
		item=(IndexMailItem *)item->itemBelow();
	}
	
	return 0;
}

void MailListView::deleteSelection()
{
  // move messages to Deleted Mail
  QStringList selection = getSelection();
  if( !selection.count() )
    return;

//  bool multiDelete=selection.count()>1;
//  
  ClientRMIModule *clientRMI = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != clientRMI);

  if( Settings::instance()->readBoolEntry("/Aethera/Misc Mail/Permanent delete") ||
      parentFolder()->path() == vfs->root()+"/Mail/Deleted Messages" )
  {
    DataHandler::deleteSelection();

//    if(multiDelete)
//      clientRMI->setAutoExpunge(true);
  }
  else
  {
    vfs->move(selection, vfs->root()+"/Mail/Deleted Messages");
  }
}

void MailListView::selectAll()
{

  QListView::selectAll(true);

}

void MailListView::unselectAll()
{
  QListView::selectAll(false);
}

void MailListView::markUnread()
{
  QStringList selection=getSelection();

  for( QStringList::Iterator it=selection.begin(); it!=selection.end(); ++it)
  {
    MailObject *obj=static_cast<MailObject *>(vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE ));
    if (obj && obj->status()=="Read")
    {
//      obj->sync();
      obj->setStatus("New");
      obj->save();
    }
  }
}

void MailListView::markRead()
{
  QStringList selection=getSelection();

  for( QStringList::Iterator it=selection.begin(); it!=selection.end(); ++it)

  {
    MailObject *obj=static_cast<MailObject *>(vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE ));
    if(obj && obj->status()=="New")
    {
//      obj->sync();
      obj->setStatus("Read");
      obj->save();
    }
  }
}

void MailListView::markRead(QString url)
{
  MailObject *obj=static_cast<MailObject *>(vfs->_fact_ref(url, VFS_MAILS_MIMETYPE ));
  if(obj && obj->status()=="New")

  {
//    obj->sync();
    obj->setStatus("Read");
    obj->save();
  }
}

void MailListView::markAllRead()
{
  for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); item; item=(IndexMailItem*)item->itemBelow())
  {
    MailObject *obj = item->vObject();
    if (obj && obj->status()=="New")
    {
//      obj->sync();
      obj->setStatus("Read");
      obj->save();
    }
  }
}

void MailListView::markAllUnread()
{
  for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); item; item=(IndexMailItem*)item->itemBelow())
  {
    MailObject *obj = item->vObject();
    if (obj && obj->status()=="Read")
    {
//      obj->sync();
      obj->setStatus("New");
      obj->save();
    }
  }
}

void MailListView::toggleFlag()
{
  QStringList selection=getSelection();


  for( QStringList::Iterator it=selection.begin(); it!=selection.end(); ++it)
  {
    MailObject *obj=(MailObject *)vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE );
    if(obj)
    {
//      obj->sync();
      obj->setFlag(!obj->flag());
      obj->save();
    }

  }
}

void MailListView::reply()
{
  IndexMailItem *item=firstSelected();
  if (item && item->vObject())
  {
    ComposerFactory::instance()->openComposer(item->path(), ComposerFactory::Reply);
  }
  else
  {
    qDebug("maillistview: no item selected");
  }
}

void MailListView::replyAll()
{
  QStringList pathList = getSelection();

  if(pathList.count()>1)
    ComposerFactory::instance()->openComposer(pathList, ComposerFactory::ReplyAll);

  else
    ComposerFactory::instance()->openComposer(pathList[0], ComposerFactory::ReplyAll);
}

void MailListView::forward()
{
  QStringList pathList = getSelection();

  if(pathList.count()>1)
    ComposerFactory::instance()->openComposer(pathList, ComposerFactory::Forward);
  else
    ComposerFactory::instance()->openComposer(pathList[0], ComposerFactory::Forward);
}

void MailListView::forwardQuoted()
{
  QStringList pathList = getSelection();

  if(pathList.count()>1)
    ComposerFactory::instance()->openComposer(pathList, ComposerFactory::ForwardQuoted);
  else
    ComposerFactory::instance()->openComposer(pathList[0], ComposerFactory::ForwardQuoted);
}

void MailListView::forwardInline()
{
  QStringList pathList = getSelection();

  if(pathList.count()>1)
    ComposerFactory::instance()->openComposer(pathList, ComposerFactory::ForwardInline);
  else
    ComposerFactory::instance()->openComposer(pathList[0], ComposerFactory::ForwardInline);
}

void MailListView::forwardAsAttachment()
{
  QStringList pathList = getSelection();

  if(pathList.count()>1)
    ComposerFactory::instance()->openComposer(pathList, ComposerFactory::Forward);
  else
    ComposerFactory::instance()->openComposer(pathList[0], ComposerFactory::ForwardAsAttachment);
}

void MailListView::editProperties()
{
  QStringList pathList = getSelection();

  if(pathList.count())
  {
    UIPartManager *manager = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_MANAGER );

      if( ref.isValid() )
        manager = dynamic_cast<UIPartManager*>(ref.instance());
    }

    if( manager )
    {
      MessagePropertiesDialog *propertiesDialog=new MessagePropertiesDialog(pathList, (QWidget*)(manager->mainWindow()));
      propertiesDialog->exec(); // TODO: Check the result maybe?
      delete propertiesDialog;
    }
  }
}

void MailListView::openSelection()
{
  QStringList pathList = getSelection();

  if( parentFolder()->viewType() == "outbox" ||
      parentFolder()->viewType() == "drafts" )
  {
    if(pathList.count()>1)
      ComposerFactory::instance()->openComposer(pathList, ComposerFactory::EditMessage);
    else
      ComposerFactory::instance()->openComposer(pathList[0], ComposerFactory::EditMessage);
  }
}

void MailListView::saveSelection()
{
  IndexMailItem *selected = firstSelected();
  if(selected)
    selected->save();
}

void MailListView::slotEditProperties()
{
  editProperties();
}

void MailListView::slotDeleteSelection()
{
  deleteSelection();
}


void MailListView::slotCutSelection()
{
  cutSelection();
}

void MailListView::slotCopySelection()
{
  copySelection();
}

void MailListView::slotPaste()
{
  paste();
}

void MailListView::slotOpenSelection()
{
  openSelection();
}

void MailListView::slotSaveSelection()
{
  saveSelection();
}

int MailListView::enumType(QString type)
{
  if(type =="outbox")
    return Magellan::outgoing;
  if(type =="sent")
    return Magellan::sent;
  if(type =="drafts")
    return Magellan::drafts;
  return Magellan::mail;
}


QString MailListView::strType(int type)
{
  if(type == Magellan::outgoing)
    return "outbox";
  if(type == Magellan::sent)
    return "sent";
  if(type == Magellan::drafts)
    return "drafts";
  return "mail";
}

bool MailListView::byRecDate()
{
  return recDate;
}


void MailListView::slotUpdateVInstance(QString path)
{
  for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); item; item=(IndexMailItem*)item->itemBelow())
    if( item->path()==path )
    {
      item->update();
      break;
    }
}

void MailListView::slotUpdateVInstances(QStringList paths)
{
  QDict<char> findDict;
  bool useProgress = false;
  AppStatus * status = AppStatus::instance();
  
  for(QStringList::Iterator it=paths.begin(); it!=paths.end(); ++it)
    findDict.insert((*it), (*it).latin1());
  
  if( findDict.count()>10 )

  {
    useProgress = true;
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
      "Updating...", 0, findDict.count());
  }
  
  int i=0;
  for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); 
      item && findDict.size(); 
      item=(IndexMailItem*)item->itemBelow())
  {
    if( findDict.find( item->path() ) )
    {
//      qDebug("Item Update %s", item->vObject()->path().latin1());
      item->update();
      findDict.remove( item->path() );

      i++;
      if(useProgress)
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
          "Updating...", i);
      }
    }
  }
  
  status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, "Updated");
}

void MailListView::slotDeleteVInstances(QStringList paths)
{
  QDict<char> findDict;
  QPtrList<IndexMailItem> deletedItems;
  
  for(QStringList::Iterator it=paths.begin(); it!=paths.end(); ++it)
    findDict.insert((*it), (*it).latin1());
    
  for(IndexMailItem *item=(IndexMailItem*)this->firstChild(); 
      item && findDict.size(); 
      item=(IndexMailItem*)item->itemBelow())
  {
    if( findDict.find( item->path() ) )
    {
      findDict.remove( item->path() );
      deletedItems.append(item);
    }
  }
{  
  for(QPtrListIterator<IndexMailItem> it(deletedItems); it.current(); ++it)
  {
//    qDebug("Delete %s", it.current()->path().latin1());
    delete it.current();
  }
}
}


QDragObject * MailListView::dragObject()
{
  qDebug("MailListView::dragObject");
  if (0 == currentItem())
  {
    qDebug("currentItem is 0");
    return 0;
  }

  qDebug("Making new VDragObject");
  VDragObject * drag = new VDragObject(viewport());

  QPixmap p(BarIcon("message-single.xpm"));

  if (p.isNull())
  {
    qDebug("Drag pixmap is null");
  }
//  qDebug("Drag pixmap is %dx%d", p.width(), p.height());

  drag->setPixmap(p);

  QListViewItemIterator it(this);

  for (; it.current(); ++it)
  {
    if (it.current()->isSelected())
    {
      IndexMailItem * i = static_cast<IndexMailItem *>(it.current());

      drag->append
        (
          i->path(),
          itemRect(i),
          QRect()
        );

    }
  }

  return drag;
}

