/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#ifndef __MailObject__
#define __MailObject__

#include <vobject.h>
#include <vobjectfactory.h>
#include <dateclass.h>
#include <addressclass.h>
#include <addresslistclass.h>

#include <qstring.h>
#include <qcolor.h>

class MAIL_PLUGIN_API MailObject : public VObject
{
  friend class MailObjectFactory;
  
  public:
    
    virtual ~MailObject();
    
    /**
     * properties
     */
    AddressClass from() const;
    void setFrom(const AddressClass &newFrom);

    AddressListClass to() const;
    void setTo(const AddressListClass &newTo);

    QString subject() const;
    void setSubject(const QString &newSubject);

    QString status() const;
    void setStatus(const QString &newStatus);

    QString priority() const;
    void setPriority(const QString &newPriority);

    DateClass sentDate() const;
    void setSentDate(const DateClass &newSentDate);

    DateClass receivedDate() const;
    void setReceivedDate(const DateClass &newReceivedDate);

    QString scheduling() const;
    void setScheduling(const QString &newScheduling);

    QStringList flags() const;
    void setFlags(const QStringList &newFlags);

    QColor color() const;
    void setColor(const QColor &newColor);

    bool flag() const;
    void setFlag(bool newFlag);

    QString inReplyTo() const;
    void setInReplyTo(const QString &newInReplyTo);

    QStringList references() const;
    void setReferences(const QStringList &newReferences);

    QString indexID() const;
    void setIndexID(const QString &newIndexID);

    QString parentIndexID() const;
    void setParentIndexID(const QString &newParentIndexID);

    QStringList relatedIndexIDList() const;
    void setRelatedIndexIDList(const QString &newRelatedIndexIDList);

    AddressClass replyTo() const;
    void setReplyTo(const AddressClass &newReplyTo);

    AddressListClass cc() const;
    void setCc(const AddressListClass &newCc);

    AddressListClass bcc() const;
    void setBcc(AddressListClass &newBcc);

    QString messageID() const;
    void setMessageID(const QString &newMessageID);

    QString organization() const;
    void setOrganization(const QString &newOrganization);

    QString uid() const;
    void setUid(const QString &newUid);

    QStringList extendedStatus() const;
    void setExtendedStatus(const QStringList &newExtendedStatus);

    QString contentType() const;
    void setContentType(const QString &newContentType);

    QString contentEncoding() const;
    void setContentEncoding(const QString &newContentEncoding);

    QString account() const;
    void setAccount(const QString &newAccount);

    QStringList route() const;
    void setRoute(const QStringList &newRoute);

    bool requestReceipt() const;
    void setRequestReceipt(bool newRequestReceipt);

    QString archive() const;
    void setArchive(const QString &newArchive);

    QString category() const;
    void setCategory(const QString &newCategory);

    int persistence() const;
    void setPersistence(int newPersistence);

    // volatile data
    bool attachments() const;
    unsigned long size() const;

    // server-query data
    QCString message() const;
    QCString header() const;
    bool hasHTML() const;
    bool hasText() const;
    
    QString text() const;
    QString html() const;
    
    bool textPart() const;
    int messageText() const;
    bool htmlPart() const;

    // reply and forward features
    bool isReplied() const;
    bool isForwarded() const;

    // this is a special kind of object, and we overload sync() from VObject
    bool sync();

    // save/load VObject methods
    QByteArray _stream();
    void _narrow(const QByteArray &);

    /**
     * query handling
     */
    virtual bool getProperty(const QString &query, Any &property);
    virtual bool setProperty(const QString &query, Any &property);
    virtual QueryProcessor::QueryType queryType(const QString &query);
  
  protected:
    MailObject(const QString & _path, const QByteArray & _data);
    MailObject(const QString & _path);
    
  private:
    // core
    AddressClass _from;
    AddressListClass _to;
    QString _subject;
    QString _status;
    QString _priority;
    QString _contentType;
    DateClass _sentDate;
    DateClass _receivedDate;
    QString _scheduling;
    QStringList _flags;
    QColor _color;
    Q_UINT8 _flag;
    QString _inReplyTo;
    QStringList _references;
    QString _indexID;
    QString _parentIndexID;
    QStringList _relatedIndexIDList;

    // other
    AddressClass _replyTo;
    AddressListClass _cc;
    AddressListClass _bcc;
    QString _messageID;
    QString _organization;
    QString _uid;
    QStringList _extendedStatus;
    QString _contentEncoding;
    QString _account;
    QStringList _route;
    Q_UINT8 _requestReceipt;
    QString _archive;
    QString _category;
    Q_UINT32 _persistence;

    /** If it has attachments. */
    Q_UINT8 _attachments;
    /** The message size.*/
    Q_UINT32 _size;

    QByteArray _extraData;

    bool __cached;
};

// the mail object factory

class MAIL_PLUGIN_API MailObjectFactory : public VObjectFactory
{
  public:
    /** Build a new mail object.*/
#ifdef _QTWIN_
    VObject *_new(const QString & _path, const QByteArray & _data);
    VObject *_new(const QString & _path);
#else
    MailObject *_new(const QString & _path, const QByteArray & _data);
    MailObject *_new(const QString & _path);
#endif
};

#endif







