#include "popupaction.h"

#include <qiconset.h>
#include <qtoolbutton.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>

PopupAction::PopupAction( const QString & text, const QIconSet & icon,
                          const QString & menuText, QKeySequence accel,
                          QObject * parent, const char * name,
                          bool toggle)
  : QAction(text, icon, menuText, accel, parent, name, toggle),
    popupMenu(0L)
{
}

PopupAction::PopupAction(QWidget *parent, const char *name)
  : QAction(parent, name),
    popupMenu(0L)
{
}

PopupAction::~PopupAction()
{
}

void PopupAction::setPopup(QPopupMenu * newPopup)
{
  if(newPopup)
  {
    if(popupMenu)
    {
      disconnect(popupMenu, SIGNAL(activated(int)), this, SLOT(slotActivated(int)));
    }
    
    popupMenu = newPopup;
    
    connect(popupMenu, SIGNAL(activated(int)), this, SLOT(slotActivated(int)));
  }
}

QPopupMenu* PopupAction::popup() const
{
  return popupMenu;
}

void PopupAction::addedTo(QWidget * actionWidget, QWidget * container)
{
  if (actionWidget->inherits("QToolButton"))
  {
    QToolButton *button = (QToolButton *)actionWidget;

    button->setPopup(popupMenu);
    button->setPopupDelay(0);
  }
  
  QAction::addedTo(actionWidget, container);
}

void PopupAction::update(int id)
{
  if(!popupMenu)
    return;
  
  QIconSet * iconSet = popupMenu->iconSet(id);
  
  if(iconSet && !iconSet->isNull() )
  {
    setIconSet(*iconSet);
  }
}

void PopupAction::slotActivated(int id)
{
  update(id);
  
  emit activated(id);
}

bool PopupAction::addTo(QWidget *w)
{
  if( !w )
    return false;
  
  if(w->inherits("QPushButton")) 
  {
    QPushButton *b = (QPushButton *)w;
    
    b->setPopup(popupMenu);
    b->setIconSet( iconSet() );
    
    connect(b, SIGNAL(destroyed()), SLOT(removeButton()));
    
    return true;
  }
  
  bool bRet = QAction::addTo(w);
  
  return bRet;
}
