//own includes
#include "notepreview.h"

//Qt includes
#include <qmultilinedit.h>
#include <qstring.h>
#include <qevent.h>
#include <qdatetime.h>
#include <qlabel.h>
#include <qframe.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qfont.h>
#include <qcolor.h>
#include <qframe.h>
#include <textfitter.h>
#include <qcursor.h>
#include <qtextstream.h>
#include <qframe.h>
#include <qsizegrip.h>

//other includes
#include <qiconloader.h>
#include <settings.h>
#include <qsettingsextra.h>
#include <ebutton.h>
#include <notewindow.h>
#include <textfitter.h>
#include <noteedit.h>
#include <vnoteobject.h>
#include <vfsmodule.h>


NotePreview::NotePreview( QWidget* parent, const char* name )
  : QFrame( parent, name )
{
  resize( 265, 270 );

  QColor defaultColor( Qt::yellow );
  QColor color
    = QSettingsExtra::readColorEntry( Settings::instance(),
                                      "/Aethera/Notes/Default color",
                                      &defaultColor
                                    );

  QColorGroup light( QColor("black"), color, QColor("black"), QColor("black"),
                     QColor("black"), QColor("black"), color
                   );
  QColorGroup dark( QColor(255, 255, 140), QColor(128, 0, 0), QColor("black"), QColor("black"),
                    QColor("black"), QColor(255, 255, 140), QColor(128, 0, 0)
                  );

  label=new QLabel(this);
  label->setText("");
  label->setAlignment( AlignHCenter);
  label->installEventFilter(this);
  label->setPalette(QPalette(dark, dark, dark));
  label->setGeometry(18, 2, width()-36, 14);

  label->setFont(QFont("helvetica", 12, QFont::Bold));

  cl_button=new EButton(BarIcon("x.xpm"), this);
  cl_button->setNormalFrameStyle(QFrame::NoFrame);
  cl_button->setFocusFrameStyle(QFrame::Box|QFrame::Plain);
  cl_button->setClickedFrameStyle(QFrame::Box|QFrame::Plain);
  cl_button->setContentMargin(0);
  cl_button->setGeometry(width()-16, 2, 14, 14);

  edit_box = new NoteEdit(this);
  edit_box->setGeometry(2, 18, width()-4 , height()-39);
  edit_box->installEventFilter(this);
  edit_box->setFocus();
  edit_box->setFrameStyle(QFrame::NoFrame);
  edit_box->setPalette(QPalette(light, light, light));
  edit_box->setWordWrap(QMultiLineEdit::WidgetWidth);
  edit_box->setText("Preview a note ... Hello  world" );
  QColor defaultTextColor( Qt::black );
  QColor textColor
    = QSettingsExtra::readColorEntry( Settings::instance(),
                                      "/Aethera/Notes/Default text color",
                                      &defaultTextColor
                                    );
  edit_box->selectAll( true );
  edit_box->setColor( textColor );
  edit_box->deselect( );
  QFont widgetFont( edit_box->font() );
  QFont savedFont = QSettingsExtra::readFontEntry
    (Settings::instance(), "/Aethera/Notes/Default font", &widgetFont);
  edit_box->setFont( savedFont );

  hline=new QFrame(this);
  hline->setFrameStyle(QFrame::HLine|QFrame::Plain);
  hline->setLineWidth(1);
  hline->setMidLineWidth(0);
  hline->setGeometry(2, height()-20, width()-2, 1);
  hline->installEventFilter(this);

  date_line=new QLabel(this);
  date_line->setGeometry(2, height()-16, width()-20, 14);
  date_line->setPalette(QPalette(light, light, light));
  date_line->setText( "Date:" );

  icon=new QLabel(this);
  icon->setPixmap(BarIcon("notes.xpm"));
  icon->setGeometry(2, 2, 14, 14);
  icon->setPalette(QPalette(light, light, light));
  icon->setBackgroundMode(PaletteBackground);

  setTitle();
}

NotePreview::~NotePreview()
{

}

void NotePreview::setTitle()
{
  QString t;

  t = edit_box->text();
  t = t.left(10) + "...";

  label->setText( t );
}

void NotePreview::slotSetBackground( const QColor& color)
{
  QColorGroup light( QColor("black"), color, QColor("black"), QColor("black"),
                     QColor("black"), QColor("black"), color
                   );
  QColorGroup dark( QColor(255, 255, 140), QColor(128, 0, 0), QColor("black"), QColor("black"),
                    QColor("black"), QColor(255, 255, 140), QColor(128, 0, 0)
                  );

  setBackgroundColor(color);
  edit_box->setBackgroundColor( color );
  label->setPalette(QPalette(dark, dark, dark));
  date_line->setPalette(QPalette(light, light, light));
  icon->setPalette(QPalette(light, light, light));
  icon->setBackgroundMode(PaletteBackground);
  edit_box->setPalette(QPalette(light, light, light));
}

void NotePreview::slotSetTextColor( const QColor& textColor)
{
  edit_box->selectAll( true );
  edit_box->setColor( textColor );
  edit_box->deselect( );
}

void NotePreview::slotSetFont( const QFont& font)
{
  edit_box->setFont( font );
}

QSize NotePreview::minimumSizeHint () const
{
  return QSize( 265, 270 );
}
