//own includes
#include <vfsnoteiconview.h>

//komnotesplugin includes
#include <vnoteobject.h>

//Aethera includes
#include<vfsmodule.h> //vfs service - required t build contacts list
#include<vfolder.h>

//QT includes
#include <qregexp.h>
#include <qiconview.h>

//tkcBase includes
#include <qiconloader.h>

VFSNoteIconView::VFSNoteIconView(QWidget* parent, const char* name)
  :VFSObjectIconView(parent,name)
{
  setSelectionMode( QIconView::Single );
  setArrangement( QIconView::LeftToRight);
  setAutoArrange( true );

  connect(this,
          SIGNAL( selectionChanged(QIconViewItem*) ),
          SLOT( slotObjectSelected(QIconViewItem*) )
         );
}


VFSNoteIconView::~VFSNoteIconView()
{

}


void VFSNoteIconView::slotObjectSelected(QIconViewItem* )
{
  QString tmp = getSelectedObject();

  emit objectSelected(tmp);
}


void VFSNoteIconView::slotSetFolder(const QString& fld)
{
  noteFilesMap.clear();//clean the contacts file map
  clear(); //clear the contacts list first

  //fill the list with contact entries
  VFSModule *vfs=0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  //scanning for contacts
  if(vfs)
  {
    VFolder *folder=0L;

    folder = vfs->folderRef( fld );
    if(!folder)
    {
      qDebug("No such folder %s",(vfs->root()+fld).latin1());
      return;
    }
    if( "notes" == folder->type() )
    {
      QPtrList<VObject> & entries = folder->objectList();
      for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
        if( folder->isSimpleObject( it1.current()->name() ) )
        {
          VNoteObject * note = dynamic_cast<VNoteObject*>(it1.current());
          if(note)
            addListObject(note,folder->path()+"/"+note->name());
        }
    }
  }
}


QString VFSNoteIconView::getSelectedObject()
{
  return noteFilesMap[ currentItem() ];
}


void VFSNoteIconView::addListObject(VNoteObject* c, const QString& path)
{

  QIconViewItem *itm = new QIconViewItem(this);
  noteFilesMap[itm]=path;

  QString noteText=c->text();

  if (noteText.isEmpty())
    noteText = "[Empty]";

  if (noteText.length() > 10)
    noteText = noteText.left(10) + "...";

  itm->setText( noteText );
  itm->setPixmap( BarIcon("notes-large-yellow.png") );
}
