//own includes
#include <vfsnotelistview.h>

//komnotesplugin includes
#include <vnoteobject.h>

//Aethera includes
#include<vfsmodule.h> //vfs service - required t build contacts list
#include<vfolder.h>

//QT includes
#include <qregexp.h>

VFSNoteListView::VFSNoteListView(QWidget* parent, const char* name)
  :VFSObjectListView(parent,name)
{
  setSelectionMode(QListView::Single);
  setShowSortIndicator(true);
  setAllColumnsShowFocus(true);
  setSorting(0);

  buildHeader();

  connect(this,
          SIGNAL( selectionChanged(QListViewItem*) ),
          SLOT( slotObjectSelected(QListViewItem*) )
         );
}

VFSNoteListView::~VFSNoteListView()
{
  
}


void VFSNoteListView::slotObjectSelected(QListViewItem* )
{
  QString tmp = getSelectedObject();

  emit objectSelected(tmp); 
}


void VFSNoteListView::slotSetFolder(const QString& fld)
{
  noteFilesMap.clear();//clean the contacts file map
  clear(); //clear the contacts list first

  //fill the list with contact entries
  VFSModule *vfs=0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  //scanning for contacts
  if(vfs)
  {
    VFolder *folder=0L;

    folder = vfs->folderRef( fld );
    if(!folder)
    {
      qDebug("No such folder %s",(vfs->root()+fld).latin1());
      return;
    }
    if( "notes" == folder->type() )
    {
      QPtrList<VObject> & entries = folder->objectList();
      for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
        if( folder->isSimpleObject( it1.current()->name() ) )
        {
          VNoteObject * note = dynamic_cast<VNoteObject*>(it1.current());
          if(note)
            addListObject(note,folder->path()+"/"+note->name());
        }
    }
  }
}


QString VFSNoteListView::getSelectedObject()
{
  return noteFilesMap[ currentItem() ];
}


void VFSNoteListView::addListObject(VNoteObject* c, const QString& path)
{
  int headerIndex=0;

  VFSObjectListViewItem *itm = new VFSObjectListViewItem(this);
  noteFilesMap[itm]=path;

  QString noteText=c->text();
  noteText.truncate(20);
  noteText.replace(QRegExp("\n")," ");

  itm->setText( headerIndex++,noteText );
  
  itm->setText(headerIndex, (c->time()).toString() );

  
}


void VFSNoteListView::buildHeader()
{

    headerColumns.append("Name");
    headerColumns.append("Creation date");

    QValueListIterator<QString> it;
    for(it=headerColumns.begin();it!=headerColumns.end();++it)
    {
            addColumn(*it);
    }
}  
