/***************************************************************************
                          imap4subfoldersjob.cpp  -  description
                             -------------------
    begin                : Tue Jun 18 2002
    copyright            : (C) 2002 by eugen
    email                : eugen@eugen
 ***************************************************************************/

#include "imap4subfoldersjob.h"

#include <imap4client.h>
#include <imap4handler.h>
#include <commandsprocessor.h>
#include <mailinfo.h>
#include <messagedevice.h>

#include <appstatus.h>

IMAP4SubfoldersJob::IMAP4SubfoldersJob
(
  QSocket           * sock,
  IMAP4Client       * parent
)
  : IMAP4Job(sock, parent),
    state_ ( START )
{
  // Empty.
}

IMAP4SubfoldersJob::~IMAP4SubfoldersJob()
{
  // Empty.
}

  void
IMAP4SubfoldersJob::run()
{
  status->message(AppStatus::Server, AppStatus::Text, AppStatus::Waiting,
    "Searching for subscribed folders...");
  
  state_ = WaitingForSubscribedFolders;
  client()->sendLsub( command()->mailbox );
}

  bool
IMAP4SubfoldersJob::processLine(const QString & line)
{
  QString firstWord;
  
  client()->parseNextWord(firstWord, line);
  if( firstWord == client()->getTag() )
  {
    if( client()->replyOK(line) )
    {
      switch( state_ )
      {
        case WaitingForSubscribedFolders :
          {
            for(MboxListIterator it(client()->subfolders()); it.current(); ++it)
            {
              folders_ += it.current()->name;
            }
          }
          state_ = WaitingForFoldersStatus;
        case WaitingForFoldersStatus :
          if( folders_.isEmpty() )
          {
            state_ = DONE;
            IMAP4Handler::ref()->slotSubfolders( client() );
            emit finished(true);
          }
          else
          {
            client()->sendList("\"\"", folders_.first() );
            folders_.pop_front();
          }
          break;
          
        default :
          state_ = DONE;
          qDebug("IMAP4SubfoldersJob::processLine() : wrong state");
          emit finished(false);
          return false;
          break;
      }
      
      return true;
    }
    else
    {
      qDebug("IMAP4SubfoldersJob::processLine() : bad answer : %s", line.latin1());
      
      emit finished(false);
      
      return false;
    }
  }
  else
    return client()->parseLine(line);
    
  return true;
}

  bool
IMAP4SubfoldersJob::processMultiLine(const QByteArray &)
{
  qDebug("IMAP4SubfoldersJob::processMultiLine() : not available");
  return false;
}

