/***************************************************************************
                          imap4syncjob.cpp  -  description
                             -------------------
    begin                : Tue Jun 18 2002
    copyright            : (C) 2002 by theKompany.com
    email                : eug@thekompany.com
 ***************************************************************************/

#include "imap4syncjob.h"

#include <imap4client.h>
#include <imap4handler.h>
#include <commandsprocessor.h>
#include <appstatus.h>
#include <mailinfo.h>
#include <messagedevice.h>
#include <remotemailfolder.h>
#include <orb.h>

#include <qapplication.h>

IMAP4SyncJob::IMAP4SyncJob
(
  QSocket           * sock,
  IMAP4Client       * parent
)
  : IMAP4Job(sock, parent),
    state_ ( START ),
    start_ (true),
    uMessages(0),
    folder (0L)
{
  // Empty.
}

IMAP4SyncJob::~IMAP4SyncJob()
{
  // Empty.
}

  void
IMAP4SyncJob::run()
{
  folder = (RemoteMailFolder*)ObjectRequestBroker::thisInstance()->mailfolderReference( command()->other );

  if( !folder )
  {
    emit finished(false);
    return;
  }

  client()->setSingleLine( true );
  
#ifdef IMAP4_DEBUG
  client()->showStatus();
  folder->showStatus();
#endif

  if( client()->uidValidity() == folder->uidValidity() &&
      client()->messagesNumber() == folder->messages() &&
      client()->messagesNumber() == folder->getMessageCount() &&
      client()->uidNext() == folder->uidNext() )
  {
    emit finished(true);
    return;
  }

  start_ = true;
  
  if( 0 == folder->uidNext() &&
      0 == folder->uidValidity() &&
      0 == folder->messages() )
  {
    /*
      We need all the messages because it is first time
      when this folder is accessed.
    */
    state_ = WaitingForAllMails;
    client()->sendUid( QString("SEARCH NOT DELETED") );
  }
  else
  {
    // Check for the new messages.
    // UID SEARCH NEW
    // TODO: this has to be changed
    state_ = CheckingAllMails;
    client()->sendUid( QString("SEARCH NOT DELETED") );
  }
}

  bool
IMAP4SyncJob::processLine(const QString & line)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4SyncJob::processLine : %s", line.latin1());
#endif

  QString firstWord;
  
  client()->parseNextWord(firstWord, line);
  if( firstWord == client()->getTag() )
  {
    if( client()->replyOK(line) )
    {
      UIDMailList &uidDict = client()->searchUID();
      
      if( start_ )
      {
        uMessages = uidDict.count();
        status->message(AppStatus::Server, AppStatus::Progress, AppStatus::Waiting,
          "Downloading...", 0, uMessages);
      }
        
//#ifdef IMAP4_DEBUG
//      QStringList temp;
//      for(UIDIterator it(uidDict); it.current(); ++it)
//      {
//        temp += it.currentKey();
//      }
//      qDebug("UID list: %s", temp.join(",").latin1());
//#endif
              
      switch( state_ )
      {
        case CheckingForDeletedMails :
          // remove from local all the mails which aren't on the server
          if( start_ )
          {
            // check what should be deleted from local
            for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
                it!=folder->uidJar->rcvList.end(); ++it)
            {
              QString uidNumber( (*it).first.c_str() );
              uidNumber = uidNumber.left( (unsigned)uidNumber.find("@") );
                
              if( !uidDict[uidNumber] )
                folder->uidJar->trashList.insert((*it));
              else
                uidDict.remove(uidNumber);
            }
              
            IMAP4Handler::ref()->slotDeleteTrashMessages( client() );
            
            start_ = false;
          }
          
          //Set the status
          folder->setStatus(client()->uidValidity(), client()->uidNext(),
                            client()->messagesNumber(), client()->unseen(),
                            client()->recent());
          state_ = DONE;
          emit finished(true);
          return true;
          break;
          
        case WaitingForAllMails :
#ifdef IMAP4_DEBUG
          qDebug("WaitingForAllMails : %u", uidDict.count());
#endif
          
          if( start_ )
          {
            start_ = false;
          }
          
          if( uidDict.isEmpty() )
          {
            //Set the status
            folder->setStatus(client()->uidValidity(), client()->uidNext(),
                              client()->messagesNumber(), client()->unseen(),
                              client()->recent());
            state_ = DONE;
            IMAP4Handler::ref()->slotProgress( client() );
            emit finished(true);
            return true;
          }
          else
          {
            status->message(AppStatus::Server, AppStatus::Progress, AppStatus::Waiting,
              "Downloading...", uMessages - uidDict.count(), uMessages);
              
            int i=0;
            QStringList uids;
            
            {
              for(UIDIterator it(uidDict); it.current(); ++it)
              {
                if( i>=25 )
                  break;
                else
                {
                  uids += it.currentKey();
                }
                
                i++;
              }
            }
          
            uids.sort();
            {
              for(QStringList::Iterator it=uids.begin(); it!=uids.end(); ++it)
              {
                uidDict.remove( *it );
              }
            }
          
            // UID FETCH it (FLAGS INTERNALDATE UID BODY[HEADER])
            client()->setSingleLine( true );
            client()->sendUid( QString("FETCH ") + uids.join(",") +
                               " (FLAGS INTERNALDATE UID BODY[HEADER])" );
          }
          break;
          
        case CheckingAllMails :
#ifdef IMAP4_DEBUG
          qDebug("CheckingAllMails : %u", uidDict.count());
#endif
          
          if( start_ )
          {
            if(client()->uidValidity() != folder->uidValidity())
            {
              // remove everything since uidvalidity is changed
              for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
                  it!=folder->uidJar->rcvList.end(); ++it)
              {
                folder->uidJar->trashList.insert((*it));
              }
            }
            else
            {
              // check what should be deleted from local
              for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
                  it!=folder->uidJar->rcvList.end(); ++it)
              {
                QString uidNumber( (*it).first.c_str() );
                uidNumber = uidNumber.left( (unsigned)uidNumber.find("@") );
                
                if( !uidDict[uidNumber] )
                  folder->uidJar->trashList.insert((*it));
                else
                  uidDict.remove(uidNumber);
              }
            }
            
            IMAP4Handler::ref()->slotDeleteTrashMessages( client() );
            
            start_ = false;
          }
          
          // download the new mails (unseen or seen)
          if( uidDict.isEmpty() )
          {
            //Set the status
            folder->setStatus(client()->uidValidity(), client()->uidNext(),
                              client()->messagesNumber(), client()->unseen(),
                              client()->recent());
            state_ = DONE;
            IMAP4Handler::ref()->slotProgress( client() );
            emit finished(true);
            return true;
          }
          else
          {
            status->message(AppStatus::Server, AppStatus::Progress, AppStatus::Waiting,
              "Downloading...", uMessages - uidDict.count(), uMessages);
            
            int i=0;
            QStringList uids;
            {
              for(UIDIterator it(uidDict); it.current(); ++it)
              {
                if( i>=25 )
                  break;
                else
                {
                  uids += it.currentKey();
                }
                
                i++;
              }
            }
            
            uids.sort();
            {
              for(QStringList::Iterator it=uids.begin(); it!=uids.end(); ++it)
              {
                uidDict.remove( *it );
              }
            }
          
            // UID FETCH it (FLAGS INTERNALDATE UID BODY[HEADER])
            client()->setSingleLine( true );
            client()->sendUid( QString("FETCH ") + uids.join(",") +
                               " (FLAGS INTERNALDATE UID BODY[HEADER])" );
          }
          break;
          
        default:
          state_ = DONE;
          qDebug("IMAP4SyncJob::processLine() : wrong state");
          emit finished(false);
          return false;
          break;
      }
      
      return true;
    }
    else
    {
      qDebug("IMAP4SyncJob::processLine() : bad answer : %s", line.latin1());
      
      emit finished(false);
      
      return false;
    }
  }
  else
    return client()->parseLine(line);
    
  emit finished(false);
  return false;
}

  bool
IMAP4SyncJob::processMultiLine(const QByteArray & block)
{
  return client()->handleFetchData(block);
}

