/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <pop3handler.h>
#include <imap4handler.h>
#include <iosynchandler.h>
#include <mailfolder.h>
#include <indexclass.h>
#include <orb.h>
#include <string>
#include <qsettings.h>
#include <messagedescriptor.h>
#include <messagedevice.h>
#include <dateclass.h>
#include <mimecodec.h>

#include <settings.h>

// Isn't it a bit crazy to link with a plugin ?
// It isn't a plugin right now, but it should be.
#include <mailfilter.h>

IOSyncHandler *IOSyncHandler::thisInstance;

WaitingOperations::WaitingOperations(const QString & folderName, ToDo action, QStringList param)
  : folderURL(folderName),
    whatToDo(action),
    parameters(param)
{
}

WaitingOperations::~WaitingOperations()
{
}


IOSyncHandler::IOSyncHandler()
{
  vfs = 0;
  mailFilter = 0;
  
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = dynamic_cast<VFSModule*>(ref.instance());
    else
    {
      qDebug("\n NO VFS Provider !\n");
    }
    
    ref = trader->registeredComponent( AETHERA_MAILFILTER_SERVICE );
    if( ref.isValid() )
      mailFilter = dynamic_cast<MailFilterV*>(ref.instance());
    else
    {
      qDebug("\n NO MAILFILTER Provider !\n");
    }
  }
  
  toDo.setAutoDelete(true);
}
  
IOSyncHandler *IOSyncHandler::ref()
{
  return thisInstance?thisInstance:(thisInstance=new IOSyncHandler());
}

void IOSyncHandler::dispatchToInbox
(
  const QCString  & messageData,
  const QCString  & uidl,
  const QString   & accountName
)
{
  MessageClass message(messageData);

  Q_ASSERT(0 != mailFilter);

  // Find folder to create the message.
  QString mFolder = mailFilter->messageFolder(&message);

//  qDebug("mFolder: %s" + mFolder);

  // XXX: There is a problem here.
  // ClientVFS should not be called into the server side.

  MailFolder * folder = 0;

  if (!mFolder.isEmpty())
  {
    folder =
      ObjectRequestBroker::thisInstance()->mailfolderReference
      (vfs->map(mFolder));

    if (!foldersToSync.contains(mFolder))
      foldersToSync.append(mFolder);
  }

  if (0 == folder)
  {
//    qDebug
//      (
//        "IOSyncHandler::dispatchToInbox: Folder for this message"
//        " (as defined by filter) seems to not exist."
//        " Trying to write to inbox instead."
//      );

    folder =
      ObjectRequestBroker::thisInstance()->mailfolderReference("sysmail_inbox");
  }

  if (0 == folder)
  {
    // TODO: Simply return false here, and don't send DELE to the server.
    qFatal
      (
        "A fatal error has occured. Inbox interface not found."
        " Message not saved (left on server)."
      );
  }
  else
  {
    // We use sync signal - last param is true!

//    qDebug("IOSyncHandler::dispatchToInbox: uidl == `%s'", uidl.data());

    IndexClass * idx =
      folder->createMessage
      (
        messageData,
        uidl,
        QDateTime::currentDateTime(),
        accountName,
        &message,
        true
      );

    if (0 == idx)
    {
      qWarning("IOSyncHandler::dispatchToInbox: idx is 0");
      return;
    }

    mailFilter->applyFilter(idx, false);
  }
}

void IOSyncHandler::dispatchMessage(const FetchInfo &msg, bool bSync)
{
  RemoteMailFolder *imapFolder = IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message discarded.");
  }
  else
  {
    QDateTime rcvTime;
    rcvTime.setTime_t(msg.rcvtime);
    
    imapFolder->createMessage(msg.message, msg.uid.latin1(), rcvTime,
                              imapFolder->account(), 0, bSync, msg.flags);
  }
}

void IOSyncHandler::updateMessage(const FetchInfo &msg, const QString & indexURL)
{
  RemoteMailFolder *imapFolder=IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message discarded.");
  }
  else
  {
    imapFolder->updateMessage(msg.message, indexURL);
    checkToDo(indexURL);
  }
}

void IOSyncHandler::deleteMessage(const QString & indexURL)
{
  RemoteMailFolder *imapFolder=IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message not deleted.");
  }
  else
  {
    imapFolder->deleteMessage(indexURL);
  }
}

void IOSyncHandler::deleteTrashMessages()
{
  RemoteMailFolder *imapFolder=IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message not deleted.");
  }
  else
  {
    imapFolder->deleteTrashMessages();
  }
}

void IOSyncHandler::pop3Sync()
{
  MailFolder *inbox=ObjectRequestBroker::thisInstance()->mailfolderReference("sysmail_inbox");
  
  if(!inbox)
  {
    printf("A fatal error has occured, inbox interface not found. Message discarded.");
  }
  else
    inbox->sync();
  
  // there is a problem here
  // ClientVFS should not be called into the server side
  for ( QStringList::Iterator it = foldersToSync.begin(); it != foldersToSync.end(); ++it ) {
    MailFolder *folder=ObjectRequestBroker::thisInstance()->mailfolderReference( vfs->map(*it) );
    if ( folder )
      folder->sync();
  }
  foldersToSync.clear();
}

void IOSyncHandler::imap4Sync()
{
  if(!IMAP4Handler::ref()->imapFolder())
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message discarded.");
  }
  else
    IMAP4Handler::ref()->imapFolder()->sync();
}

void IOSyncHandler::createFolder( const QString &parent, const QString &folderPath,
                                  const QString& type,   const QString &viewType )
{
  ObjectRequestBroker::thisInstance()->createFolder(parent, folderPath, type, viewType);
}

void IOSyncHandler::deleteFolder( const QString &parent, const QString &folderPath,
                                  const QString& type,   const QString &serverPath )
{
  ObjectRequestBroker::thisInstance()->deleteFolder(parent, folderPath, type, serverPath);
}

void IOSyncHandler::renameFolder( const QString &account, const QString &mailbox,
                                  const QString &newName, const QString &url )
{
  ObjectRequestBroker::thisInstance()->renameFolder(account, mailbox, newName, url);
}

void IOSyncHandler::slotMessageDelivered(IndexClass * idx)
{
  qDebug("IOSyncHandler::slotMessageDelivered");

  bool saveSent =
    Settings::instance()->readBoolEntry
    ("/Aethera/Send Options/Save sent");

	if (saveSent)
	{
    QCString currentDateAsString =
      DateClass(QDateTime::currentDateTime()).toQCString();

		// Replace info from the extended status.

		MessageDevice dev(idx);

		dev.loadDescriptor();

		MessageDescriptor & descriptor = dev.getDescriptor();

		descriptor.extendedStatus.clear();

		descriptor.extendedStatus.append("sent:" + currentDateAsString);
		
		descriptor.status   = "Sent";
		descriptor.sentDate = currentDateAsString;
		
		dev.saveDescriptor();

		// Move message to "Sent Mail", if needed.

		ObjectRequestBroker::thisInstance()
      ->move("sysmail_outbox/" + idx->getID(), "sysmail_sent");
	}
	else
	{
		// Delete message.
		ObjectRequestBroker::thisInstance()
      ->remove("sysmail_outbox/" + idx->getID());
	}
}

void IOSyncHandler::insertToDo(QString indexURL, QString folderName, WaitingOperations::ToDo action, QStringList param)
{
  if( indexURL.isEmpty() || 
      folderName.isEmpty() ||
      WaitingOperations::Nothing == action )
    return;
  
  WaitingOperations * waitingToDo = toDo[indexURL];
  if( !waitingToDo )
  {
    waitingToDo = new WaitingOperations(folderName, action, param);
    
    toDo.insert(indexURL, waitingToDo);
  }
}

void IOSyncHandler::checkToDo(const QString & indexURL)
{
#ifdef IMAP4_DEBUG
  qDebug("IOSyncHandler::checkToDo(%s)", indexURL.latin1());
#endif

  if( indexURL.isEmpty() )
    return;
  
  WaitingOperations * waitingToDo = toDo[indexURL];
  if( waitingToDo )
  {
    IndexClass * index = ObjectRequestBroker::thisInstance()->indexReference(indexURL);
    
    if(index)
    {
      MailFolder * mailFolder =
        ObjectRequestBroker::thisInstance()->mailfolderReference(waitingToDo->folderURL);
      
      if(mailFolder)
      {
        switch( waitingToDo->whatToDo )
        {
          case WaitingOperations::CopyObject :
            mailFolder->copyMessage(index);
            break;
          case WaitingOperations::MoveObject :
            mailFolder->moveMessage(index);
            break;
          case WaitingOperations::DeleteObject :
            mailFolder->deleteMessage(index);
            break;
          case WaitingOperations::Nothing :
          default:
            break;
        }
      }
      else
      {
        qDebug("checkToDo : bad folder");
      }
      
      toDo.remove(indexURL);
    }
    else
    {
      qDebug("checkToDo : bad index");
    }
  }
}

