/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#ifndef __IOSyncHandler__
#define __IOSyncHandler__

#include <qstringlist.h>
#include <vfsmodule.h>

class MailFilterV;
class IndexClass;
class IncomingMessage;
class FetchInfo;

class WaitingOperations
{
  public:
  
    enum ToDo { Nothing=0, CopyObject, MoveObject, DeleteObject };
    
    WaitingOperations(const QString & folderName, ToDo action, QStringList param);
    virtual ~WaitingOperations();
    
    QString folderURL;
    ToDo    whatToDo;
    QStringList parameters;
};


class IOSyncHandler : public QObject
{
  Q_OBJECT

  public:
    IOSyncHandler();
    static IOSyncHandler *ref();

    /** Dispatch the message to the local inbox folder.*/
    void dispatchToInbox
      (
        const QCString  & messageData,
        const QCString  & uidl,
        const QString   & accountName
      );

    /** Dispatch the message to the remote mailbox folder.*/
    void dispatchMessage(const FetchInfo&, bool);
    /** Update the IMAP messages.*/
    void updateMessage(const FetchInfo&, const QString&);
    /** Delete IMAP messages.*/
    void deleteMessage(const QString&);
    /** Delete IMAP messages.*/
    void deleteTrashMessages();
    /** Delete 'todelete' mail.*/
    void toDeleteMessage(const QString&);
    /**
      * It creates the messages from the list and delete the messages from trash.
      * It is a very useful and important method for the mail folders.
      */
    void pop3Sync();
    /**
      * It creates the messages from the list and delete the messages from trash.
      * It is a very useful and important method for the mail folders.
      */
    void imap4Sync();
    /** Create a new folder.*/
    void createFolder(const QString &parent, const QString &folderPath,
                      const QString& type,   const QString &viewType);
    /** Delete a folder.*/
    void deleteFolder(const QString &parent, const QString &folderPath,
                      const QString& type,   const QString &serverPath);
    void renameFolder( const QString &account, const QString &mailbox,
                       const QString &newName, const QString &url );
                      
    void insertToDo(QString, QString, WaitingOperations::ToDo, QStringList);
    void checkToDo(const QString&);

  public slots:

    void slotMessageDelivered(IndexClass *);

  private:
    static IOSyncHandler *thisInstance;
    QStringList foldersToSync;
    VFSModule *vfs;
    MailFilterV *mailFilter;
    QDict<WaitingOperations> toDo;
};

#endif






