/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#ifndef SMTP_CLIENT_H
#define SMTP_CLIENT_H

#include <qobject.h>
#include <qsocket.h>
#include <qptrqueue.h>
#include <qtimer.h>
#include <qstringlist.h>

#include <accounts.h>

// #define SMTP_DEBUG

class IndexClass;
class MessageDevice;

class SMTPClient : public QObject
{
  Q_OBJECT

  public:

    enum State
    {
      Idle=0,
      WaitingForConnection,
      WaitingForHeloResponse,
      WaitingForAuthResponse,
      WaitingForAuthChallenge,
      WaitingForMailFromResponse,
      WaitingForRcptToResponse,
      WaitingForDataBeginResponse,
      WaitingForDataEndResponse
    };

    SMTPClient(Account *, QObject * parent = 0, const char * name = 0);
    virtual ~SMTPClient();

    Account * account();

    void startSend(const QStringList&);
    void startSend(IndexClass *);

    static Account::AuthType authTypeStringToEnum(const QString&);
    static QString authTypeEnumToString(Account::AuthType type);

  protected slots:

    void slotHostFound();
    void slotConnected();
    void slotConnectionClosed();
    void slotReadyRead();
    void slotBytesWritten(int);
    void slotError(int);
    void slotIdle();

  protected:

    void startNextSend();
    void nextSend();
    void sendComplete();
    void resetIdleTimer();

    virtual void sendHelo();
    virtual void sendEhlo();
    virtual void sendAuth();
    virtual void sendMailFrom();
    virtual void sendRcptTo();
    virtual void sendDataBegin();
    virtual void sendData();
    virtual void sendQuit();
    
    virtual void handlerHelo(const QString &);
    virtual void handlerAuth(const QString &);
    virtual void handlerAuthChallenge(const QString &);
    virtual void handleLoginChallenge(const QString &);
    virtual void handlePlainChallenge(const QString &);
    virtual void handleCramMD5Challenge(const QString &);
    virtual void handleDigestMD5Challenge(const QString &);
    virtual void parseCapability(const QString &);

    virtual void writeLine(const QString &);
    virtual void setState(State);
    virtual QString stateToString(State) const;

  signals:

    void finished(SMTPClient *);
    void messageDelivered(IndexClass *);

  private:

    class Private;
    Private * d;
};

#endif // SMTP_CLIENT_H
// vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
