/***************************************************************************
                          generalconfigservice.cpp  -  description
                             -------------------
    begin                : Thu Oct 10 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/

#include "generalconfigservice.h"
#include "generalconfigprivate.h"
#include "generalconfigwidget.h"

#include <settings.h>
#include <qsettingsextra.h>
#include <qiconloader.h>

#include <qfont.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qlineedit.h>

static const char * const configGroup = "/Aethera/General/";

kore::Interface* generalconfigservice_factory()
{
  return new GeneralConfigService();
}

class GeneralConfigService::Private
{
  public:
    Private(GeneralConfigService*);
    ~Private();
    
    QFont font;
    
    GeneralConfigWidget * widget;
    GeneralConfigPrivate * qtHandler;
    GeneralConfigService * parent_;
};

GeneralConfigService::Private::Private(GeneralConfigService * parent)
  : widget(0L),
    qtHandler(0L),
    parent_(parent)
{
}

GeneralConfigService::Private::~Private()
{
  if( widget )
  {
    delete widget;
    widget = 0L;
  }
  
  if( qtHandler )
  {
    delete qtHandler;
    qtHandler = 0L;
  }
}

GeneralConfigService::GeneralConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "General Config Service");
  info->replaceAll("class", "GeneralConfigService");
  info->replaceAll("class/fqn", "GeneralConfigService");
  info->replaceAll("factory", "generalconfigservice_factory");
  
  d = new Private(this);
}

GeneralConfigService::~GeneralConfigService()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

QPixmap GeneralConfigService::icon()
{
  QPixmap p(32, 32);
  
  p.fill(Qt::blue);
  
  return p;
}

QString GeneralConfigService::caption()
{
  return QString("General");
}
  
void GeneralConfigService::load()
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }
  
  QString pattern("%1 - %2");
  QFont widgetFont(d->widget->font());
  
  bool useDefaultFont = Settings::instance()->readBoolEntry(configGroup + QString("UseDefaultFont"), true);
  d->widget->defaultFontCheckBox->setChecked( useDefaultFont );
  
  bool useDefaultLanguage = Settings::instance()->readBoolEntry(configGroup + QString("UseDefaultLanguage"), true);
  d->widget->defaultLanguageCheckBox->setChecked( useDefaultLanguage );
  
  QFont savedFont = QSettingsExtra::readFontEntry
  (Settings::instance(), configGroup + QString("Default font"), &widgetFont);
    
  d->widget->fontChooserButton->setText
    (pattern.arg(savedFont.family()).arg(savedFont.pointSize()));
    
  d->font = savedFont;
  
  int iStyle = 0;
  QIconLoader::Style style = QIconLoader::instance()->iconStyle();
  
  switch(style)
  {
    case QIconLoader::KDE_Default :
      iStyle = 1;
      break;
    case QIconLoader::QT_Default :
      iStyle = 2;
      break;
    case QIconLoader::Default :
    default:
      iStyle = 0;
      break;
  }
  
  d->widget->toolbarButtonGroup->setButton(iStyle);

  QString pluginFolder = Settings::instance()->readEntry( "/Aethera/PlugInManager/PlugInDirectory",
                                                          ""
                                                        );
  d->widget->pluginFolderLineEdit->setText( pluginFolder );

  d->widget->slotScanPluginsList();
}

void GeneralConfigService::save()
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }

  Settings::instance()->writeEntry(configGroup + QString("UseDefaultFont"),
    d->widget->defaultFontCheckBox->isChecked());
  Settings::instance()->writeEntry(configGroup + QString("UseDefaultLanguage"),
    d->widget->defaultLanguageCheckBox->isChecked());
  
  QString pattern("%1 - %2");
  QSettingsExtra::writeEntry( Settings::instance(), configGroup + QString("Default font"), d->font);
  
  int i = d->widget->toolbarButtonGroup->id( d->widget->toolbarButtonGroup->selected() );
  QIconLoader::Style style;
  
  switch( i )
  {
    case 1:
      style = QIconLoader::KDE_Default;
      break;
    case 2:
      style = QIconLoader::QT_Default;
      break;
    case 0:
    default:
      style = QIconLoader::Default;
      break;
  }
  
  QIconLoader::instance()->changeStyle(style);
  
  // Set Aethera app font
  if( !d->widget->defaultFontCheckBox->isChecked() )
    qApp->setFont(d->font, true);

  QStringList list = d->widget->loadedPlugins();
  if( !list.isEmpty() )
    Settings::instance()->writeEntry( "/Aethera/PlugInManager/LoadedPlugins", list, ',' );

  list = d->widget->allPlugins();
  if( !list.isEmpty() )
    Settings::instance()->writeEntry( "/Aethera/PlugInManager/PlugIns", list, ',' );

  QString folderPath = d->widget->pluginsFolder();
  if( !folderPath.isEmpty() )
    Settings::instance()->writeEntry( "/Aethera/PlugInManager/PlugInDirectory",
                                      d->widget->pluginFolderLineEdit->text() );
}

void GeneralConfigService::defaults()
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }

  d->widget->toolbarButtonGroup->setButton(0);

  d->widget->defaultFontCheckBox->setChecked( true );
  d->widget->defaultLanguageCheckBox->setChecked( true );

  //user selected plugin folder - none
  d->widget->pluginFolderLineEdit->setText( "" );
  
  //default plugins to load - all (the standard plugins because user plugins are not searched)
  d->widget->slotScanPluginsList();

  while( d->widget->availabilePluginsList->firstChild() )
  {
    QListViewItemIterator it( d->widget->availabilePluginsList );
    for( ; it.current(); ++it )
    {
      (*it)->setSelected( true );
      d->widget->slotAddPlugins();
    }
  }
}
  
bool GeneralConfigService::modified()
{
  return false;
}
  
QWidget * GeneralConfigService::init(QWidget * parent)
{
  if (0 == d->widget)
  {
    d->widget = new GeneralConfigWidget(parent, "GeneralConfigService widget");
  }
  
  QObject::connect( d->widget->fontChooserButton, SIGNAL(clicked()),
                    getQTHandler(), SLOT(slotChooseFont()) );
  
  return d->widget;
}

  void
GeneralConfigService::fontChosen(QFont f)
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }

  d->font = f;
  
  QString pattern("%1 - %2");
  d->widget->fontChooserButton->setText
    (pattern.arg(d->font.family()).arg(d->font.pointSize()));
}

  QFont
GeneralConfigService::currentFont()
{
  return d->font;
}

  QObject *
GeneralConfigService::createQTHandler()
{
  if (0 != d->qtHandler)
    return d->qtHandler;
  
  d->qtHandler = new GeneralConfigPrivate(this);
  
  return d->qtHandler;
}

  QObject *
GeneralConfigService::getQTHandler()
{
  createQTHandler();

  return d->qtHandler;
}

