//own includes
#include "generalconfigwidget.h"

//tkcBase includes
#include <qiconloader.h>
#include <settings.h>
#include <dirs.h>

//korelib includes
#include <kore/metainfo.h>
#include <kore/componentref.h>
#include <kore/librarylocator.h>
#include <kore/libraryloader.h>
#include <kore/componenttrader.h>
using namespace kore;

//QT includes
#include <qlistview.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qmessagebox.h>

//icon name defines
#define ADD_PLUGIN_BUTTON_ICON             "Right.png"
#define REMOVE_PLUGIN_BUTTON_ICON          "Left.png"
#define MOVE_PLUGIN_UP_BUTTON_ICON         "Up.png"
#define MOVE_PLUGIN_DOWN_BUTTON_ICON       "Down.png"

static const char* const open_data[] = { 
"16 16 7 1",
". c None",
"# c #000000",
"e c #a4a1a4",
"d c #c5c2c5",
"c c #dedede",
"a c #ffaa5a",
"b c #ffdeac",
"......####......",
".....##.####.#..",
"....#.....####..",
"...........###..",
"..........####..",
".####...........",
"#abba#######....",
"#bbbbaaaaaaa#...",
"#bb#############",
"#ba#ccccccccccc#",
"#b#cddddededee#.",
"#a#cdddededeee#.",
"##cdddededeee#..",
"##cddededeeee#..",
"#cddededeeee#...",
"#############..."};


class GeneralConfigWidget::Private
{
  public:
  
    Private(GeneralConfigWidget*);
    ~Private();
    
    QStringList allPlugins;
    
  private:
  
    GeneralConfigWidget * parent_;
};

GeneralConfigWidget::Private::Private(GeneralConfigWidget * parent)
  : parent_(parent)
{
  allPlugins = Settings::instance()->readListEntry("/Aethera/PlugInManager/PlugIns", ',');
}

GeneralConfigWidget::Private::~Private()
{
}

GeneralConfigWidget::GeneralConfigWidget( QWidget* parent, const char* name)
  : GeneralConfigBase( parent, name )
{
  d = new Private(this);
  
  QIconLoader* iconLoader = QIconLoader::instance();

  addPluginButton->setPixmap( iconLoader->loadIcon(ADD_PLUGIN_BUTTON_ICON) );
  removePluginButton->setPixmap( iconLoader->loadIcon(REMOVE_PLUGIN_BUTTON_ICON) );
  movePluginUpButton->setPixmap( iconLoader->loadIcon(MOVE_PLUGIN_UP_BUTTON_ICON) );
  movePluginDownButton->setPixmap( iconLoader->loadIcon(MOVE_PLUGIN_DOWN_BUTTON_ICON) );
  
  pluginFolderChooserButton->setPixmap( (const char **)open_data );

  loadedPluginsList->setSorting( -1 ); //no sorting
  availabilePluginsList->setSorting( -1 ); //no sorting

  //make the necessary connections
  connect( addPluginButton,
           SIGNAL( clicked() ),
           this,
           SLOT( slotAddPlugins() )
          );
  connect( removePluginButton,
           SIGNAL( clicked() ),
           this,
           SLOT( slotRemovePlugins() )
          );
  connect( pluginFolderChooserButton,
           SIGNAL( clicked() ),
           this,
           SLOT( slotSelectPluginsDir() )
         );
  connect( movePluginUpButton,
           SIGNAL( clicked() ),
           this,
           SLOT( slotMovePluginUp() )
         );
 connect( movePluginDownButton,
           SIGNAL( clicked() ),
           this,
           SLOT( slotMovePluginDown() )
         );
 connect( scanFolderButton,
          SIGNAL( clicked() ),
          this,
          SLOT( slotScanForPlugins() )
        );
}

GeneralConfigWidget::~GeneralConfigWidget()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

void GeneralConfigWidget::slotScanForPlugins()
{
  slotScanPluginsList();
  
  // TODO : Search for unknown plugins
#if 0
  //search the instalation directory AETHERA_DIR/plugins
  QString pluginDir( Dirs::locateDir("plugins") );

  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    LibraryLocator* locator = dynamic_cast<LibraryLocator*>(trader->registeredComponent("/kore/service/locator").instance());
    if( locator )
    {
      locator->scanDir( pluginDir.latin1(), false, LibraryLoader::GLOBAL );
      locator->scanDir( (pluginFolderLineEdit->text()).latin1(), true, LibraryLoader::GLOBAL );
    }
   
    ComponentRef* components =  trader->registeredComponents( "/Aethera/Service" );

    if( !components[0].isValid() )
       qDebug( "No valid componets found" );
     else
       for( unsigned int i=0; components[i].isValid(); ++i)
       {
         ComponentRef library = components[i];
         if( !(library.isValid()) )
         {
           qDebug( "Error[GeneralConfigWidget]: not a valid component" );
         }
         else
         {
           MetaInfo* libraryMetaInfo = library.metaInfo();
           if( libraryMetaInfo )
           {
             QString libraryName = libraryMetaInfo->find("library/name");

             //if the library name is not NULL enter it ONLY once in the availabilePluginList
             if( !(libraryName.isEmpty()) )
             {
               bool alreadyEntered = false;
               for(QStringList::Iterator it = d->allPlugins.begin(); it != d->allPlugins.end(); ++it )
               {
                 if( *it == libraryName )
                   alreadyEntered = true;
               }

               if( !alreadyEntered )
               {
                 // add it to all known plugins
                 d->allPlugins += libraryName;
                 
                 // add it to available
                 QListViewItem* plugin = new QListViewItem( availabilePluginsList );
                 plugin->setText(0, libraryName);
               }
             }
           }
         }
       }
    }
#endif
}

void GeneralConfigWidget::slotScanPluginsList()
{
  slotScanLoadedPlugins();

  availabilePluginsList->clear();
  for( QStringList::Iterator it = d->allPlugins.begin(); it != d->allPlugins.end(); ++it )
  {
    bool bFind = false;
    for( QListViewItemIterator it1(loadedPluginsList); it1.current(); ++it1)
    {
      if( (*it) == it1.current()->text(0) )
        bFind = true;
    }
    
    if( !bFind )
    {
      QListViewItem* plugin = new QListViewItem( availabilePluginsList );
      plugin->setText( 0, *it );
    }
  }
}


void GeneralConfigWidget::slotAddPlugins()
{
  QListViewItem* currentItem = availabilePluginsList->selectedItem();
  loadedPluginsList->insertItem( currentItem );
}

void GeneralConfigWidget::slotRemovePlugins()
{
  QListViewItem* currentItem = loadedPluginsList->selectedItem();
  availabilePluginsList->insertItem( currentItem );
}


void GeneralConfigWidget::slotMovePluginUp()
{
  //if there is an item above then swap the current selection and the item above
  if( loadedPluginsList->currentItem()->isSelected() )
  {
    if( loadedPluginsList->currentItem()->itemAbove() )
    {
      QString tmp = loadedPluginsList->currentItem()->itemAbove()->text( 0 );
      QString currentItemText = loadedPluginsList->currentItem()->text( 0 );

      loadedPluginsList->currentItem()->itemAbove()->setText( 0,  currentItemText);
      loadedPluginsList->currentItem()->setText( 0, tmp );

      loadedPluginsList->setSelected( loadedPluginsList->currentItem()->itemAbove(),
                                      true
                                    );
    }
  }
}

void GeneralConfigWidget::slotMovePluginDown()
{
 //if there is an item below then swap the current selection and the item below
 if( loadedPluginsList->currentItem()->isSelected() )
 {
   if( loadedPluginsList->currentItem()->itemBelow() )
   {
     QString tmp = loadedPluginsList->currentItem()->itemBelow()->text( 0 );
     QString currentItemText = loadedPluginsList->currentItem()->text( 0 );

     loadedPluginsList->currentItem()->itemBelow()->setText( 0,  currentItemText);
     loadedPluginsList->currentItem()->setText( 0, tmp );

     loadedPluginsList->setSelected( loadedPluginsList->currentItem()->itemBelow(),
                                     true
                                   );
   }
 }
}

void GeneralConfigWidget::slotSelectPluginsDir()
{
  //use old folder as starting point
  QString path = QFileDialog::getExistingDirectory( pluginFolderLineEdit->text() );

  pluginFolderLineEdit->setText( path );

  emit pluginFolderChanged();
}

QStringList GeneralConfigWidget::allPlugins()
{
  return d->allPlugins;
}

QStringList GeneralConfigWidget::loadedPlugins()
{
  QStringList list;
  
  for(QListViewItemIterator it(loadedPluginsList); it.current(); ++it)
    list += (*it)->text( 0 );
  
  return list;
}

QString GeneralConfigWidget::pluginsFolder()
{
  return pluginFolderLineEdit->text();
}

void GeneralConfigWidget::slotScanLoadedPlugins()
{
  /*
    we use this method to scan the loaded plugins only once - we have to do this because korelib can't
    unload a library once loaded
  */
  
  static bool firstTime = false;

  if( !firstTime )
  {
    QStringList loaded;
    
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef* components =  trader->registeredComponents( "/Aethera/Service" );

      if( !components[0].isValid() )
        qDebug( "No valid componets found" );
      else
        for( unsigned int i=0; components[i].isValid(); ++i)
        {
          ComponentRef library = components[i];
          if( !(library.isValid()) )
          {
            qDebug( "Error[GeneralConfigWidget]: not a valid component" );
          }
          else
          {
            MetaInfo* libraryMetaInfo = library.metaInfo();
            if( libraryMetaInfo )
            {
              QString libraryName = libraryMetaInfo->find("library/name");

              //if the library name is not NULL enter it ONLY once in the availabilePluginList

              if( !(libraryName.isEmpty()) )
              {
                bool alreadyEntered = false;
                for(QStringList::Iterator it = loaded.begin(); it != loaded.end(); ++it )
                {
                  if( *it == libraryName )
                    alreadyEntered = true;
                }

                if( !alreadyEntered )
                  loaded += libraryName;

              }
            }
          }
        }
    }
    
    loadedPluginsList->clear();
    for(QStringList::Iterator it = loaded.fromLast(); it != loaded.end(); --it )
    {
      QListViewItem* lvi = new QListViewItem( loadedPluginsList );
      lvi->setText( 0, *it );
    }
    firstTime = true;
  }
}



