//own includes
#include "komeventalarmservice.h"

//KOPlugin includes
#include <calendarobject.h>
#include <calendarlocal.h>
#include <icalformat.h>
#include <calendarlocal.h>
#include <event.h>

//tkcBase include
#include <qsettingsextra.h>
#include <settings.h>

//QT includes
#include <qptrlist.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdatetime.h>

//Aethera includes
#include <vfolder.h>

/*
---------------------------------------------------------------------------------------------------------------------
The KomEventAlarmService factory
---------------------------------------------------------------------------------------------------------------------
*/
kore::Interface* komeventalarmservice_factory()
{
  static KomEventAlarmService* instance = 0L;
  if( !instance )
  {
    instance = new KomEventAlarmService();
  }
  return instance;
}
 

/*
---------------------------------------------------------------------------------------------------------------------
The KomEventAlarmService class
---------------------------------------------------------------------------------------------------------------------
*/
KomEventAlarmService::KomEventAlarmService()
  : qtHandler( 0L ),
    vfs( 0L ),
    status( 0L ),
    timer( 0L ),
    interval( 60000 ),
    hasEvents( false ),
    useAlarm( true )
{
  MetaInfo *info = metaInfo();
  info->replaceAll("name", "Alarm Demon");
  info->replaceAll("class", "KomEventAlarmService");
  info->replaceAll("class/fqn", "KomEventAlarmService");
  info->replaceAll("factory", "komeventalarmservice_factory");

  //obtain status bar for event falshing
  status = AppStatus::instance();

  timer = new QTimer(0L, "timer");

  //obtain vfs
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  assert(0 != vfs);

  qtHandler = new KomEventAlarmHandler( this );
  assert( 0 != qtHandler );                                                    
  
  QObject::connect( timer, SIGNAL( timeout() ),                               
                    qtHandler, SLOT( slotCheckEvents() )
                  );

  //see wheather we should start the alarmD
  useAlarm = Settings::instance()->readBoolEntry( "/Aethera/AlarmDemon/Use", useAlarm );
}


KomEventAlarmService::~KomEventAlarmService()
{
  if(qtHandler)
  {
    delete qtHandler;
    qtHandler=0L;
  }
}


void KomEventAlarmService::start()
{
  timer->start( interval );
}


void KomEventAlarmService::stop()
{
  timer->stop();
}


void KomEventAlarmService::slotUseAlarm(bool use)
{
  if( !use )
  {
    useAlarm = false;
    stop();
    saveSettings();
  }
  else
  {
    useAlarm = true;
    start();
    saveSettings();
  }
}


void KomEventAlarmService::saveSettings()
{
  Settings::instance()->writeEntry( "/Aethera/AlarmDemon/Use", useAlarm );
}


void KomEventAlarmService::slotCheckEvents()
{
  hasEvents = false;
  checkCalendars( "/Aethera/VFS/Datebook" );
  if( hasEvents )
  {
    //write something in the AppStatus
    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Warning, "New events in one of your calendars ");
  }
}


void KomEventAlarmService::checkCalendars( const QString& path )
{
  if( !hasEvents ) //only if we don't have events continue to check
  {
    VFolder * folder = vfs->folderRef( path );

    //check the calendars in the current folder
    QStringList urls = vfs->children( folder->path() );
    QStringList temp;
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      temp.push_back(*it);
    }

    vfs->createVInstances(temp, false);

    for(QStringList::Iterator it1=temp.begin(); it1!=temp.end(); ++it1)
    {
      CalendarObject* temp = 0L;
      temp = static_cast<CalendarObject*>( vfs->_fact_ref( path + '/' + (*it1), VFS_CALENDAR_MIMETYPE) );
      if( !temp )
      {
        qDebug("Bad CalendarObject reference");
      }
      KCal::CalendarLocal* calendar = new KCal::CalendarLocal( );
      KCal::ICalFormat ical;
      ical.fromString( calendar, temp->calendarInfo() );

      //check events in current calendar
      QPtrList<KCal::Event> events = calendar->events( currentLocalTime() );
      for( QPtrListIterator<KCal::Event> it2( events ); it2.current(); ++it2 )
      {
        KCal::Event* event = *it2;
        if( event->dtStart().secsTo( currentLocalTime() ) <= interval/1000 )
        {
          hasEvents = true;
          break;
        }
      }
      delete calendar;
      calendar = 0L;
    }

    temp.clear();
    folder->freeObjects();

    //in recursive manner check the calendars in all the subfolders
    VFolderList& subFolders = folder->folderList();
    for(VFolderListIterator it(subFolders); it.current(); ++it)
    {
      VFolder* subFolder = *it;
      checkCalendars( subFolder->path() );
    }
  }
}

QDateTime KomEventAlarmService::currentLocalTime()
{
  /*
    If the difference betwean the local time and the UTC time is smaller than 4 secs then the user
    is using UTC time. Compare both ways because we don't know which is time zone. 4 secs should be
    resonable tolerance betwean function calls.
  */
  int iTemp =  QDateTime::currentDateTime(Qt::LocalTime).secsTo( QDateTime::currentDateTime(Qt::UTC) );
  if( -4 <= iTemp && iTemp <= 4 )
  {
    return QDateTime::currentDateTime(Qt::LocalTime);
  }
  else //else have to apply correction to the local time to convert to UTC time
  {
    return QDateTime::currentDateTime(Qt::LocalTime).addSecs( iTemp );
  }

  //this way we signal that something has gone wrong
  return QDateTime();
}

