
#include "komkotreeitem.h"

#include <aethera/vfolder.h>
#include <aethera/vfolderattribute.h>
#include <aethera/vfsmodule.h>
#include <aethera/vfsmimetype.h>

#include <qpainter.h>

KomKOTreeItem::KomKOTreeItem( QListView * parent, KomItemType& treeItemType )
  : KomTreeItem(parent, treeItemType),
    folder(0L),
    folderAttribute(0L),
    bUpdate(true)
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qDebug("No VFS module ?");
    return;
  }
  
  QString rootFolder = vfs->root() + VFS_CALENDAR_ROOT;
  folder = vfs->folderRef( rootFolder );
  folderAttribute = static_cast<VFolderAttribute *>(vfs->_fact_ref( rootFolder + "default.fattr", "vfolderattribute"));
}

KomKOTreeItem::KomKOTreeItem( KomTreeItem* parent, KomItemType& treeItemType )
  : KomTreeItem(parent, treeItemType),
    folder(0L),
    folderAttribute(0L),
    bUpdate(true)
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString parentPath = static_cast<KomKOTreeItem*>(parent)->path();
  
//  qDebug("FolderPath = %s", (parentPath+"/"+text(0)).latin1());
  
  folder = vfs->folderRef( parentPath+"/"+text(0) );
  folderAttribute = static_cast<VFolderAttribute *>(vfs->_fact_ref( parentPath+"/"+text(0)+"/default.fattr", "vfolderattribute"));
}

KomKOTreeItem::~KomKOTreeItem()
{
}

void KomKOTreeItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align)
{
//  qDebug("KomKOTreeItem::paintCell");

  KomTreeItem::paintCell(p, cg, column, width, align);
}

QString KomKOTreeItem::path()
{
  if( folder )
    return folder->path();
  else
    return QString::null;
}

void KomKOTreeItem::forceUpdate()
{
  bUpdate = true;
}

