/***************************************************************************
  qsystemtrayicon.cpp
  -  System tray icon with popup menu.
                             -------------------
    begin                : Tue Oct  9 21:01:08 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qapplication.h>
#include <qpopupmenu.h>
#include <qsystemtrayicon.h>

#ifdef Q_WS_X11
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#endif // Q_WS_X11

class QSystemTrayIcon::Private
{
  public:

    QWidget     * managed;
    QPopupMenu  * menu;
    int           minimizeRestoreID;
#ifdef Q_WS_X11
    Atom          kde_net_wm_system_tray_window_for;
    Atom          net_active_window;

#endif // Q_WS_X11
};

QSystemTrayIcon::QSystemTrayIcon(QWidget * managed)
  : QLabel(0, "QSystemTrayIcon")
{

  d = new Private;
  d->managed            = managed;
  d->menu               = 0L;       // Delay creation until needed.
  d->minimizeRestoreID  = -1;
#ifdef Q_WS_X11
  setBackgroundMode(X11ParentRelative);

  // Intern atoms.

  d->kde_net_wm_system_tray_window_for =
    XInternAtom(qt_xdisplay(), "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False);

  // Do we need this one ?

  d->net_active_window =
    XInternAtom(qt_xdisplay(), "_NET_ACTIVE_WINDOW", False);

  // Set system tray property.

  WId windowWeAreIconFor = d->managed->topLevelWidget()->winId();

  XChangeProperty
    (
      qt_xdisplay(),
      winId(),
      d->kde_net_wm_system_tray_window_for,
      XA_WINDOW,
      32,
      PropModeReplace,
      (unsigned char *)(&windowWeAreIconFor),
      1
    );

  // KDE says 20x20. XXX Different for WIN32 ?

  setFixedSize(20, 20);
  resize(20, 20);
#endif // Q_WS_X11

  show();
}

QSystemTrayIcon::~QSystemTrayIcon()
{
  delete d;
}

  QPopupMenu *
QSystemTrayIcon::popupMenu()
{
  if (0 == d->menu)
  {
    d->menu = new QPopupMenu(this);
    
    d->menu->insertSeparator();

    // Save this one, because we might change it later.

    d->minimizeRestoreID =
      d->menu->insertItem
      (
        tr("Minimize"),
        this,
        SLOT(slotToggleMinimizeRestore())
      );

    d->menu->insertItem(tr("&Quit"), d->managed, SLOT(close()));
  }

  return d->menu;
}

  void
QSystemTrayIcon::mousePressEvent(QMouseEvent * e)
{
  switch (e->button())
  {
    case LeftButton:
      slotToggleMinimizeRestore();
      break;

    case MidButton:
    case RightButton:

      if (d->managed->isVisible())
      {
        popupMenu()->changeItem(d->minimizeRestoreID, tr("Minimize"));
      }
      else
      {
        popupMenu()->changeItem(d->minimizeRestoreID, tr("Restore"));
      }

      // Inform derived classes.
      menuAboutToShow(popupMenu());

      popupMenu()->popup(e->globalPos());

    default:
      break;
  }
}

  void
QSystemTrayIcon::enterEvent(QEvent *)
{
  // TODO ? Is this necessary ? Seems to be some code in KSystemTray
  // to handle a focus problem.
}

  void
QSystemTrayIcon::slotToggleMinimizeRestore()
{
  if (!d->managed->isVisible())
  {
    d->managed->show();
  }
  else
  {
    d->managed->hide();
  }
}

  void
QSystemTrayIcon::menuAboutToShow(QPopupMenu *)
{
  // Empty. Used by derived classes.
}

