/***************************************************************************
                   ssldevice.h  -  QSocketDevice with SSL support
                             -------------------
    begin                : Tue Apr 23 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#ifndef SSLDEVICE_H
#define SSLDEVICE_H

#include <qsocketdevice.h>
#include <qsocket.h>

#include <sslbase.h>

namespace tkcSSL
{
  class SSLDevicePrivate;
  class TKCSSL_API SSLDevice;
  
  /**Use QSocket with SSLDevice for having a SSL connection.
    *@author Eugen Constantinescu
    */
  class SSLDevice : public QSocketDevice
  {
    public:
    
      enum ConnectionType { NOSSL=0, SSL2, SSL3, SSL23, TLS };
      enum ConnectionState { CLOSE=0, INIT, CONNECTED };
    
      SSLDevice( Type type = Stream, ConnectionType conType = NOSSL);
      SSLDevice( int socket, Type type, ConnectionType conType = NOSSL);
      virtual ~SSLDevice();
      
      // from QSocketDevice
      virtual bool open( int mode );
      virtual void close();
      virtual bool connect( const QHostAddress &, Q_UINT16 );
      
#if QT_VERSION < 300
      virtual int  bytesAvailable() const;
      virtual int  readBlock( char *data, uint maxlen );
      virtual int  writeBlock( const char *data, uint len );
#else
      virtual Q_LONG  bytesAvailable() const;
      virtual Q_LONG  readBlock( char *data, Q_ULONG maxlen );
      virtual Q_LONG  writeBlock( const char *data, Q_ULONG len );
#endif
      
      // SSL related
      virtual bool initSSL();
      virtual void closeSSL();
      virtual bool connectSSL();
    
    private:
      SSLDevicePrivate * d;
  };
};

#endif

