/***************************************************************************
                          sslsocket.cpp  -  QSocket like class
                             -------------------
    begin                : Mon Apr 29 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/
/**
  Since there wasn't a way to use my own QSocketDevice with QSocket,
  I had to build my own QSocket like class with some changes.
  I have included the QT license (see below) because I am still using big 
  part of QSocket original code.
  I hope that it's temporarly and Trolltech will fix their code.
 */
 
/****************************************************************************
** $Id: sslsocket.cpp,v 1.10 2003/07/22 15:25:28 eug Exp $
**
** Implementation of QSocket class.
**
** Created : 970521
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of the network module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition licenses may use this
** file in accordance with the Qt Commercial License Agreement provided
** with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/qpl/ for QPL licensing information.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include "sslsocket.h"

#if QT_VERSION < 300
#include <qlist.h>
#define QPtrList QList
#define Q_ASSERT ASSERT
#else
#include <qptrlist.h>
#endif

#include <qtimer.h>
#include <qdns.h>
#include <qsocketdevice.h>

#include <string.h>
#ifndef NO_ERRNO_H
#include <errno.h>
#endif

// #define SOCKET_DEBUG

class SSLSocketPrivate {
public:
    SSLSocketPrivate();
   ~SSLSocketPrivate();
    void close();

    QSocket::State  state;// connection state
    QString		host;			// host name
    Q_UINT16		port;			// host port
    QSocketDevice       *socket;			// connection socket
    QSocketNotifier     *rsn, *wsn;		// socket notifiers
    QPtrList<QByteArray> rba, wba;		// list of read/write bufs
    QHostAddress	addr;			// connection address
#if QT_VERSION < 300
    int		rsize, wsize;		// read/write total buf size
    int		rindex, windex;		// read/write index
#else
    Q_ULONG		rsize, wsize;		// read/write total buf size
    Q_ULONG		rindex, windex;		// read/write index
#endif
    QDns	       *dns;
    // added by Eugen C.
    bool isSSL;
    static QPtrList<SSLSocket> ssl_sn_read_alreadyCalled; // used to avoid unwanted recursion
};

QPtrList<SSLSocket> SSLSocketPrivate::ssl_sn_read_alreadyCalled;

SSLSocketPrivate::SSLSocketPrivate()
    : state(QSocket::Idle), host(QString::fromLatin1("")), port(0),
      socket(0), rsn(0), wsn(0), rsize(0), wsize(0), rindex(0), windex(0),
      isSSL(false)
{
    dns = 0;
    rba.setAutoDelete( TRUE );
    wba.setAutoDelete( TRUE );
}

SSLSocketPrivate::~SSLSocketPrivate()
{
    close();
    delete socket;
    delete dns;
}

void SSLSocketPrivate::close()
{
    // Order is important here - the socket notifiers must go away
    // before the socket does, otherwise libc or the kernel will
    // become unhappy.
    delete rsn;
    rsn = 0;
    delete wsn;
    wsn = 0;
    socket->close();
    rsize = wsize = 0;
    rba.clear(); wba.clear();
    rindex = windex = 0;
    isSSL = false;
}


// ----- SSLSocket ------

SSLSocket::SSLSocket( QObject *parent, const char *name )
    : QSocket( parent, name )
{
    d = new SSLSocketPrivate;
    setSocketDevice( 0 );
    setFlags( IO_Direct );
    setStatus( IO_Ok );
}


SSLSocket::~SSLSocket()
{
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s): Destroy", name() );
#endif
    if ( state() != Idle )
	close();
    Q_ASSERT( d != 0 );
    delete d;
}


QSocketDevice *SSLSocket::socketDevice()
{
    return d->socket;
}

void SSLSocket::setSocketDevice( QSocketDevice *device , bool isSSL)
{
  d->isSSL = isSSL;
  
  if ( state() != Idle )
    close();
  if( d->socket )
    delete d->socket;

  if ( !device ) {
    
#ifdef SOCKET_DEBUG
    qDebug("setSocketDevice - new Device");
#endif
    
    device = new QSocketDevice( QSocketDevice::Stream );
    device->setBlocking( FALSE );
    device->setAddressReusable( TRUE );
  }
  d->socket = device;
}

void SSLSocket::setSocketDevice( QSocketDevice *device )
{
  d->isSSL = false;
  
  if ( state() != Idle )
    close();
  if( d->socket )
    delete d->socket;

  if ( !device ) {
    
#ifdef SOCKET_DEBUG
    qDebug("setSocketDevice - new Device");
#endif
    
    device = new QSocketDevice( QSocketDevice::Stream );
    device->setBlocking( FALSE );
    device->setAddressReusable( TRUE );
  }
  d->socket = device;
}

SSLSocket::State SSLSocket::state() const
{
    return d->state;
}


void SSLSocket::connectToHost( const QString &host, Q_UINT16 port )
{
    setSocketIntern( -1 );

    d->state = QSocket::HostLookup;
    d->host = host;
    d->port = port;
    d->dns = new QDns( host, QDns::A );
    // try if the address is already available (for faster connecting...)
    tryConnecting();
    if ( d->state == HostLookup ) {
      connect( d->dns, SIGNAL(resultsReady()), this, SLOT(tryConnecting()) );
    }
}

void SSLSocket::tryConnecting()
{
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s)::tryConnecting()", name() );
#endif
    
    // ### this ifdef isn't correct - addresses() also does /etc/hosts and
    static QValueList<QHostAddress> l;
    if ( d->state == HostLookup ) {
	// numeric-address-as-string handling.
	l = d->dns->addresses();
 
#ifdef SOCKET_DEBUG
  qDebug( "SSLSocket (%s)::tryConnecting: host %s, port %d, %d addresses",
    name(), d->host.ascii(), d->port, l.count() );
#endif

	if ( l.isEmpty() ) {
	    if ( !d->dns->isWorking() ) {
		d->state = Idle;
		emit error( ErrHostNotFound );
	    }
	    return;
	}
	emit hostFound();
	d->state = Connecting; // enter the next if clause
    }

    if ( d->state == Connecting ) {
	// ### hack: just use the first address
	if ( d->socket->connect( l[0], d->port ) == FALSE ) {
	    if ( d->socket->error() == QSocketDevice::NoError ) {
		if ( d->wsn )
		    d->wsn->setEnabled( TRUE );
		return; // not serious, try again later
	    }
	    d->state = Idle;
	    emit error( ErrConnectionRefused );
	    return;
	}
	// The socket write notifier will fire when the connection succeeds
	if ( d->wsn )
	    d->wsn->setEnabled( TRUE );
    }
}


bool SSLSocket::open( int m )
{
    if ( isOpen() ) {
#ifdef SOCKET_DEBUG
      qWarning( "SSLSocket::open: Already open" );
#endif
      return FALSE;
    }
    QIODevice::setMode( m & IO_ReadWrite );
    setState( IO_Open );
    return TRUE;
}


void SSLSocket::close()
{
    if ( !isOpen() || d->state == Idle )	// already closed
	return;
    if ( d->state == Closing )
	return;
    if ( !d->rsn || !d->wsn )
	return;
 
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s): close socket", name() );
#endif
    
    if ( d->socket && d->wsize ) {		// there's data to be written
	d->state = Closing;
	if ( d->rsn )
	    d->rsn->setEnabled( FALSE );
	if ( d->wsn )
	    d->wsn->setEnabled( TRUE );
	d->rba.clear();				// clear incoming data
	d->rindex = d->rsize = 0;
	return;
    }
    setFlags( IO_Sequential );
    setStatus( IO_Ok );
	d->close();
    d->state = Idle;
}


#if QT_VERSION < 300
bool SSLSocket::consumeReadBuf( int nbytes, char *sink )
#else
bool SSLSocket::consumeReadBuf( Q_ULONG nbytes, char *sink )
#endif
{
    if ( nbytes <= 0 || nbytes > d->rsize )
	return FALSE;
 
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s): consumeReadBuf %d bytes", name(), (int)nbytes );
#endif
    
    d->rsize -= nbytes;
    for ( ;; ) {
	QByteArray *a = d->rba.first();
	if ( d->rindex + nbytes >= a->size() ) {
	    // Here we skip the whole byte array and get the next later
	    int len = a->size() - d->rindex;
	    if ( sink ) {
		memcpy( sink, a->data()+d->rindex, len );
		sink += len;
	    }
	    nbytes -= len;
	    d->rba.remove();
	    d->rindex = 0;
	    if ( nbytes == 0 ) {		// nothing more to skip
		break;
	    }
	} else {
	    // Here we skip only a part of the first byte array
	    if ( sink )
		memcpy( sink, a->data()+d->rindex, nbytes );
	    d->rindex += nbytes;
	    break;
	}
    }
    return TRUE;
}


#if QT_VERSION < 300
bool SSLSocket::consumeWriteBuf( int nbytes )
#else
bool SSLSocket::consumeWriteBuf( Q_ULONG nbytes )
#endif
{
    if ( nbytes <= 0 || nbytes > d->wsize )
	return FALSE;
 
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s): skipWriteBuf %d bytes", name(), (int)nbytes );
#endif
    
    d->wsize -= nbytes;
    for ( ;; ) {
	QByteArray *a = d->wba.first();
	if ( d->windex + nbytes >= a->size() ) {
	    nbytes -= a->size() - d->windex;
	    d->wba.remove();
	    d->windex = 0;
	    if ( nbytes == 0 )
		break;
	} else {
	    d->windex += nbytes;
	    break;
	}
    }
    return TRUE;
}


bool SSLSocket::scanNewline( QByteArray *store )
{
    if ( d->rsize == 0 )
	return FALSE;
    int i = 0; // index into 'store'
    QByteArray *a = 0;
    char *p;
    int n;
    for ( ;; ) {
	if ( !a ) {
	    a = d->rba.first();
	    if ( !a || a->size() == 0 )
		return FALSE;
	    p = a->data() + d->rindex;
	    n = a->size() - d->rindex;
	} else {
	    a = d->rba.next();
	    if ( !a || a->size() == 0 )
		return FALSE;
	    p = a->data();
	    n = a->size();
	}
	if ( store ) {
	    while ( n-- > 0 ) {
		*(store->data()+i) = *p;
		if ( ++i == (int)store->size() )
		    store->resize( store->size() < 256
				   ? 1024 : store->size()*4 );
		switch ( *p ) {
		    case '\0':
      
#ifdef SOCKET_DEBUG
			qDebug( "SSLSocket (%s)::scanNewline: Oops, unexpected "
				"0-terminated text read <%s>",
				name(), store->data() );
#endif
    
			store->resize( i );
			return FALSE;
		    case '\n':
			*(store->data()+i) = '\0';
			store->resize( i );
			return TRUE;
		}
		p++;
	    }
	} else {
	    while ( n-- > 0 ) {
		switch ( *p++ ) {
		    case '\0':
			return FALSE;
		    case '\n':
			return TRUE;
		}
	    }
	}
    }
}


void SSLSocket::flush()
{
    bool osBufferFull = FALSE;
    int consumed = 0;
    while( !osBufferFull && d->state >= Connecting && d->wsize > 0 ) {

#ifdef SOCKET_DEBUG
	qDebug( "SSLSocket (%s): flush: Write data to the socket", name() );
#endif
 
	QByteArray *a = d->wba.first();
	int nwritten;
	int i = 0;
	if ( (int)a->size() - d->windex < 1460 ) {
	    // Concatenate many smaller blocks.  the first may be
	    // partial, but each subsequent block is copied entirely
	    // or not at all.  the sizes here are picked so that we
	    // generally won't trigger nagle's algorithm in the tcp
	    // implementation: we concatenate if we'd otherwise send
	    // less than PMTU bytes (we assume PMTU is 1460 bytes),
	    // and concatenate up to the largest payload TCP/IP can
	    // carry.  with these precautions, nagle's algorithm
	    // should apply only when really appropriate.
	    QByteArray out( 65536 );
	    int j = d->windex;
	    int s = a->size() - j;
	    while ( a && i+s < (int)out.size() ) {
		memcpy( out.data()+i, a->data()+j, s );
		j = 0;
		i += s;
		a = d->wba.next();
		s = a ? a->size() : 0;
	    }
	    nwritten = d->socket->writeBlock( out.data(), i );
	    if ( d->wsn )
		d->wsn->setEnabled( FALSE ); // the QSocketNotifier documentation says so
	} else {
	    // Big block, write it immediately
	    i = a->size() - d->windex;
	    nwritten = d->socket->writeBlock( a->data() + d->windex, i );
	    if ( d->wsn )
		d->wsn->setEnabled( FALSE ); // the QSocketNotifier documentation says so
	}
	if ( nwritten ) {
	    if ( SSLSocket::consumeWriteBuf( nwritten ) )
		consumed += nwritten;
	}
	if ( nwritten < i )
	    osBufferFull = TRUE;
    }
    if ( consumed > 0 ) {
#ifdef SOCKET_DEBUG
	qDebug( "SSLSocket (%s): flush: wrote %d bytes, %d left",
		name(), consumed, (int)d->wsize );
#endif
	emit bytesWritten( consumed );
    }
    if ( d->state == Closing && d->wsize == 0 ) {
#ifdef SOCKET_DEBUG
	qDebug( "SSLSocket (%s): flush: Delayed close done. Terminating.",
		name() );
#endif
	setFlags( IO_Sequential );
	setStatus( IO_Ok );
	d->close();
	d->state = Idle;
	emit delayedCloseFinished();
	return;
    }
    if ( d->wsn )
	d->wsn->setEnabled( d->wsize > 0 ); // write if there's data
}


#if QT_VERSION < 300
uint SSLSocket::size() const
#else
QIODevice::Offset SSLSocket::size() const
#endif
{
    return bytesAvailable();
}


#if QT_VERSION < 300
int SSLSocket::at() const
#else
QIODevice::Offset SSLSocket::at() const
#endif
{
    return 0;
}


#if QT_VERSION < 300
bool SSLSocket::at( int index )
#else
bool SSLSocket::at( Offset index )
#endif
{
    if ( index > d->rsize )
	return FALSE;
    SSLSocket::consumeReadBuf( (Q_ULONG)index, 0 );			// throw away data 0..index-1
    return TRUE;
}


bool SSLSocket::atEnd() const
{
    if ( d->socket == 0 )
	return TRUE;
    SSLSocket * that = (SSLSocket *)this;

    // added by Eugen C.
    int nbytes = d->isSSL ? ((tkcSSL::SSLDevice*)that->d->socket)->bytesAvailable()
                          : that->d->socket->bytesAvailable();

    if ( nbytes )	// a little slow, perhaps...
	that->sn_read();
    return that->d->rsize == 0;
}


#if QT_VERSION < 300
int SSLSocket::bytesAvailable() const
#else
Q_ULONG SSLSocket::bytesAvailable() const
#endif
{
    if ( d->socket == 0 )
	return 0;
    SSLSocket * that = (SSLSocket *)this;

    // added by Eugen C.
    int nbytes = d->isSSL ? ((tkcSSL::SSLDevice*)that->d->socket)->bytesAvailable()
                          : that->d->socket->bytesAvailable();

    if ( nbytes ) // a little slow, perhaps...
	(void)that->sn_read();
	
    return that->d->rsize;
}


#if QT_VERSION < 300
int SSLSocket::waitForMore( int msecs ) const
#else
Q_ULONG SSLSocket::waitForMore( int msecs ) const
#endif
{
    if ( d->socket == 0 )
	return 0;
    SSLSocket * that = (SSLSocket *)this;
    if ( that->d->socket->waitForMore( msecs ) > 0 )
	(void)that->sn_read( TRUE );
    return that->d->rsize;
}


#if QT_VERSION < 300
int SSLSocket::bytesToWrite() const
#else
Q_ULONG SSLSocket::bytesToWrite() const
#endif
{
    return d->wsize;
}

#if QT_VERSION < 300
int SSLSocket::readBlock( char *data, uint maxlen )
#else
Q_LONG SSLSocket::readBlock( char *data, Q_ULONG maxlen )
#endif
{
    if ( data == 0 && maxlen != 0 ) {
      qWarning( "SSLSocket::readBlock: Null pointer error" );
      return -1;
    }
    if ( !isOpen() ) {
      qWarning( "SSLSocket::readBlock: Socket is not open" );
      return -1;
    }
    if ( maxlen >= d->rsize )
      maxlen = d->rsize;
 
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s): readBlock %d bytes", name(), (int)maxlen );
#endif

    SSLSocket::consumeReadBuf( maxlen, data );
    return maxlen;
}

#if QT_VERSION < 300
int SSLSocket::writeBlock( const char *data, uint len )
#else
Q_LONG SSLSocket::writeBlock( const char *data, Q_ULONG len )
#endif
{
    if ( data == 0 && len != 0 ) {
      qWarning( "SSLSocket::writeBlock: Null pointer error" );
    }
    if ( !isOpen() ) {
      qWarning( "SSLSocket::writeBlock: Socket is not open" );
      return -1;
    }
    if ( d->state == Closing ) {
      qWarning( "SSLSocket::writeBlock: Cannot write, socket is closing" );
    }
    if ( len == 0 || d->state == Closing )
      return 0;
    
    QByteArray *a = d->wba.last();

    // next bit is sensitive.  if we're writing really small chunks,
    // try to buffer up since system calls are expensive, and nagle's
    // algorithm is even more expensive.  but if anything even
    // remotely large is being written, try to issue a write at once.

    bool writeNow = ( d->wsize + len >= 1400 || len > 512 );

    if ( a && a->size() + len < 128 ) {
	// small buffer, resize
	int i = a->size();
	a->resize( i+len );
	memcpy( a->data()+i, data, len );
    } else {
	// append new buffer
	a = new QByteArray( len );
	memcpy( a->data(), data, len );
	d->wba.append( a );
    }
    d->wsize += len;
    if ( writeNow )
	flush();
    else if ( d->wsn )
	d->wsn->setEnabled( TRUE );
#ifdef SOCKET_DEBUG
    qDebug( "SSLSocket (%s): writeBlock %d bytes", name(), (int)len );
#endif
    return len;
}


int SSLSocket::getch()
{
    if ( isOpen() && d->rsize > 0 ) {
	uchar c;
	SSLSocket::consumeReadBuf( 1, (char*)&c );
	return c;
    }
    return -1;
}


int SSLSocket::putch( int ch )
{
    char buf[2];
    buf[0] = ch;
    return writeBlock(buf, 1) == 1 ? ch : -1;
}


int SSLSocket::ungetch( int ch )
{
    if ( !isOpen() ) {
      qWarning( "SSLSocket::ungetch: Socket not open" );
      return -1;
    }

    if ( d->rba.isEmpty() || d->rindex==0 ) {
	// we need a new QByteArray
	QByteArray *ba = new QByteArray( 1 );
	d->rba.insert( 0, ba );
	d->rsize++;
	ba->at( 0 ) = ch;
    } else {
	// we can reuse a place in the buffer
	QByteArray *ba = d->rba.first();
	d->rindex--;
	d->rsize++;
	ba->at( d->rindex ) = ch;
    }
    return ch;
}


bool SSLSocket::canReadLine() const
{
    if ( ((SSLSocket*)this)->scanNewline( 0 ) )
	return TRUE;
    return ( bytesAvailable() > 0 &&
	     ((SSLSocket*)this)->scanNewline( 0 ) );
}

#if QT_VERSION < 300
int SSLSocket::readLine( char *data, uint maxlen )
#else
Q_LONG SSLSocket::readLine( char *data, Q_ULONG maxlen )
#endif
{
    return QIODevice::readLine(data,maxlen);
}

QString SSLSocket::readLine()
{
    QByteArray a(256);
    bool nl = scanNewline( &a );
    QString s;
    if ( nl ) {
	at( a.size() );				// skips the data read
	s = QString( a );
    }
    return s;
}

#if QT_VERSION < 300
void SSLSocket::sn_read()
{
  sn_read(false);
}
#endif

void SSLSocket::sn_read( bool force )
{
    // Use SSLSocketPrivate::sn_read_alreadyCalled to avoid recursive calls of
    // sn_read() (and as a result avoid emitting the readyRead() signal in a
    // slot for readyRead(), if you use bytesAvailable()).
    if ( !force && SSLSocketPrivate::ssl_sn_read_alreadyCalled.findRef(this) != -1 && d->rsize )
	return;
    SSLSocketPrivate::ssl_sn_read_alreadyCalled.append( this );

    char buf[4096];
    // added by Eugen C.
    int nbytes = d->isSSL ? ((tkcSSL::SSLDevice*)d->socket)->bytesAvailable()
                          : d->socket->bytesAvailable();

    int  nread = 0;
    QByteArray *a = 0;

    if ( state() == Connecting ) {
	if ( nbytes > 0 ) {
	    tryConnection();
	} else {
	    // nothing to do, nothing to care about
	    SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
	    return;
	}
    }
    if ( state() == Idle ) {
	SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
	return;
    }

    if ( nbytes <= 0 ) {			// connection closed?
	// On Windows this may happen when the connection is still open.
	// This happens when the system is heavily loaded and we have
	// read all the data on the socket before a new WSAsyncSelect
	// event is processed. A new read operation would then block.
	// This code is also useful when SSLSocket is used without an
	// event loop.
	nread = d->socket->readBlock( buf, sizeof(buf) );
#if defined(_OS_WIN32_)
	if ( nread < 0 ) {
	    switch ( WSAGetLastError() ) {
		case WSAECONNABORTED:		// connection lost for
		case WSAETIMEDOUT:		//   some reason
		case WSAECONNRESET:
		    nread = 0;
		    break;
	    }
	}
#endif
	if ( nread == 0 ) {			// really closed

#ifdef SOCKET_DEBUG
	    qDebug( "SSLSocket (%s): sn_read: Connection closed", name() );
#endif
		return;

	    // We keep the open state in case there's unread incoming data
	    d->state = Idle;
	    if ( d->rsn )
		d->rsn->setEnabled( FALSE );
	    if ( d->wsn )
		d->wsn->setEnabled( FALSE );
	    d->socket->close();
	    d->wba.clear();			// clear write buffer
	    d->windex = d->wsize = 0;
	    emit connectionClosed();
	    SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
	    return;
	} else {
	    if ( nread < 0 ) {
		if ( d->socket->error() == QSocketDevice::NoError ) {
		    // all is fine
		    SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
		    return;
		}

#ifdef SOCKET_DEBUG
		qWarning( "SSLSocket::sn_read (%s): Close error", name() );
#endif

		if (d->rsn)
		    d->rsn->setEnabled( FALSE );
		emit error( ErrSocketRead );
		SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
		return;
	    }
	    a = new QByteArray( nread );
	    memcpy( a->data(), buf, nread );
	}

    } else {					// data to be read

#ifdef SOCKET_DEBUG
	qDebug( "SSLSocket (%s): sn_read: %d incoming bytes", name(), nbytes );
#endif

	if ( nbytes > (int)sizeof(buf) ) {
	    // big
	    a = new QByteArray( nbytes );
	    if( a )
	      nread = d->socket->readBlock( a->data(), nbytes );
	} else {
	    a = 0;
	    nread = d->socket->readBlock( buf, sizeof(buf) );
	    if ( nread > 0 ) {
		// ##### could setRawData
		a = new QByteArray( nread );
		memcpy(a->data(),buf,nread);
	    }
	}
	if ( nread < 0 ) {
	    if ( d->socket->error() == QSocketDevice::NoError ) {
		// all is fine
		SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
		return;
	    }
	    qWarning( "SSLSocket::sn_read: Read error" );

	    delete a;
	    if (d->rsn)
		d->rsn->setEnabled( FALSE );
	    emit error( ErrSocketRead );
	    SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
	    return;
	}
	if ( a && nread != (int)a->size() ) {		// unexpected
#if defined(CHECK_RANGE) && !defined(Q_OS_WIN32)
	    qWarning( "SSLSocket::sn_read: Unexpected short read" );
#endif
	    a->resize( nread );
	}
    }

    d->rba.append( a );
    d->rsize += nread;
    
    if ( !force ) {
	if ( d->rsn )
	    d->rsn->setEnabled( FALSE );
	emit readyRead();
	if ( d->rsn )
	    d->rsn->setEnabled( TRUE );
    }

    SSLSocketPrivate::ssl_sn_read_alreadyCalled.removeRef( this );
}


void SSLSocket::sn_write()
{
    if ( d->state == Connecting )		// connection established?
	tryConnection();
    flush();
}

void SSLSocket::emitErrorConnectionRefused()
{
    emit error( ErrConnectionRefused );
}

void SSLSocket::tryConnection()
{
    if ( d->socket->connect( d->addr, d->port ) ) {
#if QT_VERSION < 300
	d->state = Connection;
#else
	d->state = Connected;
#endif
 
#ifdef SOCKET_DEBUG
	qDebug( "SSLSocket (%s): sn_write: Got connection to %s",
		name(), peerName().ascii() );
#endif
  
  // added by Eugen C.
  if( d->isSSL )
  {
    tkcSSL::SSLDevice * device = (tkcSSL::SSLDevice*)(socketDevice());
    device->connectSSL();
  }
  
	if ( d->rsn )
	    d->rsn->setEnabled( TRUE );
	emit connected();
    } else {
	d->state = Idle;
	QTimer::singleShot( 0, this, SLOT(emitErrorConnectionRefused()) );
	return;
    }
}

int SSLSocket::socket() const
{
    if ( d->socket == 0 )
	return -1;
    return d->socket->socket();
}

void SSLSocket::setSocket( int socket )
{
#ifdef SOCKET_DEBUG
  qDebug("SSLSocket::setSocket");
#endif
    setSocketIntern( socket );
    d->state = Connection;
    d->rsn->setEnabled( TRUE );
}


void SSLSocket::setSocketIntern( int socket )
{

#ifdef SOCKET_DEBUG
  qDebug("SSLSocket::setSocketIntern %d", socket);
#endif
  
  if ( state() != QSocket::Idle )
      close();

  if( socket>=0 && socket!=d->socket->socket() )
    d->socket->setSocket(socket, QSocketDevice::Stream);
  
  d->socket->setBlocking( FALSE );
  d->socket->setAddressReusable( TRUE );
  d->state = Idle;
  d->rsn = new QSocketNotifier( d->socket->socket(), QSocketNotifier::Read,
                                this, "read" );
  d->wsn = new QSocketNotifier( d->socket->socket(), QSocketNotifier::Write,
                                this, "write" );
  connect( d->rsn, SIGNAL(activated(int)), SLOT(sn_read()) );
  d->rsn->setEnabled( FALSE );
  connect( d->wsn, SIGNAL(activated(int)), SLOT(sn_write()) );
  d->wsn->setEnabled( FALSE );
  // Initialize the IO device flags
  setFlags( IO_Direct );
  setStatus( IO_Ok );
  open( IO_ReadWrite );

  // hm... this is not very nice.
  d->host = QString::null;
  d->port = 0;
  delete d->dns;
  d->dns = 0;
}


Q_UINT16 SSLSocket::port() const
{
    if ( d->socket == 0 )
	return 0;
    return d->socket->port();
}


Q_UINT16 SSLSocket::peerPort() const
{
    if ( d->socket == 0 )
	return 0;
    return d->socket->peerPort();
}


QHostAddress SSLSocket::address() const
{
    if ( d->socket == 0 ) {
	QHostAddress tmp;
	return tmp;
    }
    return d->socket->address();
}


QHostAddress SSLSocket::peerAddress() const
{
    if ( d->socket == 0 ) {
	QHostAddress tmp;
	return tmp;
    }
    return d->socket->peerAddress();
}


QString SSLSocket::peerName() const
{
    return d->host;
}

