/***************************************************************************
                          sslsocket.h  -  QSocket like class
                             -------------------
    begin                : Mon Apr 29 2002
    copyright            : (C) 2002 by Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/
/**
  Since there wasn't a way to use my own QSocketDevice with QSocket,
  I had to build my own QSocket like class with some changes.
  I have included the QT license (see below) because I am still using big 
  part of QSocket original code.
  I hope that it's temporarly and Trolltech will fix their code.
 */
 
/****************************************************************************
** $Id: sslsocket.h,v 1.6 2003/03/10 14:41:22 eug Exp $
**
** Implementation of QSocket class.
**
** Created : 970521
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of the network module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition licenses may use this
** file in accordance with the Qt Commercial License Agreement provided
** with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/qpl/ for QPL licensing information.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#ifndef SSLSOCKET_H
#define SSLSOCKET_H

#include "ssldevice.h"

#include <qsocket.h>

class SSLSocketPrivate;
class QSocketDevice;

#if QT_VERSION < 300
typedef unsigned long Q_ULONG;
#endif

/**It's a copy of QSocket but QSocketDevice is really working.
  *It is using SSLDevice instead of QSocketDevice.
  *@author Eugen Constantinescu
  */
class TKCSSL_API SSLSocket : public QSocket
{
  Q_OBJECT
    
  public:
    
    SSLSocket( QObject *parent=0, const char *name=0 );
    virtual ~SSLSocket();
    
    virtual State state() const;
    
    virtual int	 socket() const;
    virtual void setSocket( int );
    
    virtual QSocketDevice *socketDevice();
    virtual void setSocketDevice( QSocketDevice * );
    virtual void setSocketDevice( QSocketDevice * , bool );
    
    virtual void connectToHost( const QString &host, Q_UINT16 port );
    virtual QString	 peerName() const;
    
    // Implementation of QIODevice abstract virtual functions
    virtual bool	 open( int mode );
    virtual void	 close();
    virtual void	 flush();
    
#if QT_VERSION < 300
    virtual uint size() const;
    virtual int  at() const;
    virtual bool at( int );
    virtual bool atEnd() const;
    
    virtual int bytesAvailable() const;
    virtual int waitForMore( int msecs ) const;
    virtual int bytesToWrite() const;
    
    virtual int readBlock( char *data, uint maxlen );
    virtual int writeBlock( const char *data, uint len );
    virtual int readLine( char *data, uint maxlen );
#else
    virtual Offset	 size() const;
    virtual Offset	 at() const;
    virtual bool	 at( Offset );
    virtual bool	 atEnd() const;
    
    virtual Q_ULONG	 bytesAvailable() const;
    virtual Q_ULONG	 waitForMore( int msecs ) const;
    virtual Q_ULONG	 bytesToWrite() const;
    
    virtual Q_LONG	 readBlock( char *data, Q_ULONG maxlen );
    virtual Q_LONG	 writeBlock( const char *data, Q_ULONG len );
    virtual Q_LONG	 readLine( char *data, Q_ULONG maxlen );
#endif
    
    int		 getch();
    int		 putch( int );
    int		 ungetch(int);
    
    bool	 canReadLine() const;
    virtual	 QString readLine();
    
    Q_UINT16	 port() const;
    Q_UINT16	 peerPort() const;
    QHostAddress address() const;
    QHostAddress peerAddress() const;
    
  signals:
    void	 hostFound();
    void	 connected();
    void	 connectionClosed();
    void	 delayedCloseFinished();
    void	 readyRead();
    void	 bytesWritten( int nbytes );
    void	 error( int );
    
  protected slots:
#if QT_VERSION < 300
    virtual void sn_read();
    virtual void sn_read( bool force );
#else
    virtual void sn_read( bool force=FALSE );
#endif
    virtual void sn_write();
    
  private slots:
    void	tryConnecting();
    void	emitErrorConnectionRefused();
    
  private:
    SSLSocketPrivate *d;
    
#if QT_VERSION < 300
    virtual bool	 consumeReadBuf( int nbytes, char * );
    virtual bool	 consumeWriteBuf( int nbytes );
#else
    virtual bool	 consumeReadBuf( Q_ULONG nbytes, char * );
    virtual bool	 consumeWriteBuf( Q_ULONG nbytes );
#endif
    virtual bool	 scanNewline( QByteArray * = 0 );
    virtual void	 tryConnection();
    virtual void   setSocketIntern( int socket );
    
  private:	// Disabled copy constructor and operator=
      SSLSocket( const SSLSocket & );
      SSLSocket &operator=( const SSLSocket & );
};

#endif

